; title  "ALLMONL2"                          Copyright TAPR 1994
; modified from bst3.asm code
; Loops until reset manually.  Used to monitor Radio Port 2 
; Features AIO loopthru and named buffers for Port variables.
; includes FHP parameters for TA & TB to set 20 kHz sample rate.
; Updated to meet Port 07h limitations  23 Dec 93 lon cecil
; Modified to allow Radio Port definitions and active level setups
; set for PTT*/FD*/FU* where all control lines are active low.
; LED 2 is used to indicate active operation.
; Dedicated to copy audio from defined port to speaker output.
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
; 
; This document, and it executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
; 
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
; 
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
; 
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;
;       Auxillary register names needed for compiler variable list
AR0      .EQU    00h
AR1      .EQU    01h
AR2      .EQU    02h
AR3      .EQU    03h
AR4      .EQU    04h
AR5      .EQU    05h
AR6      .EQU    06h
AR7      .EQU    07h

;       Assigned memory mapped registers 
DRR      .EQU    00h        ; data receive register address.
DXR      .EQU    01h        ; data xmit register address.
TIM      .EQU    02h        ; Timer count register address.
PRD      .EQU    03h        ; TImer period register address.
IMR      .EQU    04h        ; Interrupt mask register address (5-0).
GREG     .EQU    05h        ; Global address control (7-0).
;       Block B0  0200h to 02FF  in Data Page 4-5 (512- 767 decimal)
;       Block B1  0300h to 03FF  in Data Page 6-7 (768-1023 decimal)
;       External  1000h to 7FFFh or 0FFFFh  in Data Page 8 to 511
;
;       Block  B2 for 60h to 7Fh in Data Page 0  (96-127 decimal)
;       Reserved on chip RAM as flags and storages.   
;       (block  B2 108 through 127.)
;
PIO      .EQU     07h       ; Latch Buffer
PIO_BUF  .EQU     60h        ; Value to Port 07h  (Radio Select/Gain)
TNC_BUF  .EQU     61h        ; Value to Port 06h  (TNC Output Latch)
PDC_BUF  .EQU     62h        ; Value to Port 0Bh  (Radio Digital Control)
BUFO     .EQU     65h        ; SIO output buffer
BUFI     .EQU     66h        ; SIO input buffer
DO       .EQU     67h        ; data output buffer
UARTO    .EQU     68h        ; UART output hold buffer
UARTI    .EQU     69h        ; UART input hold buffer
UARTW    .EQU     6Ah        ; UART word to send

; CMDA    .EQU    0A14h      ; load TA=5 for 15 us conversion
; CMDA    .EQU    0E1Ch      ; load TA=7 for 28 us conversion
; CMDA    .EQU    1020h      ; load TA=8 for 31 us conversion
; power-up default is 65 us conversion at 40 MHz CPU clock, TA=RA

CMDA     .EQU     0C18h      ; AIO load TA=6 for 24 us conversion   
CMDB     .EQU     1E3Eh      ; AIO load TB=15 decimal
CMDC     .EQU     0263h      ; AIO load configuration  Sync, no HPF 
;  Port 07h bit D15 for AIO *Enable, D14 for AIO *Reset

; TNC IO Locations          Input/Output Relative to DSP-93 CPU
SIO_RTS  .EQU     2         ; Input 
SIO      .EQU     3         ; Output
CD       .EQU     5         ; Input Carrier Detect
CONN_ST  .EQU     4         ; Input Connection status
CTS      .EQU     1         ; Output Clear To Send
X_CLK    .EQU     2         ; Output Xmit clock
R_CLK    .EQU     8         ; Input receive clock
TXD      .EQU     8         ; Output Xmit data
RXD      .EQU     10h       ; Input receive data
ROC_CLK  .EQU     20h       ; Recovered Clock from EPROMs in TNC

;  TASM does not assume the necessary 1000h starting address
	 .ORG     1000h

MODEM   
	 B       TINT
	 B       RINT
	 B       XINT
	 B       TRAP
	 B       GO

GO
	LDPK    00h            ; point to DATA PAGE 00 as default for dsp
			       ; except for port 7 must use page 8, for U210
	ZAC                    ; zero accumulator to assure 0000h loaded
	SACL    PIO_BUF        
	OUT     PIO_BUF,06h    ; write to TNC Output latch
	SACL    PDC_BUF
	OUT     PDC_BUF,0Bh    ; write to Radio Port Digital latch
	LDPK    08h
			       ; default REC 1 is LALK 8025h default
			       ; REC 2 uses LALK 8027h
	LALK    8027h          ; Reset D15 to DISCONNECT AIO (DX&DR)
			       ; Gain step 4h  on REC 1 input
	SACL    PIO_BUF        ; Clear D14 to hold AIO in RESET mode
	OUT     PIO_BUF,07h    ; write to AIO Enable/Gain & Radio Select
	LDPK    00h            

	ZAC
	LALK    0000h           ; preset port control to PTT*/FD*/FU*
	SACL    PDC_BUF
	OUT     PDC_BUF,0Bh

;       LDPK    08h            ; FOR U210 ACCESS LIMIT
;       LALK    0067h          ; Use only pages above DATA PAGE 7 during 
;       SACL    PIO_BUF        ; access to port 07h
;       OUT     PIO_BUF,07h
	LDPK    00h

			       ;  AIO RESET
LOOP2
	LDPK    08h            ; Data page is 0h
	LALK    0067h          ; Reset D15 to enable DR & DX to CPU
	SACL    PIO_BUF        ; and light L207
	OUT     PIO_BUF,07h 
	CALL    WAIT4
	LALK    4027h          ; Set D14 to end AIO reset pulse
	SACL    PIO_BUF        
	OUT     PIO_BUF,07h
	CALL    WAIT4

; AIO ENABLE
	LDPK    00h      
	ZAC
	SSXM
	SOVM
	SPM     00h
	SFSM  
	RTXM  
	LALK    0020h         ; Mask SIO TX interrupt active
	SACL    IMR
	ZAC
	SACL    BUFO          ; Assure output buffer is clear
	EINT
	IDLE
	LALK    0003h         ; load AIO command flag
	SACL    BUFO
	IDLE
	LALK    CMDC          ; load AIO CONFIGURATION command
	SACL    BUFO
	IDLE
	LALK    0003h         ;load AIO command flag
	SACL    BUFO
	IDLE
	LALK    CMDB          ; load TB REGISTER value (bigger than TA?)
	SACL    BUFO
	IDLE
	LALK    0003h         ; load AIO command flag 
	SACL    BUFO
	IDLE
	LALK    CMDA          ; load TA REGISTER value 
	SACL    BUFO
	IDLE
	DINT
	LALK    0030h          ; mask SIO RX and TX interrupts active
	SACL    IMR            ; store mask in IMR
	ZAC
	SACL    DO 
	EINT
	ZAC

LOOPPGM                        ; pointing to data page 0
	NOP                    ; loop here between interrupts
	B       LOOPPGM
	
	
LEDOFF  LDPK    08h
	LALK    4027h          ; dark LED 202 & LED 201
	SACL    PIO_BUF
	OUT     PIO_BUF,07h 
	LDPK    00h
	RET

G_ON    LDPK    08h
	LALK    4067h          ; LED 202 to indicate G operation
	SACL    PIO_BUF
	OUT     PIO_BUF,07h
	LDPK    00h
	RET

RINT    
	CALL    G_ON           ; Use LED 202 to indicate Receive interrupt
	LAC     DRR            ; and high speed operation
	SACL    BUFI
	CALL    DSP
	EINT
	RET

XINT    LAC     BUFO
	SACL    DXR
	EINT
	RET
	
DSP     LAC     BUFI
	ANDK    0FFFCh
	ROL                    ; Approx. unity gain
	SACL    BUFO
;  The DSP calculations driven by receive data expand in this section
;  until the entire period is consumed!

NDSP    
	CALL    LEDOFF
	ZAC
	RET

INBIT   LDPK    00h      ; Point to page 0 not 08h
	LALK    0500h
	SACL    UARTO
	OUT     UARTO,0Bh
	IN      UARTO,5    ; Get the Line CRTL on UART
	LALK    0001h      ; Set bit for receive hold register RHR
	AND     UARTO     ; Test the 0th bit Line CRTL
	BZ      INBIT
	ZAC
	SACL    UARTO
	OUT     UARTO,0Bh  ; Set up UART register 
	IN      UARTI,5    ; Get the data byte
	SACL    UARTW      ; Save into UART Word storage cell
	LDPK    00h
	RET

OUTBIT  LDPK    00h
	LALK    0500h        ; Set A0 and A1 high, A2 low
	SACL    UARTO
	OUT     UARTO,0Bh   ; Set up UART register 
	IN      UARTO,5     ; Get the Line CRTL on UART
	LACK    20h         ; Set bit for Transmit Hold Register THR
	AND     UARTO       ; Test the 6th bit Line CTRL
	BZ      OUTBIT      ; THR (not) Empty
	ZAC
	SACL    UARTO
	OUT     UARTO,0Bh   ;Set up UART register
	OUT     UARTW,1     ;Send the data byte
	LDPK    00h
	RET


TINT    RET

TRAP    RET


WAIT8   LARK    AR2,02h
	LARK    AR3,05h
	B       WAIT_B
	CALL    WAIT4
	CALL    WAIT4
	RET
WAIT4   LARK    AR2,0FFh
	CALL    WAIT2
	CALL    WAIT2
	RET
WAIT2   LARK    AR2,0FFh
	CALL    WAIT1
	CALL    WAIT1
	RET
WAIT1   LARK    AR2,0FFh
WAIT_A  LARK    AR3,0FFh
WAIT_B  LARP    AR3
	BANZ    WAIT_B    ; Loop through count
	LARP    AR2
	BANZ    WAIT_A    ; Loop through count
	RET

	.end              ; end of program

