;  Experimental FIR CW filter, 94.05.28 @ 09:37 CDT
;  Version for TAPR/AMSAT DSP-93, File - CW_93D.ASM
;  TLC32044 AIO @ 9,765.625 samples/sec (102.4 sec/sample), 40 MHz CPU clock
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Set up for cascade of two 100 tap FIRs
;  Uses radio port #1
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1991, 1992, 1993 
; and 1994, Frank H. Perkins, Jr., WB5IPM. All rights reserved.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact Greg Jones at TAPR or the author for information on
; commercial licenses of this software.
; 
; Frank H. Perkins, Jr.
;
AR0       .EQU      00h       ;  aux register 0 (equate used by TASM)
AR1       .EQU      01h       ;  aux register 1 (equate used by TASM)
AR2       .EQU      02h       ;  aux register 2 (equate used by TASM)
AR3       .EQU      03h       ;  aux register 3 (equate used by TASM)
AR4       .EQU      04h       ;  aux register 4 (equate used by TASM)
AR5       .EQU      05h       ;  aux register 5 (equate used by TASM)
AR6       .EQU      06h       ;  aux register 6 (equate used by TASM)
AR7       .EQU      07h       ;  aux register 7 (equate used by TASM)

DRR       .EQU      00h       ;  SIO (AIO) receive buffer
DXR       .EQU      01h       ;  SIO (AIO) transmit buffer
TIM       .EQU      02h       ;  TIMER countdown buffer
PRD       .EQU      03h       ;  TIMER period buffer
IMR       .EQU      04h       ;  INT mask buffer

BUFI      .EQU      060h      ;  AIO input buffer
BUFO      .EQU      061h      ;  AIO output buffer

DO        .EQU      062h      ;  data output buffer
SI        .EQU      063h      ;  data output buffer
SO        .EQU      064h      ;  sig output buffer 

CMDA      .EQU      02040h    ;  AIO load RA/TA = 16 for SCF @ 312.5 kHz
CMDB      .EQU      04082h    ;  AIO load RB/TB = 32 for 9,765.625 samples/sec 
CMDC      .EQU      0227h     ;  AIO configuration: HPF in, 3 V half scale

LEV       .EQU      1024      ;  level threshold

CFG       .EQU      02h       ;  AIO CFG buffer (data mem page 8)


          .ORG      1000h


          B         GO        ;  branch to program start (TINT)
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)


GO        DINT                ;  disable interrupts
          LDPK      08h       ;  data mem page 8 (external)

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       00h       ;  set P shift = 0

AIO_RST   LALK      8025h     ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      0065h     ;  load AIO reset, radio 1, gain -> 2
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LALK      4025h     ;  load AIO run, radio 1, gain -> 2
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LDPK      00h       ;  data mem page 0 (internal)

          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          SACL      IMR       ;  store INT mask

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts
          LALK      0030h     ;  mask SIO RX and TX interrupts active
          SACL      IMR       ;  store mask in IMR

DSP_INI   ZAC                 ;  zero A

          SACL      DO        ;  zero data out buffer
          SACL      SI        ;  zero data out buffer
          SACL      SO        ;  zero sig buffer

          EINT                ;  enable interrupts             


LOOP      NOP
          B         LOOP      ;  loop here between interrupts


RINT      LAC       DRR       ;  get received AIO data sample from DRR
          SACL      BUFI      ;  store DDR in AIO input buffer
          CALL      DSP       ;  call DSP process

          EINT
          RET


XINT      LAC       BUFO      ;  get data from AIO output buffer
          SACL      DXR       ;  store data in DXR to transmit to AIO

          EINT
          RET

DSP       LAC       DO        ;  load DO
          ORK       0400h     ;  LED 207 is duty cycle
          SACL      DO        ;  store DO
          OUT       DO,06h    ;  OUT to TNC

          LAC       BUFI,0Dh  ;  3 V nominal signal input
          SACH      SI        ;  13 bits in SI

LEVEL     LAC       SI        ;  get SI
          SBLK      LEV       ;  subtract LEV

          BLZ       L_LO      ;  if < 0 goto L_LO

L_HI      LAC       DO        ;  load DO
          ORK       0200h     ;  OR to set level LED
          SACL      DO        ;  store DO

          B         BP_FIR1   ;  goto BP_FIR

L_LO      LAC       DO        ;  load DO
          ANDK      0FDFFh    ;  AND to reset level LED
          SACL      DO        ;  store DO

BP_FIR1   LAC       SI        ;  get SI
          LRLK      AR2,039Bh ;  039Bh is start of delay line
          LARP      AR2       ;  AR2 pointer to 039Bh
          SACL      *         ;  store SI 

          LRLK     AR1,03FEh  ;  03FEh is end of delay line 
          LARP     AR1        ;  AR1 pointer to 03FEh  

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     063h       ;  repeat (99 + 1) times to do FIR
          MACD     BPF,*-     ;  MACD up thru FIR coeffs, down thru data
          APAC                ;  accum last product
          SACH     SO,03h     ;  store hi, 2^13 * 2^3 * 2^-16 norm

BP_FIR2   LAC       SO,01h    ;  get SO with 2^1 gain
          LRLK      AR2,0336h ;  0336h is start of delay line
          LARP      AR2       ;  AR2 pointer to 0336h
          SACL      *         ;  store SI 

          LRLK     AR1,0399h  ;  0399h is end of 2nd delay line 
          LARP     AR1        ;  AR1 pointer to 0399h  

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     063h       ;  repeat (99 + 1) times to do FIR
          MACD     BPF,*-     ;  MACD up thru FIR coeffs, down thru data
          APAC                ;  accum last product
          SACH     SO,03h     ;  store hi, 2^13 * 2^3 * 2^-16 norm

TX_OUT    LAC       SO,2      ;  get sine value, shift 2^2
          ANDK      0FFFCh    ;  mask out AIO control bits
          SACL      BUFO      ;  AIO out

END_DSP   LAC       DO        ;  load DO
          ANDK      0FBFFh    ;  reset duty cycle
          SACL      DO        ;  store DO
          OUT       DO,06h    ;  output to TNC port 06h 

          RET                 ;  done!!


WAIT4     LARK      AR2,0FFh
          CALL      WAIT2
          CALL      WAIT2

          RET


WAIT2     LARK      AR2,0FFh
          CALL      WAIT1
          CALL      WAIT1

          RET


WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ; Loop through count

          LARP      AR2
          BANZ      WAIT_A    ; Loop through count

          RET


          .MSFIRST

; 100 FIR coefficients, * 2^13, CW BPF

BPF       .WORD        -0
          .WORD        -0
          .WORD        -0
          .WORD        -0
          .WORD        -1
          .WORD        -1
          .WORD        -0
          .WORD         1
          .WORD         2
          .WORD         4
          .WORD         5
          .WORD         5
          .WORD         3
          .WORD        -1
          .WORD        -6
          .WORD       -12
          .WORD       -16
          .WORD       -18
          .WORD       -14
          .WORD        -6
          .WORD         7
          .WORD        21
          .WORD        34
          .WORD        42
          .WORD        40
          .WORD        27
          .WORD         4
          .WORD       -24
          .WORD       -52
          .WORD       -72
          .WORD       -78
          .WORD       -66
          .WORD       -35
          .WORD         8
          .WORD        55
          .WORD        95
          .WORD       117
          .WORD       114
          .WORD        83
          .WORD        31
          .WORD       -32
          .WORD       -93
          .WORD      -137
          .WORD      -151
          .WORD      -132
          .WORD       -83
          .WORD       -13
          .WORD        62
          .WORD       124
          .WORD       159
          .WORD       159
          .WORD       124
          .WORD        62
          .WORD       -13
          .WORD       -83
          .WORD      -132
          .WORD      -151
          .WORD      -137
          .WORD       -93
          .WORD       -32
          .WORD        31
          .WORD        83
          .WORD       114
          .WORD       117
          .WORD        95
          .WORD        55
          .WORD         8
          .WORD       -35
          .WORD       -66
          .WORD       -78
          .WORD       -72
          .WORD       -52
          .WORD       -24
          .WORD         4
          .WORD        27
          .WORD        40
          .WORD        42
          .WORD        34
          .WORD        21
          .WORD         7
          .WORD        -6
          .WORD       -14
          .WORD       -18
          .WORD       -16
          .WORD       -12
          .WORD        -6
          .WORD        -1
          .WORD         3
          .WORD         5
          .WORD         5
          .WORD         4
          .WORD         2
          .WORD         1
          .WORD        -0
          .WORD        -1
          .WORD        -1
          .WORD        -0
          .WORD        -0
          .WORD        -0
          .WORD        -0

          .END
