;  Experimental 300 bps HF modem, 94.06.12 @ 12:05 CDT
;  Version for TAPR/AMSAT DSP-93, File - HF_93A.ASM
;  TLC32044 AIO @ 10,822.51 samples/sec (92.4 sec/sample), 40 MHz CPU clock
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Uses radio port #1, TNC disconnect header (set TNC to 300 bps!!)
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1991, 1992, 1993 
; and 1994, Frank H. Perkins, Jr., WB5IPM. All rights reserved.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact Greg Jones at TAPR or the author for information on
; commercial licenses of this software.
; 
; Frank H. Perkins, Jr.
;
AR0       .EQU      00h       ;  aux register 0 (equate used by TASM)
AR1       .EQU      01h       ;  aux register 1 (equate used by TASM)
AR2       .EQU      02h       ;  aux register 2 (equate used by TASM)
AR3       .EQU      03h       ;  aux register 3 (equate used by TASM)
AR4       .EQU      04h       ;  aux register 4 (equate used by TASM)
AR5       .EQU      05h       ;  aux register 5 (equate used by TASM)
AR6       .EQU      06h       ;  aux register 6 (equate used by TASM)
AR7       .EQU      07h       ;  aux register 7 (equate used by TASM)

DRR       .EQU      00h       ;  SIO (AIO) receive buffer
DXR       .EQU      01h       ;  SIO (AIO) transmit buffer
TIM       .EQU      02h       ;  TIMER countdown buffer
PRD       .EQU      03h       ;  TIMER period buffer
IMR       .EQU      04h       ;  INT mask buffer

BUFI      .EQU      060h      ;  AIO input buffer
BUFO      .EQU      061h      ;  AIO output buffer

DO        .EQU      062h      ;  data output buffer
SO        .EQU      063h      ;  sig buffer 

RB        .EQU      064h      ;  ramp buffer
TS        .EQU      065h      ;  sine buffer
DI        .EQU      066h      ;  TX data in buffer
DC        .EQU      067h      ;  DCD count down buffer
RO        .EQU      068h      ;  radio control buffer (PTT)
TXE       .EQU      069h      ;  TX enable counter buffer

CMDA      .EQU      02A54h    ;  AIO load RA/TA = 21 for SCF @ 238.095 kHz
CMDB      .EQU      02C5Ah    ;  AIO load RB/TB = 22 for 10,822.51 samples/sec
CMDC      .EQU      0227h     ;  AIO configuration: HPF in, 3 V half scale

TOP       .EQU      04000h    ;  ramp top value
INCL      .EQU      0C91h     ;  incr value for 2125 Hz
INCH      .EQU      0D92h     ;  incr value for 2295 Hz

DCDC      .EQU      02C9h     ;  50 ms count for DCD
LEV       .EQU      128       ;  level threshold
TXEC      .EQU      400       ;  TX enable count

CFG       .EQU      02h       ;  AIO CFG buffer (data mem page 8)


          .ORG      1000h


          B         GO        ;  branch to program start (TINT)
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)


GO        DINT                ;  disable interrupts
          LDPK      08h       ;  data mem page 8 (external)

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       00h       ;  set P shift = 0

AIO_RST   LALK      8035h     ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      0075h     ;  load AIO reset, radio 1, gain -> 50
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LALK      4035h     ;  load AIO run, radio 1, gain -> 50
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LDPK      00h       ;  data mem page 0 (internal)

          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          SACL      IMR       ;  store INT mask

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts
          LALK      0030h     ;  mask SIO RX and TX interrupts active
          SACL      IMR       ;  store mask in IMR

DSP_INI   ZAC                 ;  zero A
          SACL      DO        ;  zero data out buffer
          SACL      SO        ;  zero sig buffer

          SACL      RB        ;  zero ramp buffer
          SACL      TS        ;  zero sine buffer
          SACL      DI        ;  zero TX data in buffer
          SACL      DC        ;  zero DC buffer

          LALK      TXEC      ;  get TX enable count
          SACL      TXE       ;  store in TX enable buffer

          ZAC                 ;  zero A

DL_INI    LRLK      AR0,039Fh ;  039Fh is BPF DL input 
          LARP      AR0       ;  make AR0 @ 039Fh
          ZAC                 ;  zero A
          RPTK      05Fh      ;  repeat (95 + 1) times:
          SACL      *+        ;  store 0, increment pointer

          LRLK      AR0,033Eh ;  033Eh is LPF DL input 
          LARP      AR0       ;  make AR0 @ 03EEh
          ZAC                 ;  zero A
          RPTK      05Fh      ;  repeat (95 + 1) times:
          SACL      *+        ;  store 0, increment pointer

          LRLK      AR0,0332h ;  0332h is Discm DL input 
          LARP      AR0       ;  make AR0 @ 03EEh
          ZAC                 ;  zero A
          RPTK      0Ah       ;  repeat (10 + 1) times:
          SACL      *+        ;  store 0, increment pointer

          LRLK      AR3,033Ch ;  AR3 pointer @ X^-9

          EINT                ;  enable interrupts             


LOOP      NOP
          B         LOOP      ;  loop here between interrupts


RINT      LAC       DRR       ;  get received AIO data sample from DRR
          SACL      BUFI      ;  store DDR in AIO input buffer
          CALL      DSP       ;  call DSP process

          EINT
          RET


XINT      LAC       BUFO      ;  get data from AIO output buffer
          SACL      DXR       ;  store data in DXR to transmit to AIO

          EINT
          RET

DSP       LAC       DO        ;  load DO
          ORK       0400h     ;  LED 207 is duty cycle
          SACL      DO        ;  store DO
          OUT       DO,06h    ;  OUT to TNC

          LAC       BUFI,0Ah  ;  3 V nominal signal input
          SACH      SO        ;  7 to 9 bits in SO

LEVEL     LAC       SO        ;  get SO
          SBLK      LEV       ;  subtract LEV

          BLZ       L_LO      ;  if < 0 goto L_LO

L_HI      LAC       DO        ;  load DO
          ORK       0200h     ;  OR to set level LED
          SACL      DO        ;  store DO

          B         BP_FIR    ;  goto BP_FIR

L_LO      LAC       DO        ;  load DO
          ANDK      0FDFFh    ;  AND to reset level LED
          SACL      DO        ;  store DO

BP_FIR    LAC       SO        ;  get SO
          LRLK      AR2,039Fh ;  039Fh is delay line input
          LARP      AR2       ;  AR2 pointer to input
          SACL      *         ;  store SO 

          LRLK     AR1,03FEh  ;  03FEh is end of delay line 
          LARP     AR1        ;  AR1 pointer to end  

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     05Fh       ;  repeat (95 + 1) times to do FIR
          MACD     BPF,*-     ;  MACD up thru BPF coeffs, down thru data
          APAC                ;  accum last product
          SACH     SO,03h     ;  store hi, 2^13 * 2^3 * 2^-16 norm

CLIP_H    LAC      SO         ;  load SO
          SBLK     72         ;  subtract 72

          BLZ      CLIP_LO    ;  SO < 72, goto CLIP_LO

          LALK     72         ;  else limit to +72
          SACL     SO         ;  and store SO

          B        DL_DISCM   ;  goto DL_DISCM

CLIP_LO   LAC      SO         ;  load SO
          ADLK     72         ;  add 72

          BGZ      DL_DISCM   ;  SO > -72, goto DL_DISCM

          LALK     -72        ;  else limit to -72
          SACL     SO         ;  and store in SO

DL_DISCM  LRLK      AR0,033Bh ;  load AR0 with location of X^-10
          LARP      AR0       ;  make AR0 pointer 

          RPTK      09h       ;  repeat (9 + 1) times:
          DMOV      *-        ;  bump data up, pointer down

          DMOV      *         ;  bump last input up, AR0 @ X^0
          LAC       SO        ;  load data
          SACL      *         ;  store in X^0
          NEG                 ;  SO * -1
          SACL      SO        ;  store in SO

          LARP      AR3       ;  AR3 pointer @ X^-11
          LT        *         ;  copy X^-11 into T
          MPY       SO        ;  X^-11 * SO into P
          PAC                 ;  P into A (16 bits max @ 100 Hz)

LP_FIR    LRLK      AR2,033Eh ;  033Eh is start of delay line
          LARP      AR2       ;  AR2 pointer to start of delay line
          SACL      *         ;  store discm product in SO 

          LRLK     AR1,039Dh  ;  039Dh is end of data delay line 
          LARP     AR1        ;  AR1 pointer to data delay line start

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     05Fh       ;  repeat (95 + 1) times to do FIR
          MACD     LPF,*-     ;  MACD, up thru LPF coeffs, down thru data
          APAC                ;  accum last product
          SACH     SO,03h     ;  store A hi -> 2^13 * 2^3 * 2^-16 norm

T_DCD     LAC      SO         ;  get SO
          SBLK     2000       ;  subtract 2000 

          BLZ      DO_TN      ;  SO < 2000, goto DO_TN

          LALK      DCDC      ;  get DCD count
          SACL      DC        ;  back into buffer

DO_TN     CALL        TUNE    ;  show tuning      

          LAC       SO        ;  load SO

          BGEZ      HI        ;  if => 0 goto HI

LO        LAC       DO        ;  else get DO
          ANDK      0BFFFh    ;  AND DATA OUT to 0
          SACL      DO        ;  store in data out

          B         TX_ONOFF  ;  branch to TX_ONOFF

HI        LAC       DO        ;  get DO
          ORK       4000h     ;  OR DATA OUT to 1
          SACL      DO        ;  store in data out

TX_ONOFF  LAC       TXE       ;  get count in TXE

          BLZ       TX_OFF    ;  if < 0 goto TX_OFF

          IN        DI,0Ah    ;  get TNC input port
          LAC       DI        ;  load data
          ANDK      0200h     ;  strip garbage around 2nd bit

          BNZ       TX_OFF    ;  if <> 0 goto TX_OFF

TX_ON     LAC       RO        ;  get RO
          ORK       0001h     ;  turn PTT on
          SACL      RO        ;  store RO

          B         TX        ;  goto TX

TX_OFF    LAC       RO        ;  get RO
          ANDK      0FFFEh    ;  turn PTT off
          SACL      RO        ;  store RO

TX        IN        DI,0Ah    ;  get input TNC port
          LAC       DI        ;  load data
          ANDK      08000h    ;  strip garbage around 16th bit

          BNZ       THI       ;  if <> 0 goto THI

TLO       LAC       TXE       ;  get count in TXE

          BLZ       TL        ;  if < 0 goto TL

          SUBK      01h       ;  else decrement count in TXE
          SACL      TXE       ;  and store

TL        LAC       RB        ;  get last ramp value
          ADLK      INCL      ;  add space incr value to ramp

          B         WRAP      ;  goto WRAP

THI       LALK      TXEC      ;  get TXEC count
          SACL      TXE       ;  reset TXE
          LAC       RB        ;  get last ramp value
          ADLK      INCH      ;  add mark incr value to ramp

WRAP      SACL      RB        ;  store back in RB
          SBLK      TOP       ;  subtract TOP value

          BLEZ      GETS      ;  ramp not > top, go GETSC

          SACL      RB        ;  else put wrapped value in ramp buffer

GETS      ZAC                 ;  zero A
          ADD       RB,9      ;  get ramp offset * 2^7, shift 2^9
          SACH      TS        ;  store high for 2^-16 normalize
          LALK      TRG_TBL   ;  point to trig table
          ADD       TS        ;  add normalized ramp offset
          TBLR      TS        ;  read trig value * 2^10 into sine buff


TX_OUT    LAC       TS,2      ;  get sine value, shift 2^2
;  TX_OUT    LAC       SO,2      ;  get eye value, shift 2^2
          ANDK      0FFFCh    ;  mask out AIO control bits
          SACL      BUFO      ;  AIO out

DOUT      LAC       DC        ;  get DC

          BLEZ      NODCD     ;  if <= 0 goto NODCD

          SUBK      01h       ;  else decrement DC
          SACL      DC        ;  store

          B         DCD       ;  goto DCD

NODCD     LAC       DO        ;  get DO
          ORK       0100h     ;  set DCD bit high
          SACL      DO        ;  store DO

          B         END_DSP   ;  goto END_DSP
          
DCD       LAC       DO        ;  get DO
          ANDK      0FEFFh    ;  set DCD bit low
          SACL      DO        ;  store DO

END_DSP   LAC       DO        ;  load DO
          ANDK      0FBFFh    ;  reset duty cycle
          SACL      DO        ;  store DO
          OUT       DO,06h    ;  output to TNC port 06h 

          LDPK      08h       ;  data mem page 8
          OUT       CFG,07h   ;  output to TNC port 07h 
          LDPK      00h       ;  data mem page 0

          OUT       RO,0Bh    ;  output PTT to radio_1 port 0Bh

          RET                 ;  done!!


TUNE      NOP                 ;  show tuning LED204 -> LED201

L208      LAC       SO        ;  load SO
          SBLK      1910      ;  SO - 1.4 V

          BLZ       L207      ;  < 1.4, goto L207

          LDPK      08h       ;  else data mem page 8
          LAC       CFG       ;  load CFG
          ORK       0080h     ;  turn LED201 ON
          ANDK      0FFBFh    ;  turn LED202 OFF
          SACL      CFG       ;  store XO
          LDPK      00h       ;  data mem page 0

          LAC       RO        ;  load RO
          ANDK      0FF3Fh    ;  turn LED204 and LED203 OFF
          SACL      RO        ;  store RO

          B         TD          

L207      LAC       SO        ;  load SO
          SBLK      819       ;  SO - 0.6 V

          BLZ       LNUL      ;  < 0.6, goto LNUL

          LDPK      08h       ;  else data mem page 8
          LAC       CFG       ;  load CFG
          ORK       0040h     ;  turn LED202 ON
          ANDK      0FF7Fh    ;  turn LED201 OFF
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  data mem page 0

          LAC       RO        ;  load RO
          ANDK      0FF3Fh    ;  turn LED204 and LED203 OFF
          SACL      RO        ;  store RO

          B         TD          

LNUL      LAC       SO        ;  load SO
          ADLK      819       ;  SO + 0.6 V

          BLZ       L206      ;  < -0.6, goto L206

          LDPK      08h       ;  else data mem page 8
          LAC       CFG       ;  load CFG
          ANDK      0FF3Fh    ;  turn LED202 and LED 201 OFF
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  data mem page 0

          LAC       RO        ;  load RO
          ANDK      0FF3Fh    ;  turn LED204 and LED203 OFF
          SACL      RO        ;  store RO

          B         TD          

L206      LAC       SO        ;  load SO
          ADLK      1910      ;  SO + 1.4 V

          BLZ       L205      ;  < -1.4, goto L205

          LDPK      08h       ;  else data mem page 8
          LAC       CFG       ;  load CFG
          ANDK      0FF3Fh    ;  turn LED202 and LED201 OFF
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  data mem page 0

          LAC       RO        ;  load RO
          ORK       0040h     ;  turn LED203 ON
          ANDK      0FF7Fh    ;  turn LED204 OFF
          SACL      RO        ;  store RO

          B         TD          

L205      LDPK      08h       ;  else data mem page 8
          LAC       CFG       ;  load CFG
          ANDK      0FF3Fh    ;  turn LED202 and LED201 OFF
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  data mem page 0

          LAC       RO        ;  load RO
          ORK       0080h     ;  turn LED204 ON
          ANDK      0FFBFh    ;  turn LED203 OFF
          SACL      RO        ;  store RO

TD        RET                 ;  TUNE done!


AGC       NOP                 ;  to be done....

          RET                 ;  AGC done



WAIT4     LARK      AR2,0FFh
          CALL      WAIT2
          CALL      WAIT2

          RET


WAIT2     LARK      AR2,0FFh
          CALL      WAIT1
          CALL      WAIT1

          RET


WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ; Loop through count

          LARP      AR2
          BANZ      WAIT_A    ; Loop through count

          RET


          .MSFIRST

BPF       .WORD         2        ; hamming BPF, 18.57% <-> 22.27%, *2^13
          .WORD         7
          .WORD         2
          .WORD        -8
          .WORD        -8
          .WORD         5
          .WORD        14
          .WORD         2
          .WORD       -17
          .WORD       -14
          .WORD        13
          .WORD        24
          .WORD        -0
          .WORD       -27
          .WORD       -15
          .WORD        18
          .WORD        24
          .WORD        -3
          .WORD       -18
          .WORD        -5
          .WORD         3
          .WORD        -5
          .WORD         5
          .WORD        32
          .WORD        15
          .WORD       -56
          .WORD       -68
          .WORD        44
          .WORD       136
          .WORD        28
          .WORD      -173
          .WORD      -152
          .WORD       130
          .WORD       279
          .WORD        13
          .WORD      -334
          .WORD      -223
          .WORD       257
          .WORD       416
          .WORD       -43
          .WORD      -493
          .WORD      -246
          .WORD       393
          .WORD       495
          .WORD      -128
          .WORD      -593
          .WORD      -208
          .WORD       485
          .WORD       485
          .WORD      -208
          .WORD      -593
          .WORD      -128
          .WORD       495
          .WORD       393
          .WORD      -246
          .WORD      -493
          .WORD       -43
          .WORD       416
          .WORD       257
          .WORD      -223
          .WORD      -334
          .WORD        13
          .WORD       279
          .WORD       130
          .WORD      -152
          .WORD      -173
          .WORD        28
          .WORD       136
          .WORD        44
          .WORD       -68
          .WORD       -56
          .WORD        15
          .WORD        32
          .WORD         5
          .WORD        -5
          .WORD         3
          .WORD        -5
          .WORD       -18
          .WORD        -3
          .WORD        24
          .WORD        18
          .WORD       -15
          .WORD       -27
          .WORD        -0
          .WORD        24
          .WORD        13
          .WORD       -14
          .WORD       -17
          .WORD         2
          .WORD        14
          .WORD         5
          .WORD        -8
          .WORD        -8
          .WORD         2
          .WORD         7
          .WORD         2

LPF       .WORD         4        ; hamming LPF, 2.44%, *2^13
          .WORD         3
          .WORD         3
          .WORD         3
          .WORD         2
          .WORD         2
          .WORD         1
          .WORD        -0
          .WORD        -2
          .WORD        -4
          .WORD        -6
          .WORD        -9
          .WORD       -12
          .WORD       -15
          .WORD       -19
          .WORD       -22
          .WORD       -26
          .WORD       -29
          .WORD       -32
          .WORD       -34
          .WORD       -36
          .WORD       -36
          .WORD       -35
          .WORD       -32
          .WORD       -28
          .WORD       -21
          .WORD       -12
          .WORD        -1
          .WORD        12
          .WORD        28
          .WORD        47
          .WORD        67
          .WORD        90
          .WORD       114
          .WORD       140
          .WORD       167
          .WORD       194
          .WORD       222
          .WORD       249
          .WORD       275
          .WORD       300
          .WORD       324
          .WORD       344
          .WORD       363
          .WORD       378
          .WORD       389
          .WORD       397
          .WORD       401
          .WORD       401
          .WORD       397
          .WORD       389
          .WORD       378
          .WORD       363
          .WORD       344
          .WORD       324
          .WORD       300
          .WORD       275
          .WORD       249
          .WORD       222
          .WORD       194
          .WORD       167
          .WORD       140
          .WORD       114
          .WORD        90
          .WORD        67
          .WORD        47
          .WORD        28
          .WORD        12
          .WORD        -1
          .WORD       -12
          .WORD       -21
          .WORD       -28
          .WORD       -32
          .WORD       -35
          .WORD       -36
          .WORD       -36
          .WORD       -34
          .WORD       -32
          .WORD       -29
          .WORD       -26
          .WORD       -22
          .WORD       -19
          .WORD       -15
          .WORD       -12
          .WORD        -9
          .WORD        -6
          .WORD        -4
          .WORD        -2
          .WORD        -0
          .WORD         1
          .WORD         2
          .WORD         2
          .WORD         3
          .WORD         3
          .WORD         3
          .WORD         4

TRG_TBL   .WORD    00h        ;  sine table, scaled 2^10, 128 + 1 entries
          .WORD    032h
          .WORD    064h
          .WORD    096h
          .WORD    0C8h
          .WORD    0F9h
          .WORD    0129h
          .WORD    0159h
          .WORD    0188h
          .WORD    01B6h
          .WORD    01E3h
          .WORD    020Eh
          .WORD    0239h
          .WORD    0262h
          .WORD    028Ah
          .WORD    02B0h
          .WORD    02D4h
          .WORD    02F7h
          .WORD    0318h
          .WORD    0336h
          .WORD    0353h
          .WORD    036Eh
          .WORD    0387h
          .WORD    039Eh
          .WORD    03B2h
          .WORD    03C4h
          .WORD    03D4h
          .WORD    03E1h
          .WORD    03ECh
          .WORD    03F5h
          .WORD    03FBh
          .WORD    03FFh
          .WORD    0400h
          .WORD    03FFh
          .WORD    03FBh
          .WORD    03F5h
          .WORD    03ECh
          .WORD    03E1h
          .WORD    03D4h
          .WORD    03C4h
          .WORD    03B2h
          .WORD    039Eh
          .WORD    0387h
          .WORD    036Eh
          .WORD    0353h
          .WORD    0336h
          .WORD    0318h
          .WORD    02F7h
          .WORD    02D4h
          .WORD    02B0h
          .WORD    028Ah
          .WORD    0262h
          .WORD    0239h
          .WORD    020Eh
          .WORD    01E3h
          .WORD    01B6h
          .WORD    0188h
          .WORD    0159h
          .WORD    0129h
          .WORD    0F9h
          .WORD    0C8h
          .WORD    096h
          .WORD    064h
          .WORD    032h
          .WORD    00h
          .WORD    0FFCEh
          .WORD    0FF9Ch
          .WORD    0FF6Ah
          .WORD    0FF38h
          .WORD    0FF07h
          .WORD    0FED7h
          .WORD    0FEA7h
          .WORD    0FE78h
          .WORD    0FE4Ah
          .WORD    0FE1Dh
          .WORD    0FDF2h
          .WORD    0FDC7h
          .WORD    0FD9Eh
          .WORD    0FD76h
          .WORD    0FD50h
          .WORD    0FD2Ch
          .WORD    0FD09h
          .WORD    0FCE8h
          .WORD    0FCCAh
          .WORD    0FCADh
          .WORD    0FC92h
          .WORD    0FC79h
          .WORD    0FC62h
          .WORD    0FC4Eh
          .WORD    0FC3Ch
          .WORD    0FC2Ch
          .WORD    0FC1Fh
          .WORD    0FC14h
          .WORD    0FC0Bh
          .WORD    0FC05h
          .WORD    0FC01h
          .WORD    0FC00h
          .WORD    0FC01h
          .WORD    0FC05h
          .WORD    0FC0Bh
          .WORD    0FC14h
          .WORD    0FC1Fh
          .WORD    0FC2Ch
          .WORD    0FC3Ch
          .WORD    0FC4Eh
          .WORD    0FC62h
          .WORD    0FC79h
          .WORD    0FC92h
          .WORD    0FCADh
          .WORD    0FCCAh
          .WORD    0FCE8h
          .WORD    0FD09h
          .WORD    0FD2Ch
          .WORD    0FD50h
          .WORD    0FD76h
          .WORD    0FD9Eh
          .WORD    0FDC7h
          .WORD    0FDF2h
          .WORD    0FE1Dh
          .WORD    0FE4Ah
          .WORD    0FE78h
          .WORD    0FEA7h
          .WORD    0FED7h
          .WORD    0FF07h
          .WORD    0FF38h
          .WORD    0FF6Ah
          .WORD    0FF9Ch
          .WORD    0FFCEh
          .WORD    00h


          .END
