	.TITLE	"FSK modem/TNC for HF RTTY and SITOR"
;
; See readme.txt [hot_hf93.doc] file in this distribution for setup
; instructions and command summary.
;
;************************************************************************
;*                                                                      *
;* Copyright (c) David L. Mills 1995                                    *
;*                                                                      *
;* Permission to use, copy, modify, and distribute this software and    *
;* its documentation for any purpose and without fee is hereby granted, *
;* provided that the above copyright notice appears in all copies and   *
;* that both the copyright notice and this permission notice appear in  *
;* supporting documentation, and that the name University of Delaware   *
;* not be used in advertising or publicity pertaining to distribution   *
;* of the software without specific, written prior permission. The      *
;* University of Delaware makes no representations about the            *
;* suitability this software for any purpose. It is provided "as is"    *
;* without express or implied warranty.                                 *
;*                                                                      *
;************************************************************************
; This program implements a FSK modem/TNC for HF asynchronous Baudot
; (ITA-2) and synchronous DITOR/AMTOR (CCIR 476 Mode B) signalling. It
; operates at speeds to 100 baud, either as a TNC with ASCII input/outut
; via the onboard UART, or as a regenerator with clean AFSK output
; reconstructed from a possibly noisy input. The algorithms use
; Butterworth and matched filters, soft-decision detection, maximum a-
; priory (MAP) estimation and Viterbi decoding. In case of continuous
; asynchronous signalling, the program can operate in a synchronous mode
; using a phase-lock loop to flywheel through momentary carrier fades.
; The predetection design and many of the operator features were
; inspired by the Dovetron TU of long ago, but implemented in DSP,
; rather than analog form. The postdetection design, including MAP
; estimation, Viterbi and matched-filter decoding and synchronous mode
; was inspired by a LSI-11 program also of long ago. If this thing can't
; pull RTTY and SITOR signals from the muck, nothing can.
;
; Include files
;
	.NOLIST
#INCLUDE "regs.inc"		; TMC320 registers
#INCLUDE "ports.inc"		; DSP93 ports
#INCLUDE "serial.inc"		; 16550 UART
	.LIST
;
; Program memory map
;
; External	1000-7fff	program
;
; Data memory map
;
; Block B2	0060-007f	interrupt variables and stack
; Block B0	0200-02ff	filter coefficients (mapped to prog mem)
; Block B1	0300-03ff	program variables and filter delay lines
; External	0400-7fff	program variables
;
; Auxiliary register usage
;
; AR0	utility compare and temporary register
; AR1	utility pointer
; AR2	utility counter/pointer
; AR3	utility pointer
; AR4	spare
; AR5	interrupt stack pointer
; AR6	AIO buffer pointer
; AR7	AIO buffer pointer
;
; Define macros
;
#DEFINE FAC 120*256/SFREQ	; Hz to phase increment
;
; Status bits (STATUS)
;
SIGSTA	.EQU	0		; signal gate
PULSTA	.EQU	1		; pulse gate
SYNSTA	.EQU	2		; sync mode
RSHSTA	.EQU	3		; receive Baudot figures shift
XSHSTA	.EQU	4		; transmit Baudot figures shift
MRKSTA	.EQU	5		; mark channel disable
SPCSTA	.EQU	6		; space channel disable
RTSSTA	.EQU	7		; transmit request to send
CTSSTA	.EQU	8		; transmit clear to send
;
; Status bits (MODE)
;
SYNMOD 	.EQU	0		; sync mode enable
PULMOD	.EQU	1		; pulse gate enable
SIGMOD	.EQU	2		; mark signal gate enable
NARMOD	.EQU	3		; narrow input filter
PORMOD	.EQU	4		; radio port 1/2 select
TORMOD	.EQU	5		; SITOR Mode B enable
ECHMOD	.EQU	7		; digital echo on transmit
FDXMOD	.EQU	8		; full-duplex
REVMOD	.EQU	9		; reverse shift
;
; AIO Configuration constants: HPF in, sync, 3-V half scale, sin x/x.
; RA is chosen to get the SCF as close to 288 kHz as possible; RB is
; chosen to get the sample frequency as close to 8.0 kHz as possible.
; This gives a 3-dB bandwidth of about 100-3700 Hz at the AIO input.
;
CFGA	.EQU	CMDA_VAL(17)	; AIO load RA=TA=17 for SCF 294.1 kHz
CFGB	.EQU	CMDB_VAL(37)	; AIO load RB=TB=37 for 7.949 kHz
AIO_GAIN .EQU	AIO_GAIND60|AIO_GAIND70 ; AIO gain
CFGC	.EQU	AIO_CONFIG|AIO_INSADHPF|AIO_SYNC|AIO_GAIN|AIO_INSSINX
;
; Tone frequency definitions 
;
DEGREE	.EQU	360/3		; sin table entries (3-degree steps)
PI2	.EQU	DEGREE<<8	; radix for output phase wrap
SFREQ	.EQU	7949		; sample frequency (5000000 / (17 * 37))
INCR	.EQU	126		; sample interval (us) (1000000 / SFREQ)
TUNEX	.EQU	39		; 10 Hz (10 * 120 * 256 / SFREQ)
SITBIT	.EQU	80		; samples per SITOR encoder bit
;
; Sizes of arrays
;
BINSIZ	.EQU	8		; samples per baud interval
CHRSIZ	.EQU	8		; baud intervals per character + 1
SRVSIZ	.EQU	CHRSIZ*BINSIZ	; size of survivor table
MFRSIZ	.EQU	3000		; max size of matched filter delay line
OUTSIZ	.EQU	1000		; size of circular buffers
FIRSIZ	.EQU	71		; size of FIR bpf delay line
TORSIZ	.EQU	7*6		; size of SITOR character shift register
IIRSIZ	.EQU	3*4		; size of IIR lpf delay lines (4)
STRSIZ	.EQU	16		; size of filter structure
BUFSIZ	.EQU	50		; size of AIO buffer
;
; Miscellaneous definitions
;
LIMIT	.EQU	10000		; limiter threshold (LED 1)
ALIMIT	.EQU	10000		; autostart preset threshold
CLIMIT	.EQU	9000		; carrier preset threshold
SYNJTR	.EQU	BINSIZ/2	; sync counter max jitter
FRMSIZ	.EQU	SRVSIZ-BINSIZ	; sync counter radix (samples / char)
CCONST	.EQU	16-4		; scaled carrier/noise time constant
DCONST	.EQU	16-5		; scaled SITOR time constant
TCONST	.EQU	16-4		; scaled acquistion pll time constant
SCONST	.EQU	16-4		; scaled tracking pll time constant
PGAIN	.EQU	4+5		; scaled pll gain
P1_MSK	.EQU	PTT_R1+FREQ_UP_R1+FREQ_DN_R1 ; RADIO_CTRL port 1 mask
P2_MSK	.EQU	PTT_R2+FREQ_UP_R2+FREQ_DN_R2 ; RADIO_CTRL port 2 mask
TUNEW	.EQU	10*8		; tune pulse width (ms * 8))
AHOLD	.EQU	200		; autostart hold interval (char times)
RHOLD	.EQU	10		; RTS hold interval (char times)
PIO2	.EQU	51472>>1	; pi / 2 in q14 format
RQ	.EQU	1100110b	; CCIR 476 signal RQ
ALPHA	.EQU	0001111b	; CCIR 476 signal alpha
;
; Nonprinting character codes
;
LTRS	.EQU	1Fh		; ITA-2 LTRS shift
FIGS	.EQU	1Bh		; ITA-2 FIGS shift
NUL	.EQU	00h		; ITA-2/ASCII null
CR	.EQU	0Dh		; ASCII carriage return
LF	.EQU	0Ah		; ASCII	line feed
BEL	.EQU	07h		; ASCII bell
SYN	.EQU	16h		; ASCII SYN (Baudot LTRS/diddle)
EOT	.EQU	04h		; ASCII EOT (down PTT immediately)
ACK	.EQU	06h		; ASCII ACK (down PTT when buffer empty)
LCTST	.EQU	5		; ITA-2 LTRS test
LC	.EQU	1<<LCTST	; ITA-2 LTRS bit
UCTST	.EQU	6		; ITA-2 LTRS test
UC	.EQU	1<<UCTST	; ITA-2 FIGS bit

;
; Interesting block addresses and memory page pointers
;
B_PAGE	.EQU	0h		; base page pointer
U_PAGE	.EQU	8h		; user page pointer
U_ADDR	.EQU	U_PAGE<<7	; user page address
B0P	.EQU	0FF00h		; block B0 program address
B0D	.EQU	200h		; block B0 data address
B1D	.EQU	300h		; block B1 data address

;
; Following are variables accessed by either absolute or indirect
; addressing modes.
;
; Program variables in internal memory (block B0) initialized by
; program. These include FIR and IIR filter coefficients, as well as the
; SITOR bit vector delay line. The first set of addresses are valid
; following a CNFD instruction, which maps block B0 to data memory; the
; second set are valid following a CNFP instructuion, which maps block
; B0 to program memory.
;
BPFCOD	.EQU	B0D		; FIR bandpass filter coefficients
LPFCOD	.EQU	BPFCOD+FIRSIZ	; IIR lowpass filter coefficients
CHRDLD	.EQU	LPFCOD+IIRSIZ	; SITOR character vector
TORDLD	.EQU	CHRDLD+7	; SITOR bit vector shift register
B0DEND	.EQU	TORDLD+(6*7)	; end of block B0
;
BPFCOP	.EQU	BPFCOD+B0P-B0D	; FIR bandpass filter coefficients
LPFCOP	.EQU	LPFCOD+B0P-B0D	; IIR lowpass filter coefficients
CHRDLP	.EQU	CHRDLD+B0P-B0D	; SITOR character vector
TORDLP	.EQU	TORDLD+B0P-B0D	; SITOR bit vector shift register
;
; Program variables in internal memory (block B1) cleared at reset.
; These consist of delay lines used by the MACD and DMOV instructions,
; which require the data to be in internal memory.
;
DELBPF	.EQU	B1D		; FIR bandpass filter delay line
DELLPF	.EQU	DELBPF+FIRSIZ+1	; IIR lowpass filter delay line
;
; The survivor table contains a shift-register survivor for eight
; samples of each bit and eight bits of the character. It is used to
; construct the maximum and minimum amplitudes, as well as the slice
; level, carrier and noise estimates.
;
SRVBGN	.EQU	DELLPF+IIRSIZ	; beginning of survivor table
SRVEND	.EQU	SRVBGN+SRVSIZ	; end of survivor table
;
; Program variables in external memory (U_PAGE) cleared at reset. We
; protect one location at the beginning for I/O status.
;
USRBGN	.EQU	U_ADDR+1	; beginning of clear region
;
; These structures are used by the FIR, IIR and MF filters. For the IIR
; and MF filters, the mark and space filter center frequencies are
; determined by the phase increment. For these filters, the mark filter
; is fixed and the space filter is tunable. The FIR filters are not
; tunable.
;
; Structure format
; 0	filter routine entry point
; 1	phase increment
; 2	current phase
; 3-4	I channel output
; 5-6	Q channel output
; 7-8	sum of squares
; 9-10	detector output
; 11-15 not used
;
MSTRUCT	.EQU	U_ADDR+100h	; mark filter structure
SSTRUCT	.EQU	MSTRUCT+STRSIZ	; space filter structure
;
; The AIO buffer is a circular structure indexed by AR6 and AR7.
; AR6 is used only by interrupt routines. The receive interrupt routine
; stores the sample at AR6 and then increments AR6. The transmit
; interrupt routine loads the sample at AR6, which is the oldest in the
; circular buffer. The main program loops until AR7 is not equal to AR6,
; then loads the sample found at AR7, which is the oldest sample not yet
; processed. The program loads the output at AR7 and then increments
; AR7. Since input and output interrupts are synchronous, no data are
; lost or duplicated.
;
INTBUF	.EQU	SSTRUCT+STRSIZ	; AIO buffer
;
; Circular buffers. These are used for character I/O, one each for input
; (modem and echo to UART) and output (UART to modem).
;
; Buffer structure format
; 0	address of first word beyond buffer
; 1	address of first word of buffer
; 2	put pointer
; 4	get pointer
;
INPBCB	.EQU	INTBUF+BUFSIZ	; input buffer structure
OUTBCB	.EQU	INPBCB+4	; output buffer structure
INPBUF	.EQU	OUTBCB+4	; transmit circular buffer
OUTBUF	.EQU	INPBUF+OUTSIZ	; receive circular buffer
;
; Circular delay lines. The matched filter can grow up to 3000 words,
; depending on baud interval. In the predetection case, it contains four
; samples for each cycle; in the baseband case, it contains two samples
; interleaved with two dummy words for a total of four. The mark and
; space delay lines are used to compute the carrier and noise estimates.
;
DELMFR	.EQU	OUTBUF+OUTSIZ	; matched filter delay line
DELMRK	.EQU	DELMFR+MFRSIZ	; mark delay line
DELSPC	.EQU	DELMRK+SRVSIZ	; space delay line
;
; This vector is used to integrate the distance for each of 14 phase
; positions of the CCIR 476 sync sequence RQ-ALPHA.
;
; Structure format
; 0	distance
; 1	shift register
; 2	shift register population count
;
TORSTR	.EQU	DELSPC+SRVSIZ	; phase distances
;
; Table of FIR coefficients for matched filter receiver. We run through
; these to find the most likely vector with four bits one and three bits
; zero. There are 35 of these suckers, 32 of which have Baudot
; equivalents and three of which are special characters.
;
TORTAB	.EQU	TORSTR+(3*14)	; SITOR coefficients
USREND	.EQU	TORTAB+(35*7)	; end of clear region

;
; Following are variables accessed by direct addressing mode. These are
; offsets relative the the data page pointer.
;
: Program variable offsets in block B2 (B_PAGE) initialized at startup.
;
TNC_BUF	.EQU	60h		; TNC_OUTPUT port status
PDC_BUF	.EQU	61h		; RADIO_CTRL port status
;
; Program variable offsets in block B2 (B_PAGE)
;
UARTI	.EQU	62h		; UART input buffer
UARTO	.EQU	63h		; UART output buffer
UARTC	.EQU	64h		; UART modem control
XTMP	.EQU	65h		; temp
SPY	.EQU	70h		; spy temp (debug)
STACK	.EQU	80h		; interrupt stack (grows down)
;
; Program variable offsets in external memory (U_PAGE) initialized at
; startup.
;
PIO_BUF	.EQU	00h		; RADIO_GAIN port status
;
; Program variable offsets in external memory (U_PAGE) cleared at reset.
;
STATUS	.EQU	01h		; status bits
MODE	.EQU	02h		; mode bits
;
; RF analog signal variables and pointers
;
INPSIG	.EQU	03h		; AIO input buffer
FLTSIG	.EQU	04h		; filtered/limited input signal
MARK	.EQU	05h		; mark channel output
SPACE	.EQU	06h		; space channel output
MFPTR	.EQU	07h		; matched filter pointer
MFSIZE	.EQU	08h		; matched filter size
MFGAIN	.EQU	09h		; matched filter gain
;
; Baseband analog signal variables and pointers
;
ACCUM	.EQU	10h		; scope detector output
SSYNC	.EQU	11h		; scope sync
POSMAX	.EQU	12h		; max positive signal
NEGMAX	.EQU	13h		; max negative signal
CARDET	.EQU	14h		; carrier estimator
NOISE	.EQU	15h		; noise estimator
SLICE	.EQU	16h		; comparator slice level
MSPTR	.EQU	17h		; mark/space memory offset
;
; Signal quality variables
;
XPHASE	.EQU	30h		; phase error signal
DIST	.EQU	31h		; max survivor distance
ATHRESH	.EQU	32h		; autostart threshold
AUTSTR	.EQU	33h		; autostart signal
AUTSRG	.EQU	34h		; autostart shift register
CTHRESH	.EQU	35h		; carrier threshold
CARSTR	.EQU	36h		; carrier signal
CARSRG	.EQU	37h		; carrier shift register
;
; Counters, we got counters
;
FECCTR	.EQU	46h		; SITOR character counter
ASYCTR	.EQU	47h		; start/stop character counter
SMPCTR	.EQU	48h		; receive sample counter
BITCTR	.EQU	49h		; receive bit counter
SRVCTR	.EQU	4Ah		; receive strobe counter
SYNCTR	.EQU	4Bh		; sync counter
AUTCTR	.EQU	4Ch		; autostart counter
XBTCTR	.EQU	4Dh		; transmit bit counter
XSMCTR	.EQU	4Eh		; transmit sample counter
RTSCTR	.EQU	4Fh		; transmit RTS counter
;
; Tuning parameters
;
RANGE	.EQU	50h		; baud interval counter radix
COUNT	.EQU	51h		; baud interval counter
GAIN	.EQU	52h		; filter gain
LGAIN	.EQU	53h		; limiter gain (shift)
SYNDEL	.EQU	54h		; frame interval
;
; SITOR/AMTOR variables
;
TORPTR	.EQU	58h		; decoder sync bit pointer
TORMAX	.EQU	59h		; decoder max sync bit distance
TORTMP	.EQU	5Ah		; decoder sync pointer
TORLST	.EQU	5Bh		; decoder last sync pointer
FECBUF	.EQU	5Ch		; encoder circular buffer pointer (2)
FECCHR	.EQU	FECBUF+2	; encoder buffer
;
; AFSK output encoder variables
;
KEY	.EQU	60h		; transmit shift register
OUTSW	.EQU	61h		; display pointer
OMARK	.EQU	62h		; AFSK mark phase increment
OSPACE	.EQU	63h		; AFSK space phase increment
OPHASE	.EQU	64h		; AFSK output phase
;
; Radio control variables
;
TUNE	.EQU	68h		; radio port tune indicator
TUNCTR	.EQU	69h		; tune pulse counter
;
; Command interpreter variables
;
UTMP1	.EQU	70h		; user temp 1
UTMP2	.EQU	71h		; user temp 2
UCHAR	.EQU	72h		; character temp
UPTR	.EQU	73h		; pointer temp
UADR	.EQU	74h		; address temp
UCMD	.EQU	75h		; command table pointer
UMOD	.EQU	76h		; saved command table pointer
;
; Local temporaries
;
TMP	.EQU	78h		; temporary
STS	.EQU	79h		; temporary
PEEK	.EQU	7Ah		; debug peekaboo
;
;  Program transfer vector
;
	.ORG	1000h		; monitor origin
	B       TINT		; timer interrupt
	B       RINT		; AIO receive interrupt
XIN10	B       XINT		; AIO transmit interrupt
	B       IINT		; TRAP instruction interrupt
	B       GO		; program starting address

;
; AIO receive interrupt. This is designed for minimal intrusion on
; variables used by interruptable code. AIO rx interrupt is enabled when
; tx initialization is complete.
;
; Note: This is the only place where AR6 is incremented. Immediately
; after increment, AR6 is checked for overflow and wrapped.
;
; Variables
; AR6		AIO buffer pointer
; DRR		AIO input buffer
;
RINT	LARP	AR5		; save context
	MAR	*-
	SST1	*-
	SST	*-
	SAR	AR0,*,AR6	; AR6 is AIO buffer pointer
	LDPK	B_PAGE
	LAR	AR0,DRR		; read AIO input buffer
	SAR	AR0,*+
	LRLK	AR0,INTBUF+BUFSIZ ; is pointer at buffer end
	CMPR	
	BBZ	RIN10		; branch if no
	LRLK	AR6,INTBUF	; yes. reset pointer
RIN10	LARP	AR5
	CALL	DUMP		; dump status
	LAR	AR0,*+		; restore context
	LST	*+
	LST1	*+
	EINT			; enable interrupts
	RET

;
; AIO transmit interrupt used in normal processing
;
; Variables
; AR6		AIO buffer pointer
; DXR		AIO output buffer
;
XINT	LARP	AR5		; save context
	MAR	*-
	SST1	*-
	SST	*-
	SAR	AR0,*,AR6	; AR6 is AIO buffer pointer
	LDPK	B_PAGE
	LAR	AR0,*,AR5	; write AIO output buffer
	SAR	AR0,DXR
	CALL	DUMP		; dump status
	LAR	AR0,*+		; restore context
	LST	*+
	LST1	*+
	EINT			; enable interrupts
	RET

;
; AIO transmit interrupt routine used only during AIO configuration.
; Note that the ARP and designated AR point to configuration data.
;
; Variables
; DXR		AIO output buffer
;
XINT2	LAC	*+		; this has to be quick and ugly
	SACL	DXR
	EINT
	RET

;
; Trap interrupt (for debug). TRAP instructions can be sprinkled in the
; code to watch critical sections and leave tracks useful for debugging.
;
IINT	DINT			; avoid interrupts here
	LARP	AR5		; save context
	MAR	*-
	SST1	*-
	SST	*-
	SAR	AR0,*
	LDPK	B_PAGE
	CALL	DUMP		; dump status
	LAR	AR0,*+		; restore context
	LST	*+
	LST1	*+
	EINT			; enable interrupts
	RET

;
; Routine to dump trace data for debug. This is done on every I/O and
; TRAP interrupt. At this point, SS1, SS0 and AR0 are already on the
; stack (AR5) and the data page pointer set to B_PAGE (0). The remaining
; registers and top two words on the internal CPU stack are saved.
;
DUMP	SAR	AR5,XTMP	; save stack pointer
	MAR	*-
	SAR	AR1,*-		; save aux registers AR1-AR7
	SAR	AR2,*-
	SAR	AR3,*-
	SAR	AR4,*-
	SAR	AR5,*-
	SAR	AR6,*-
	SAR	AR7,*-
	POPD	*-		; save calling address
	POPD	*		; save interrupting address
	PSHD	*+
	PSHD	*-
	LAR	AR5,XTMP	; restore stack pointer
	RET

;
; Timer interrupt (not used)
;
TINT	EINT			; enable interrupts
	RET

;
; Initialize processor configuration. These set the options as does the
; CPU upon first coming up; they are here in case the monitor screws up.
;
GO	CNFP			; configure block B0 as program
	SSXM			; set sign extension mode
	SOVM			; set overflow mode
	SPM	1		; set P register output shift for q15
	FORT	0		; set AIO 16-bit mode
	SFSM			; set FSM (1) frame sync mode
	RTXM			; set TXM (0) transmit mode
	LDPK	U_PAGE
	LALK	AIO_RESET+GAIN4+REC_IN_1 ; reset AIO
	SACL	PIO_BUF
	OUT	PIO_BUF,RADIO_GAIN
	CALL    WAIT4
	LALK    LED_202+GAIN4+REC_IN_1 ; disable AIO; LED 2 on (debug)
	SACL    PIO_BUF
	OUT     PIO_BUF,RADIO_GAIN 
	CALL    WAIT4
	LALK    AIO_ENABLE+GAIN4+REC_IN_1 ;enable AIO
	SACL    PIO_BUF        
	OUT     PIO_BUF,RADIO_GAIN
	CALL    WAIT4
	LDPK	B_PAGE
	LACK	UART_MCR_RTS	; raise RTS (CTS at the connector)
	SACL	UARTC
	RPTK	7-1		; flush stack
	POP
	LRLK	AR5,STACK	; initialize stack pointer
	LRLK	AR6,INTBUF	; initialize AIO buffer pointers
	LRLK	AR7,INTBUF
;
; Set up for AIO initialization. Keep this a deep dark secret and don't
; tell anybody the transmit interrupt routines are switched on-the-wing
; for configuration. This is because the AIO rascal has unreal latency
; requirements only during initialization. There seems to be no way for
; an interrupt routine to satisfy these without appearing non-
; transparent for general purpose processing.
;
	DINT			; turn out the lights
	LALK	XINT2
	SACL	XTMP
	LALK	XIN10+1		; clobber transmit vector
	TBLW	XTMP
	LARP	AR7		; copy AIO initialization vector
	RPTK	7-1
	BLKP	INITAB,*+
	LACK	20h		; enable AIO tx interrupt
	SACL	IMR
	ZAC			; flush AIO tx buffer
	SACL	DXR
	EINT			; turn on the lights
	LARP	AR6		; load initialization word
INI10	IDLE			; wait for interrupt and AC
	BNZ	INI10		; branch if more
	DINT			; done. darkness again
	LALK	XINT		; unclobber transmit vector
	SACL	XTMP
	LALK	XIN10+1
	TBLW	XTMP
	LACK	30h		; enable AIO rx and tx interrupts
	SACL	IMR
	LDPK	U_PAGE
	CALL	RESET		; initialize program variables
	EINT			; enable interrupts
;
; Loop looking for a AIO input sample. When found, the RF input
; processing routine is called to filter and limit the input signal and
; develop the mark and space baseband signals. The RF output processing
; routine is also called to generate the AIO output signal. The input
; samples are decimated at 8 samples per bit interval. At each sample
; the baseband input routine is called to generate the input FSK keying
; waveform and decode the ASCII character. Then, the baseband output
; routine is called to encode the ASCII character and generate the
; output keying waveform.
;
; Variables
; RANGE		baud interval counter radix
; COUNT		baud interval counter
; TUNE		radio port tune indicator
; TUNCTR	tune pulse counter
;
; Note: This is the only place where AR6 is compared to AR7. The CMPR
; insruction represents the atomic interaction in order to determine
; whether data are in the buffer or not; thus, there are no races. Even
; if there were, unread data would get caught on the next cycle through
; the loop.
;
LOOP	LARP	AR7		; AR7 is AIO buffer pointer
	SAR	AR6,TMP		; are there data in the buffer
	LAR	AR0,TMP
	CMPR	
	BBZ	LOP10		; branch if yes
	LALK	LED_202		; no. LED 2 on
	CALL    LEDON
	CALL	LIGHTS		; blink the control panel
	CALL	GETCH		; UART input processing
	CALL	PUTCH		; UART output processing
	LALK	LED_202		; LED 2 off
	CALL    LEDOFF
	LAC	TUNCTR		; is tune pulse counter running
	BZ	LOOP		; branch if no
	SUBK	1		; yes. decrement by 125 us
	SACL	TUNCTR		; did counter underflow
	BGZ	LOOP		; branch if no
	ZAC			; yes. kill the tune bits
	SACL	TUNE
	B       LOOP
;
; A new input sample is in the AIO input buffer. The processing routines
; are called according to the following decimation schedule:
;
; AIO sample rate. This occurs at the programmed AIO sample interval,
; nominally 8000 Hz, but actually 7949 Hz (SFREQ), which causes a little
; pain sometimes. The baseband mark and space signals are developed here
; along with the AFSK output signals.
;
; Baseband sample rate. This occurs at eight times the nominal baud
; rate, but is variable over a small range for SITOR symbol tracking.
; The maximum and minimum differential signals, slice level and
; mark/space memory signals are developed here. The RTTY and SITOR input
; processing is done here using matched-filter techniques in which the
; calculated distance is used as a quality indicator.
;
; Bit rate. This occurs at the programmed baud rate, which is as
; close to the specified baud rate as possible. The RTTY and SITOR
; output processing is done here and the carrier and noise estimates are
; developed.
;
; Character rate. This occurs at the nominal character rate, which is
; 7.5 bit intervals for RTTY and 14 bit intervals for SITOR.
;
LOP10	CALL	RFINP		; RF input processing
	CALL	RFOUT		; RF output processing
	BIT	MODE,TORMOD	; is this SITOR
	BBZ	LOP13		; branch if no
	LAC	XSMCTR		; yes. precision decimation
	SBLK	256
	SACL	XSMCTR
	BGEZ	LOP13
	ADLK	20349		; (79.49*256)
	SACL	XSMCTR
	CALL	SITOUT		; SITOR encoder processing
LOP13	LAC	COUNT		; is it time for the next sample
	SUBK	INCR
	SACL	COUNT
	BGEZ	LOOP		; branch if no
	ADD	RANGE		; yes. restore decimation counter
	SACL	COUNT
	CALL	DSPINP		; baseband input processing
	CALL	BITS
	BIT	MODE,TORMOD	; is alternate decoder selected
	BBNZ	LOP12		; branch if yes
	CALL	DSPOUT		; no. RTTY encoder processing
	CALL	SSTOP		; RTTY decoder processing
	B	LOOP
;
LOP12	CALL	SITOR		; SITOR decoder processing
	B	LOOP

;
; RF input processing
;
; This routine processes the AIO input signal to produce the mark and
; space signals used by the baseband input routine.
;
; Variables
; AR7		AIO buffer pointer
; INPSIG	AIO input buffer
; FLTSIG	filtered/limited input signal
; MARK		mark channel output
; SPACE		space channel output
; DELBPF	FIR bandpass filter delay line
; DELLPF	IIR lowpass filter delay line
; MSTRUCT	mark filter structure
; SSTRUCT	space filter structure
;
; Clip signals with energy above an arbitrary threshold (LIMIT). This
; helps to reduce errors due to noise spikes and static crashes, etc.
; The threshold is chosen rather arbitrary at 10dB below overload. LED 1
; is lit when in limiting condition.
;
RFINP	LARP	AR7		; AR7 is AIO buffer pointer
	LAC	*,2,AR1		; load sample in q15 format
	SACL	INPSIG
	ABS			; is signal above threshold
	SBLK	LIMIT
	BLZ	INP10		; branch if no
	LALK	LED_201		; yes. LED 1 on
	CALL	LEDON
	B	INP11
;
INP10	LALK	LED_201		; signal below threshold. LED 1 off
	CALL	LEDOFF
INP11	LAC	INPSIG		; clip signal at max
	SBLK	LIMIT
	BLZ	INP12
	ZAC
INP12	ADLK	2*LIMIT		; clip signal at min
	BGZ	INP13
	ZAC
INP13	SBLK	LIMIT
	SACL	FLTSIG
;
; Filter the clipped signal using a FIR filter. The filter bandwidth is
; determined by the coefficients loaded in block B0P. A narrow (about
; 400 Hz) filter is used for 170-Hz shift and wide (about 1000 Hz)
; filter used for the wider shifts. These filters are used primarily to
; reduce the effects of aliasing in the following limiter. The l{0-5}
; command is used to select the limiter gain.
;
	LRLK	AR1,DELBPF
	LAC	FLTSIG
	SACL	*
	ADRK	FIRSIZ-1	; point to end of delay line
	MPYK	0		; initialize sum and product registers
	ZAC
	RPTK	FIRSIZ-1	; multiply and accumulate
	MACD	BPFCOP,*-
	APAC			; include last product
	SACH	FLTSIG
;
; Limit signal at selected gain 0-30dB. Under good conditions, up to 30-
; dB limiting can be used with IIR filters for ease of tuning. When
; things get tough, use less limiting and matched filters.
; 
	LT	LGAIN		; get limiter gain
	LACT	FLTSIG		; limit signal at max
	SBLK	LIMIT
	BLZ	INP20
	ZAC
INP20	ADLK	2*LIMIT		; limit signal at min
	BGZ	INP21
	ZAC
INP21	SBLK	LIMIT
	SACL	FLTSIG
;
; The demodulator center frequency and mark-space shift is selected with
; the m{1-8} command. The center frequency is 1700 Hz for wide shifts
; (850, 600, and 400 Hz) and 2210 Hz for narrow shift (170 Hz). For the
; synchronous filters, the receiver is first tuned to the mark
; frequency, which is fixed at 1275, 1400, 1500 and 2125 Hz for 850,
; 600, 400 and 170-Hz shifts, respectively. At these shifts the nominal
; space frequency is 2125, 2000. 1900 and 2295 Hz, respectively. The t+
; and t- commands can be used to fine-tune the space frequency. The
; asynchronous filter is not tunable. These particular choices are
; compatible with the AEA PK232 TNC. Note that the mark and space
; channels can be switched or either one can be disabled.
;
	LRLK	AR2,DELLPF	; set shared IIR delay line pointer
	LRLK	AR1,MSTRUCT	; process mark channel
	LAC	*+
	CALA
	BIT	STATUS,MRKSTA	; is channel disabled
	BBZ	INP33		; branch if no
	ZAC			; yes. kill it
INP33	SACL	MARK		; save mark
	LRLK	AR1,SSTRUCT	; process space channel
	LAC	*+
	CALA
	BIT	STATUS,SPCSTA	; is space channel disabled
	BBZ	INP34		; branch if no
	ZAC			; yes. kill it
INP34	SACL	SPACE		; save space
	LAC	MFPTR		; adjust MF filter pointer
	SBLK	DELMFR
	SUB	MFSIZE
	BLZ	INP32
	LALK	DELMFR
	SACL	MFPTR
INP32	RET

;
; RF output processing
;
; This routine synthesizes the AIO output signal from the FSK waveform
; produced by the baseband output routine.
;
; Variables
; AR7		AIO buffer pointer
; KEY		output shift register (LSB = 1 indicates mark)
; OUTSW		display pointer
; OMARK		AFSK mark phase increment
; OSPACE	AFSK space phase increment
; OPHASE	AFSK output phase
;
; The AFSK mark and space frequencies are determined according to mode
; and tables. In receive mode, provision is made to monitor the AFSK
; output, as well as various signals and state variables selected by the
; d{0-9} command. In transmit mode, the signal generated is always the
; AFSK output. Note that the AFSK output sinewave is attenuated about
; 10dB, in order to match the nominal receiver signal level.
;
; Note: This is the only place where AR7 is incremented. Immediately
; after increment, AR7 is checked for overflow and wrapped.
;
RFOUT	LARP	AR1		; AR1 means business
	BIT	KEY,0		; is lsb bit one
	BBZ	OUT11		; branch if no
	LAC	OMARK		; yes. set mark interval
	B	OUT12
;
OUT11	LAC	OSPACE		; lsb zero. set space interval
OUT12	ADD	OPHASE		; decrement phase
	BGEZ	OUT10		; branch if no underflow
	ADLK	PI2		; underflow. add 2 * pi
OUT10	SACL	OPHASE
	LAC	OPHASE,8	; make table pointer
	SACH	UTMP1
	LALK	SINTAB
	ADD	UTMP1
	TBLR	STS		; convert to sin amplitude
	LALK	LIMIT		; scale for convenient A/B compare
	SACL	TMP
	LT	TMP
	MPY	STS
	PAC
	SACH	STS
	LAC	STS
	BIT	STATUS,CTSSTA	; is this transmit mode
	BBNZ	OUT14		; branch if yes
	LAR	AR1,OUTSW	; no. fetch source signal in q15 format
	LAC	*
OUT14	ANDK    0FFFCh
	LARP	AR7		; AR7 is AIO buffer pointer
	SACL	*+		; store output sample in q15 format
	LRLK	AR0,INTBUF+BUFSIZ ; is pointer at buffer end
	CMPR	
	BBZ	OUT13		; branch if no
	LRLK	AR7,INTBUF	; yes. reset pointer
OUT13	RET

;
; RTTY/SITOR baseband input processing
;
; This routine processes the baseband mark and space signals produced by
; the RF input routine and generates useful signals used by the
; character decode routines. It is called once for each sample decimate,
; which occurs at eight times the baud rate. The signals include the
; carrier and noise estimates, maximum and minimum envelope estimates,
; comparator slice level and autostart signal.
;
; Variables
; POSMAX	max positive signal
; NEGMAX	max negative signal
; SLICE		comparator slice level
; SRVBGN	survivor table
; MSPTR		mark/space memory offset
; DELMRK	mark sample memory
; DELSPC	space sample memory
;
; Determine the mark-space differential signal and save it along with
; the mark and space signals in delay lines. The differential signal is
; used to derive the regenerator output. The mark and space signals are
; used to derive carrier and noise estimates.
;
DSPINP	LARP	AR1		; AR1 means business
	LAC	MARK		; compute differential signal
	SUB	SPACE
	BIT	MODE,REVMOD	; is shift reversed
	BBZ	DSI11		; branch if no
	NEG			; yes. invert signal
DSI11	LRLK	AR1,SRVBGN+SRVSIZ-2 ; point to end of survivor table
	RPTK	SRVSIZ-2	; shift old samples
	DMOV	*-
	MAR	*+
	SACL	*		; insert new sample first
	LAC	MSPTR		; decrement memory offset pointer
	SUBK	1		; is it below lower limit
	BGEZ	DSI12		; branch if no
	ADLK	SRVSIZ		; yes. wrap to upper limit
DSI12	SACL	MSPTR
	LAR	AR0,MSPTR	; get mark/space memory offset
	LRLK	AR1,DELMRK	; insert mark in memory
	MAR	*0+
	LAC	MARK
	SACL	*
	LRLK	AR1,DELSPC	; insert space in memory
	MAR	*0+
	LAC	SPACE
	SACL	*
	CALL	MODEL12		; generate scope sync signal
;
; Compute the maximum and minimum of all survivors. The loop is unrolled
; twice in order to reduce overhead; however, the frequent branches
; effectively stall the pipeline.
;
	LRLK	AR1,SRVBGN	; point to beginning of survivor table
	LRLK	AR0,SRVSIZ+SRVBGN ; point to end of survivor table
	LALK	8000h		; initialize for max search
	SACL	POSMAX
	LALK	03777h		; initialize for min search
	SACL	NEGMAX
DSI21	LAC	*		; is sample > max
	SUB	POSMAX
	BLEZ	DSI22		; branch if no
	ADD	POSMAX		; yes. update max
	SACL	POSMAX
	SAR	AR1,TMP
DSI22	LAC	*+		; is sample < min
	SUB	NEGMAX
	BGEZ	DSI23		; branch if no
	ADD	NEGMAX		; yes. update min
	SACL	NEGMAX
	SAR	AR1,STS
DSI23	LAC	*		; is sample > max
	SUB	POSMAX
	BLEZ	DSI24		; branch if no
	ADD	POSMAX		; yes. update max
	SACL	POSMAX
	SAR	AR1,TMP
DSI24	LAC	*+		; is sample < min
	SUB	NEGMAX
	BGEZ	DSI25		; branch if no
	ADD	NEGMAX		; yes. update min
	SACL	NEGMAX
	SAR	AR1,STS
DSI25	CMPR			; is search complete
	BBZ	DSI21		; branch if no
	LAC	POSMAX		; compute slice level
	ADD	NEGMAX
	SFR
	SACL	SLICE
	RET
;
; This routine is executed for once per bit interval to tease out
; good signal and noise estimates.
;
; Variables
; SMPCTR	receive sample counter
; CARDET	carrier estimator
; NOISE		noise estimator
; CTHRESH	carrier threshold
; CARSRG	carrier shift register
; CARSTR	carrier signal
;
; When the differential signal is most positive, the corresponding mark
; signal is most likely a carrier sample and the space signal a noise
; sample. When the differential signal is most negative, the
; corresponding space signal is most likely a carrier sample and the
; mark signal a noise sample. The carrier estimate is the larger of the
; carrier samples, while the noise estimate is the exponential time
; average of the average of the noise samples. Note that older samples
; correspond to higher addresses in both the survivor table and
; mark/space memories.
;
BITS	LAC	SMPCTR		; is it time for the next bit
	SUBK	1
	SACL	SMPCTR
	BGZ	BTSEND		; branch if no
	LACK	BINSIZ		; yes. restore decimation counter
	SACL	SMPCTR
	LAC	TMP		; compute max mark memory offset
	SBLK	SRVBGN
	ADD	MSPTR
	SBLK	SRVSIZ		; is it above upper limit
	BGEZ	BTS11		; branch if yes (wrap)
	ADLK	SRVSIZ		; no. restore
BTS11	SACL	TMP
	LAR	AR0,TMP		; get mark/space memory offset
	ZAC
	SACL	TMP
	LRLK	AR1,DELMRK	; point to mark memory
	MAR	*0+
	LAC	*		; max mark is carrier
	SACL	CARDET
	LRLK	AR1,DELSPC	; point to space memory
	MAR	*0+
	SUB	*		; is mark greater than space
	BLZ	BTS12		; branch if no
	LAC	*		; yes. space is noise
	SACL	TMP
BTS12	LAC	STS		; compute max space memory offset
	SBLK	SRVBGN
	ADD	MSPTR
	SBLK	SRVSIZ		; is it above upper limit
	BGEZ	BTS13		; branch if yes (wrap)
	ADLK	SRVSIZ		; no. restore
BTS13	SACL	STS
	LAR	AR0,STS		; get mark/space memory offset
	LRLK	AR1,DELSPC	; point to space memory
	MAR	*0+
	BIT	MODE,SIGMOD	; is mark signal gate enabled
	BBZ	BTS17		; branch if no
	BIT	MODE,REVMOD	; yes. is shift upright
	BBNZ	BTS19		; branch if no
	B	BTS14		; yes. mark is carrier
;
BTS17	LAC	CARDET		; is max mark greater than max space
	SUB	*
	BGEZ	BTS14		; branch if yes
BTS19	LAC	*		; no. max space is carrier
	SACL	CARDET
BTS14	LAC	*
	LRLK	AR1,DELMRK	; point to mark memory
	MAR	*0+
	SUB	*		; is space greater than mark
	BLZ	BTS15		; branch if no
	LAC	*,16-1
	ADD	TMP,16-1	; yes. mark is noise
	SACH	TMP
BTS15	ZALR	NOISE		; compute exponential average
	ADD	TMP,CCONST
	SUB	NOISE,CCONST
	SACH	NOISE
;
; If the signal less noise is below the carrier threshold (CTHRESH), a
; zero is; shifted in the carrier register (CARSRG); otherwise, a one is
; shifted. If the number of ones is at least 12 in the last 15, we
; declare the carrier up. If the population of ones in the last 15
; samples falls below eight, we declare the carrier down.
;
	LAC	CTHRESH		; get distance above threshold
	SUB	CARDET
	ADD	NOISE
	ROL			; shift sign bit to register
	LAC	CARSRG
	ROL
	SACL	CARSRG
	LAC	CARSTR		; update ones population count
	BIT	CARSRG,0	; up for LSB
	BBZ	BTS20
	ADLK	LIMIT/8
BTS20	BIT	CARSRG,15	; down for MSB
	BBZ	BTS21
	SBLK	LIMIT/8
BTS21	SACL	CARSTR		; is population at least 12
	SBLK	LIMIT*3/4
	BGZ	BTS22		; branch if yes
	SBLK	LIMIT*2		; no. is population at least eight
	BGZ	BTSEND		; branch if yes
	LALK	~(1<<SIGSTA)	; no. clear carrier bit
	AND	STATUS
	SACL	STATUS
	B	BTSEND
;
BTS22	LALK	1<<SIGSTA	; carrier ok. set carrier bit
	OR	STATUS
	SACL	STATUS
BTSEND	RET

;
; This routine is called by the RTTY and SITOR decoders as each
; character is decoded to manage the signal quality processing. It also
; handles mundane chores like the autostart counter.
;
; Variables
; DIST		max survivor distance
; ATHRESH	autostart threshold
; ASYCTR	start/stop character counter
; AUTCTR	autostart counter
;
; If the signal gate is not open, a zero bit is shifted in the autostart
; register (AUTSRG). The character is valid if the signal gate is open
; and the character distance exceeds the autostart threshold (ATHRESH).
; If so, a one bit is shifted; otherwise, a zero bit is shifted.
; Decoders can use the LSB of the register to suppress printing of
; invalid characters.
;
AUTOS	ZAC			; reset for start/stop convenience
	SACL	ASYCTR
	ZAC			; is carrier ok
	BIT	STATUS,SIGSTA 
	BBZ	AUT14		; branch if no
	LAC	ATHRESH		; yes. get distance above threshold
	SUB	DIST
AUT14	ROL			; shift sign bit to register
	LAC	AUTSRG
	ROL
	SACL	AUTSRG
	LAC	AUTSTR		; update ones population count
	BIT	AUTSRG,0	; up for LSB
	BBZ	AUT11
	ADLK	LIMIT/4
AUT11	BIT	AUTSRG,8	; down for MSB
	BBZ	AUT12
	SBLK	LIMIT/4
AUT12
;
; If the number of one bits in the last eight shifts is at least four,
; the autostart bit is set. If so and the pulse gate is enabled, the
; autostart counter is reset. If the number of one bits is less than
; two, this and allied bits are doused. This results in a maximum sync
; delay of four characters, a maximum hold delay of six characters, and
; a minimum hysteresis of two character.
;
	SACL	AUTSTR		; is population at least four
	SBLK	LIMIT
	BGZ	AUT23		; branch if yes
	SBLK	LIMIT/2		; no. is population at least two
	BGZ	AUT25		; branch if yes
	LALK	~(1<<SYNSTA)	; no. clear bit
	AND	STATUS
	SACL	STATUS
	BIT	MODE,PULMOD	; is pulse gate enabled
	BBZ	AUT22		; branch if no
	LAC	AUTCTR		; is autostart counter alive
	BNZ	AUT25		; branch if yes
AUT22	LALK	~(1<<PULSTA)	; no. clear bit
	AND	STATUS
	SACL	STATUS
	ZAC			; kill autostart counter
	SACL	AUTCTR
	B	AUT25
;
AUT23	LALK	1<<PULSTA	; autostart ok. set autostart bit
	BIT	MODE,SYNMOD	; is sync mode enabled
	BBZ	AUT24		; branch if no
	ORK	1<<SYNSTA	; yes. set sync mode bit
AUT24	OR	STATUS
	SACL	STATUS
	LALK	AHOLD		; yes. freshen autostart counter
	SACL	AUTCTR
AUT25	LAC	AUTCTR		; downdate autostart counter
	SUBK	1
	BLZ	AUTEND		; branch if life ended
	SACL	AUTCTR		; there is life left
AUTEND	RET

;
; RTTY encoder processing
;
; This routine is called at each sample decimate, which occurs at eight
; times the baud rate. It produces a Baudot character with one start bit
; and 1.5 stop bits for a total of 7.5 bits.
;
; Variables
; OUTBCB	buffer structure
; XBTCTR	transmit bit counter
; XSMCTR	transmit sample counter
; KEY		transmit shift register
; RTSCTR	transmit RTS counter
;
; The 16-bit output waveform (KEY) can contain two characters, one each
; in the LSB and MSB bytes. For transmission, the word is shifted left
; one bit to make the start (space) bit of each byte and the high order
; 2 bits of each byte are set to make the stop (mark) bits. The word is
; then right shifted by one bit for each bit of transmission, but
; stopped midway in the eighth bit. The last character sent is forced to
; mark-hold in order to avoid a garble on the transmit/receive switch.
;
DSPOUT	LARP	AR1		; AR1 means business
	LAC	XBTCTR		; is output in progress
	BZ	XMT10		; branch if no
	SUBK	1		; yes. is this the last sample
	SACL	XBTCTR
	BNZ	XMT13		; branch if no
	LAC	RTSCTR		; no. is this RTS delay
	BZ	XMT19		; branch if no
	SUBK	1		; yes. downdate the counter
	SACL	RTSCTR
	B	XMT21
;
XMT19	LAC	KEY		; yes. is another char waiting
	SFR
	SBLK	0FFC0h
	BNZ	XMT12		; branch if yes
XMT10	LRLK	AR1,OUTBCB	; no. is something hiding in the buffer
	CALL	GETBUF
	BNZ	XMT17		; branch if yes
XMT21	BIT	STATUS,CTSSTA	; no. is this transmit mode
	BBZ	XMT16		; branch if no
	BIT	STATUS,RTSSTA	; yes. is RTS enabled
	BBNZ	XMT20		; branch if yes
	CALL	FLUSH		; no. flush buffers
	LALK	00FFh		; send mark-hold
	B	XMT12
;
XMT20	BIT	MODE,SYNMOD	; is sync mode enabled
	BBZ	XMT16		; branch if no
	LACK	SYN		; yes. send diddle (LTRS)
XMT17	CALL	ASC2BDT		; translate to Baudot (maybe two chars)
	SFL			; frame with start and stop bits
XMT12	ORK	0C0C0h
	SACL	KEY
	LACK	(7*BINSIZ)+(BINSIZ/2) ; initialize counters
	SACL	XBTCTR
	LACK	BINSIZ
	SACL	XSMCTR
	B	XMT16
;
XMT13	LAC	XSMCTR		; is this a bit time
	SUBK	1
	BGZ	XMT15		; branch if no
XMT14	LAC	KEY		; yes. get next output bit
	SFR
	SACL	KEY
	LACK	BINSIZ		; reset sample counter
XMT15	SACL	XSMCTR
XMT16	RET

;
; RTTY decoder processing
;
; At this point the last eight bits (64 samples) are in the survivor
; shift register. The oldest bit in the register is the stop bit of the
; previous character, the next oldest bit is the start bit of the
; current character, and the youngest is the stop bit of this character.
; A survivor is defined as the vector of samples beginning in the oldest
; bit and extending through each bit in turn to the youngest bit. There
; are eight survivors. one for each sample in the oldest bit. The
; problem here is to identify whether the oldest sample in the start bit
; begins a character and which of the eight survivors represents the
; most likely transmitted signal.
;
; The exponential average of the max survivor distance is called the
; autostart signal. The program operates in asynchronous mode until it
; locks on a character stream and the autostart signal has increased
; above a threshold (ATHRESH). If so, and if enabled, the start of a
; character is determined by the phase-lock loop.
;
; Variables
; SYNCTR	sync counter
; SYNDEL	frame interval
; BITCTR	receive bit counter
; SRVCTR	receive strobe counter
; XPHASE	phase error signal
; DIST		max survivor distance
; ATHRESH	autostart threshold
;
; Here we test for good start and stop bits and select the survivor with
; maximum distance. The maximum difference between the mark and space
; channels over the last eight bits is called the signal range. The
; midpoint of this range is the slice level. If a survivor sample is
; above the slice level, it is a mark; otherwise, it is a space. The
; interval since the last start bit is called the frame time. In order
; for a character to be recognized, the frame time must be at least
; seven bits. For signals with stop bits greater than one unit, we
; expect the frame time to be that much longer; however, in order to
; affect the frame time estimate used by the phase-lock loop, it must
; not be longer than eight bits.
;
SSTOP	LARP	AR1		; AR1 means business
	LAC	SYNCTR		; increment sync counter
	ADLK	1<<8
	SBLK	(FRMSIZ+BINSIZ)<<8 ; did it overflow
	BLEZ	CYC11		; branch if no
	ZAC			; yes. clamp at max
CYC11	ADLK	(FRMSIZ+BINSIZ)<<8	
	SACL	SYNCTR
	ZAC			; preset sync and distance invalid
	SACL	STS
	LALK	8000h
	SACL	DIST
	BIT	STATUS,SYNSTA	; is this sync mode
	BBZ	CYC12		; branch if no
;
; RTTY synchronous mode. In this mode the program has phase-locked to
; the start-bit transition. We assume that the program has already
; refined the frame time in RTTY asynchronous mode.
;
; In a nominal 75-baud, 10 char/s system, the bit rate is 75 Hz and
; sample rate eight time times this, 600 Hz. The corresponding bit
; interval is 13.33 ms and sample interval 1.667 ms, respectively.
;
; The phase error is proportional to the oldest sample in the start bit.
; This represents a phase detector (PD) gain of 20000 (2*LIMIT) V/(2 pi
; radians). The VCO (SYNDEL) scaling is such that a change of 256
; results in a change of one 1.667-ms sample in one 0.1-s character
; time. This represents a VCO gain of 65.1e-6 (2 pi radians)/(V-s). The
; open-loop gain constant is then the product of the PD and VCO gains,
; 1.302 1/s.
;
; At the smallest frequency resolution of 65.1e-6, the time to cross a
; 1.667-ms sample interval is about 25.6 s, or 256 characters. We use a
; maximum time constant of 256, which is implemented using a right shift
; of eight (SCONST). Smaller time constants must be used during initial
; acquisition. In the optimum (Butterworth) design, the product of the
; time constant, gain constant and gain factor should be 1/2. To make
; this happen, the gain factor should be 1/(2 25.6 521e-6) = .015. We
; approximate this with a gain factor equal to a right shift of 6 bits. ;
; The maximum range of the VCO is limited by the magnitude of the error
; signal. The phase error is normalized in q11 format. To achieve a gain
; factor of 32, the phase error must be shifted by 9 bits to the left.
; This gives 4 bits of precision (signed) in the frame time estimate.
; The maximum frequency deviation in this configuration is 1.053e-3 s/s.
; Therefore, we must rely on the asynchronous mode to reliably fill in
; the 4 bits remaining between the decimal point of the frame time and
; the high-order bit of the phase error.
; 
	LAC	SYNCTR		; sync mode. is sample time too early
	SUB	SYNDEL
	BLZ	CYCEND		; branch if yes
	SACL	SYNCTR		; no. update frame phase
	ZALR	SYNCTR
	ADD	XPHASE,PGAIN
	SACH	SYNCTR
	BIT	STATUS,SIGSTA	; is signal gate enabled
	BBZ	CYC21		; branch if no
	LAC	CTHRESH		; yes. is signal range ok
	SUB	POSMAX
	ADD	NEGMAX
	BGZ	CYC21		; branch if no
	LRLK	AR1,SRVBGN+(7*BINSIZ)-1 ; yes. get oldest sample
	LAC	SLICE
	SUB	*
	SACL	TMP
	ABS			; does it exceed limit
	SBLK	LIMIT/2
	BGZ	CYC21		; branch if yes
	ZALR	XPHASE		; no. update sample phase
	ADD	TMP,SCONST
	SUB	XPHASE,SCONST
	SACH	XPHASE
	LACK	1		; set sync valid
	SACL	STS
	B	CYC21		; continue to select survivor
;
; RTTY asynchronous mode. In this mode characters may arrive at any
; time. The following little bit of paranoia insists that (a) the signal
; gate is enabled, (b) the range is above half threshold, (c) in the
; last eight bit times, the oldest bit (stop bit of the previous
; character) is a mark, (d) the next oldest bit (start bit of the
; current character) is a space, and (e) the youngest bit (stop bit of
; the current character) is a mark. These steps help to resist false
; starts due to impulse noise and fading and, especially, resist garbles
; due to noise.
;
CYC12	LAC	SYNCTR		; async mode. is sample time too early
	SBLK	FRMSIZ<<8
	BLZ	CYCEND		; branch if yes
	BIT	STATUS,SIGSTA	; no. is signal gate enabled
	BBZ	CYCEND		; branch if no
	LAC	CTHRESH		; yes. is signal range ok
	SUB	POSMAX
	ADD	NEGMAX
	BGZ	CYCEND		; branch if no
	LRLK	AR1,SRVBGN+BINSIZ-1 ;yes. is stop bit mark
	LAC	*
	SUB	SLICE
	BLZ	CYCEND		; branch if no
	ADRK	6*BINSIZ	; yes. is start bit space
	LAC	*
	SUB	SLICE
	BGZ	CYCEND		; branch if no
	ADRK	BINSIZ		; yes. is last stop bit mark
	LAC	*
	SUB	SLICE
	BLZ	CYCEND		; branch if no
;
; We measure the intercharacter interval and determine the frame time as
; the exponential average of the interval since the last start bit. The
; intent is to provide a seamless transition to synchronous mode. In
; order for the frame time to be updated, the intercharacter interval
; must fall within a window of four samples centered in the eighth bit.
;
	LAC	SYNCTR		; is sample time in range
	SBLK	(FRMSIZ+2)<<8
	BLZ	CYC13		; branch if no
	SBLK	(BINSIZ/2)<<8
	BGEZ	CYC13		; branch if no
	ZALR	SYNDEL		; yes. update the frame estimate
	ADD	SYNCTR,TCONST
	SUB	SYNDEL,TCONST
	SACH	SYNDEL
	LACK	1		; set sync valid
	SACL	STS
CYC13	ZAC			; set up for the next frame
	SACL	SYNCTR
	SACL	XPHASE
;
; We have found the beginning of a valid character. The phase error is
; the signal at the oldest sample in the start bit. It is exponentially
; averaged to form the error signal used to drive the PLL. We compute
; the distance for each survivor according to the Viterbi algorithm.
; Note that the distance here is the negative of the Viterbi distance,
; in that the distance increases with increasing probability. The loop
; computes the distance for each survivor ending in the stop bit. We
; discard all survivors that do not show mark in the stop bit and space
; in the start bit. The autostart signal is updated by the exponential
; average of the distance. 
;
CYC21	LRLK	AR1,SRVBGN	; initialize for survivor search
	LARK	AR2,BINSIZ-1
CYC22	LAC	*		; is stop bit mark
	SUB	SLICE
	BGEZ	CYC23		; branch if yes
	ADRK	6*BINSIZ	; no. kill survivor
	B	CYC24
;
CYC23	LAC	*,16-2		; compute distance for stop bit
	SUB	NEGMAX,16-2
	SACH	TMP
	ADRK	BINSIZ
	LAC	*,16-2		; add distance for data bit 5
	SUB	SLICE,16-2
	ABS
	ADDH	TMP
	SACH	TMP
	ADRK	BINSIZ
	LAC	*,16-2		; add distance for data bit 4
	SUB	SLICE,16-2
	ABS
	ADDH	TMP
	SACH	TMP
	ADRK	BINSIZ
	LAC	*,16-2		; add distance for data bit 3
	SUB	SLICE,16-2
	ABS
	ADDH	TMP
	SACH	TMP
	ADRK	BINSIZ
	LAC	*,16-2		; add distance for data bit 2
	SUB	SLICE,16-2
	ABS
	ADDH	TMP
	SACH	TMP
	ADRK	BINSIZ
	LAC	*,16-2		; add distance for data bit 1
	SUB	SLICE,16-2
	ABS
	ADDH	TMP
	SACH	TMP
	ADRK	BINSIZ
	LAC	*		; is start bit space
	SUB	SLICE
	BGZ	CYC24		; branch if no
	LAC	POSMAX,16-2	; yes. add distance for start bit
	SUB	*,16-2
	ADDH	TMP
	SUBH	DIST		; is this max survivor
	BLEZ	CYC24		; brunch if no
	ADDH	DIST		; yes. remember it
	SACH	DIST
	SAR	AR1,SRVCTR
CYC24	SBRK	(6*BINSIZ)-1	; point to the next survivor
	LARP	AR2		; AR2 is repeat counter
	BANZ	CYC22,*-,AR1	; AR1 is survivor table pointer
	LAR	AR1,SRVCTR	; restore max survivor pointer
	CALL	AUTOS
	BIT	AUTSRG,0	; is character ok
	BBZ	CYC33		; branch if no
;
; We have found the maximum distance survivor, which allegedly
; represents the bit string of a character. If the pulse gate is enabled
; and the autostart bit is not set, the character is discarded.
; Otherwise, we snip off five bits to form the Baudot character. AR1
; points to the start bit of the max survivor at this point.
;
	BIT	STATUS,PULSTA	; is autostart ok
	BBNZ	CYC32		; branch if yes
	LALK	(1<<PULMOD)+(1<<SIGMOD) ; no. is autostart enabled
	AND	MODE
	BNZ	CYC33		; branch if yes
CYC32	LALK	SRVBGN+(7*BINSIZ) ; yes. initialize strobe counter
	SUB	SRVCTR
	SACL	SRVCTR
	LACK	7		; initialize bit counter
	SACL	BITCTR

	LALK	LIMIT*2		; make char strobe for scope
	SACL	SSYNC

	ZAC			; initialize for survivor scan
	SACL	TMP
	SBRK	BINSIZ*5
	LAC	SLICE		; data bit 5
	SUB	*
	ROL
	LAC	TMP
	ROL
	SACL	TMP
	ADRK	BINSIZ
	LAC	SLICE		; data bit 4
	SUB	*
	ROL
	LAC	TMP
	ROL
	SACL	TMP
	ADRK	BINSIZ
	LAC	SLICE		; data bit 3
	SUB	*
	ROL
	LAC	TMP
	ROL
	SACL	TMP
	ADRK	BINSIZ
	LAC	SLICE		; data bit 2
	SUB	*
	ROL
	LAC	TMP
	ROL
	SACL	TMP
	ADRK	BINSIZ
	LAC	SLICE		; data bit 1
	SUB	*
	ROL
	LAC	TMP
	ROL
	CALL	INCHAR		; process character
CYC33	RET
;
CYCEND	LAC	ASYCTR		; no signal. is it time for distance
	ADDK	1
	SACL	ASYCTR
	SUBK	FRMSIZ+BINSIZ
	BLZ	CYC41		; branch if no
	CALL	AUTOS		; yes. do the ugly
CYC41	RET

;
; SITOR encoder processing
;
; This routine is called at each bit interval of 10 ms. It produces a
; 14-bit CCIR 476 code consisting of a new character followed by an old
; character delayed by five characters. CCIR 476 characters consists of
; four bits one and three bits zero for a total of seven bits.
;
; Variables
; OUTBCB	buffer structure
; FECCHR	encoder buffer
; FECBUF	encoder circular buffer pointer (2)
; XBTCTR	transmit bit counter
; KEY		transmit shift register
; RTSCTR	transmit RTS counter
;
; The implementation uses a three-word shift register, of which the
; least significant 14 bits represent the CCIR codeword. The least
; signficant seven bits of the first word represent a new character,
; while the most significant seven bits represent the repetition of the
; fifth character in the past. The figure shows the progression of the
; sequence "abc", where the least significant seven bits in all but the
; first word are not used.
;
;	 +---+---+     +---+---+     +---+---+     +---+---+
; word 3 | a |   |     | b |   |     | c |   |     |ALF|   |
;	 +---+---+     +---+---+     +---+---+     +---+---+
; word 2 |ALF|   |     | a |   |     | b |   |     | c |   |
;	 +---+---+     +---+---+     +---+---+     +---+---+
; word 1 |ALF| a |     |ALF| b |     | a | c |     | b |RQ |
;	 +---+---+     +---+---+     +---+---+     +---+---+
;
; Note that a 14-bit CCIR codeword is 140 ms in length at 100 baud (10
; ms per bit), while a standard 7.5-bit Baudot character is 150 ms in
; length at 50 baud (20 ms per bit). Thus, an unrestrained CCIR
; transmitter gains 10 ms per character. To avoid overruning the
; receiver, the transmitter inserts five CCIR RQ-ALPHA sync codewords
; after 70 non-sync codewords have been transmitted. This 10.5-s
; sequence continues indefinately as long as characters remain to be
; transmitted.
;
SITOUT	LARP	AR1		; AR1 means business
	LAC	KEY		; shift next bit
	SFR
	SACL	KEY
	LAC	XBTCTR		; is this a char time
	SUBK	1
	SACL	XBTCTR
	BGZ	STXEND		; branch if no
	LAC	RTSCTR		; yes. is this RTS delay
	BZ	STX11		; branch if no
	SUBK	1		; yes. downdate the counter
	SACL	RTSCTR
	B	STX12
;
STX11	LAC	FECCHR,16-8	; is there another char
	SACH	FECCHR
	LAC	FECCHR
	BNZ	STX14		; branch if yes
	LRLK	AR1,OUTBCB	; no. is something hiding in the buffer
	CALL	GETBUF
	BNZ	STX13		; branch if yes
	BIT	STATUS,CTSSTA	; no. is this transmit mode
	BBZ	STX12		; branch if no
	BIT	STATUS,RTSSTA	; yes. is RTS enabled
	BBNZ	STX12		; branch if yes
	CALL	FLUSH		; no. flush buffers
STX12	LALK	RQ		; send sync code
	OR	FECBUF
	SACL	KEY
	LAC	FECBUF+1
	SACL	FECBUF
	LALK	ALPHA<<7
	SACL	FECBUF+1
	B	STX21
;
STX13	CALL	ASC2BDT		; buffer hiding. translate to Baudot
	SACL	FECCHR		; (note there could be two chars here)
STX14	CALL	BDT2CCR		; translate to CCIR
;
; Assemble characters for transmission. Each transmission consists of a
; new character plus an old one saved from five characters previously.
; This code forces a five-character sync sequence every 70 characters,
; which is crude, but effective.
;
	OR	FECBUF		; get previous char for repetition
	SACL	KEY
	LAC	FECBUF+1
	SACL	FECBUF
	LAC	KEY,7		; save current char for repetition
	SACL	FECBUF+1
STX21	LACK	14		; assemble 14 bits
	SACL	XBTCTR
	LAC	FECCTR		; is sync interval complete
	SUBK	1
	SACL	FECCTR
	BGZ	STXEND		; branch if no
	LACK	70		; yes. force a sync sequence
	SACL	FECCTR
	LACK	5
	SACL	RTSCTR
STXEND	RET

;
; SITOR decoder processing
;
; At this point the last eight bits (64 samples) are in the survivor
; shift register in order to estimate the maximum and minimum carrier
; samples, but we need only the first bit (eight samples). The first
; eight samples in the survivor register are used to derive the VCO
; signal for the PLL, which is implemented by modulating the bit sample
; interval. The PLL controls the symbol phase independently of character
; synchronization. Then, the first bit in the survivor shift register is
; corrected for mark/space bias and shifted into a six character (42 bit
; samples) bit vector delay line to be used later.
;
; Variables
; BITCTR	receive bit counter
; SMPCTR	receive strobe counter
; XPHASE	phase error signal
; DIST		max survivor distance
; ATHRESH	autostart threshold
;
; Think of a CCIR 476 transmission as a sequence of 14-bit code vectors,
; each consisting of two seven-bit character. The least significant
; character represents an original transmission, while the most
; sifnigicant one represents a repetition of the character transmitted
; as the least significant one in the second 14-bit word preceeding. In
; the normal case, the original and repeat characters are the same, so
; can be vector summed before correlation. When no characters for
; transmission are available, or in order to provide fill, a special
; synchronization sequence is used consisting of the CCIR 476 characters
; RQ followed by ALPHA; therefore, this sequence must be correlated
; separately.
;
SITOR	LARP	AR1		; AR1 means business
	LAC	SMPCTR		; is this a bit sample
	SUBK	BINSIZ
	BNZ	SITEND		; branch if no
	CNFD			; yes. map bit vector to data memory
	LRLK	AR1,SRVBGN
	LAC	*		; get bias-corrected bit sample
	SUB	SLICE
	LRLK	AR1,TORDLD+TORSIZ-2 ; shift bit vector delay line right
	RPTK	TORSIZ-2 
	DMOV	*-
	MAR	*+
	SACL	*		; insert new bit in bit vector
	CNFP			; map bit vector to program memory
;
; If the signal polarity at the beginning and end of the bit is
; opposite, the midpoint of the bit is likely a good estimate of the
; symbol phase. This is used to update the PLL. We only do this if the
; carrier and signal range are ok.
;
	BIT	STATUS,SIGSTA	; is carrier ok
	BBZ	SIT13		; branch if no
	LAC	CTHRESH		; yes. is signal range ok
	SUB	POSMAX
	ADD	NEGMAX
	BGZ	SIT13		; branch if no
	LRLK	AR1,SRVBGN	; yes. is new bit positive
	LAC	*
	SUB	SLICE
	BLZ	SIT11		; branch if no
	ADRK	BINSIZ		; yes. is last bit also positive 
	LAC	*
	SUB	SLICE
	BGZ	SIT13		; branch if yes
	SBRK	BINSIZ/2	; no. extract midpoint sample
	LAC	*
	SUB	SLICE
	B	SIT12
;
SIT11	ADRK	BINSIZ		; new bit is negative
	LAC	*		; is last bit also negative
	SUB	SLICE
	BLZ	SIT13		; branch if yes
	SBRK	BINSIZ/2	; no. extract midpoint sample
	LAC	SLICE
	SUB	*
SIT12	SACL	TMP

	SACL	PEEK

	ZALR	XPHASE		; update symbol phase
	ADD	TMP,SCONST
	SUB	XPHASE,SCONST
	SACH	XPHASE
SIT13	ZALR	COUNT		; update VCO phase
	SUB	XPHASE,DCONST
	SACH	COUNT
;
; To determine character sync, we correlate the characters in the normal
; and repeat position with the CCIR sequence RQ-ALPHA. The correlation
; function, or distance, is continously measured for each of 14 bit
; positions and compared with the autostart threshold (ATHRESH). A one
; bit is shifted into a register corresponding to each bit position if
; the distance exceeds the threshold. The correct phase has been found
; if (1) the number of one bits in the most recent five bits is at least
; three, (2) the distance is the greatest over each of the other 13 bit
; positions and (3) the bit position has been chosen on the previous
; cycle. This strategy is chosen to maximize the probability of correct
; bit synchronization with the 700-ms restrain sequence (five
; repetitions of RQ-ALPHA), normally transmitted every 70 characters)..
;
; The cyclic autocorrelation function of the CCIR 476 RQ-ALPHA sequence
; is rather nice:
;
; offset 0  1  2  3  4  5  6  7  8  9 10 11 12 13
; value	14  6 -2 -2 -2 -2 -2 -2 -2 -2 -2 -2 -2  6
;
	ZAC			; initialize for correlation
	MPYK	0
	LRLK	AR1,TORTAB+(7*(CCTALF-CCIR))
	RPTK	7-1
	MAC	TORDLP,*+	; look for ALPHA in repeat position
	LRLK	AR1,TORTAB+(7*(CCTRQ-CCIR))
	RPTK	7-1
	MAC	TORDLP+(7*5),*+	; look for RQ in normal position
	APAC
	SACH	STS		; save distance for later
	LAR	AR1,TORPTR	; AR1 is structure pointer
	SACH	*		; (0) distance
	LAC	ATHRESH		; get distance above threshold
	SUB	*+
	ROL			; shift sign bit to register
	LAC	*
	ROL
	SACL	*+		; (1) shift register
	LAC	*-		; update ones population count
	BIT	*,0		; up for LSB
	BBZ	SIT30
	ADDK	1
SIT30	BIT	*+,5		; down for MSB
	BBZ	SIT31
	SUBK	1
SIT31	SACL	*+		; (2) population count
	SBLK	3		; is it at least three
	BLZ	SIT20		; branch if no
	LAC	STS		; yes. is this max distance
	SUB	TORMAX
	BLEZ	SIT20		; branch if no
	ADD	TORMAX		; yes. remember things
	SACL	TORMAX
	LAC	TORPTR		; is it same phase as last time
	SACL	TORTMP
	SUB	TORLST
	BNZ	SIT20		; branch if no
	ZAC			; yes. reset phase
	SACL	BITCTR
SIT20	LRLK	AR0,TORSTR+(3*14) ; is this end of scan
	CMPR
	BBZ	SIT21		; branch if no
	LRLK	AR1,TORSTR	; yes. reset for next scan
	LALK	8000h
	SACL	TORMAX
	LAC	TORTMP
	SACL	TORLST
SIT21	SAR	AR1,TORPTR
	LAC	BITCTR		; is this end of char
	BNZ	SITEND		; branch if no
;
; Character processing. Fiddle with code conversion and distance. Note
; that we have already computed the distance for the RQ-ALPHA sync code.
;
	LALK	LIMIT*2		; make char strobe for scope
	SACL	ACCUM

	LACK	14		; initialize for next char
	SACL	BITCTR
	LACK	BINSIZ
	SACL	SRVCTR
	CALL	CCR2BDT		; convert to Baudot and find distance
	SACL	TMP
	LAC	DIST		; is distance greater than sync distance
	SUB	STS
	BGZ	SIT24		; branch if yes
	LAC	23h		; no. this must be sync
	SACL	TMP
	LAC	STS
	SACL	DIST
SIT24	CALL	AUTOS		; update autostart
	BIT	STATUS,SYNSTA	; is decoder synchronized
	BBZ	SITEND		; branch if no
	BIT	AUTSRG,0	; yes. is this character valid
	BBZ	SITEND		; branch if no
	LAC	TMP		; yes. is this a valid Baudot char
	SUBK	20h
	BGEZ	SITEND		; branch if no
	ADDK	20h		; yes. outfumble the dang thing
	CALL	INCHAR
SITEND	RET

;
; The following two routines implement two different synchronous
; channel filter/detector/lowpass filters. These routines process the
; input filtered and limited signal to produce baseband mark and space
; signals. Each routine is optimized for a different mode. In the
; synchronous filters, the input signal is multiplied by sin/cos
; functions and the I and Q quadrature signals are processed in four
; identical lowpass filters. The baseband signal is then computed as the
; square root of the sum of the squares of the I and Q channels. All
; routines have a common calling sequence and return the basband signal
; in the AC.
;
; Variables
; AR1		channel structure pointer
; AR2		IIR delay line pointer (if needed)
; FLTSIG	input signal
; MFSIZE	size of matched filter (samples * 4)
; MFGAIN	matched filter scale
; GAIN		filter gain
; returns	filtered signal
;
; Matched filter
;
; This routine implements a filter/detector optimized for high
; selectivity. It uses synchronous matched filters tuned to the baud
; interval. A single interleaved shift register is used for the I and Q
; channels and mark and space frequencies.
;
MFILT	LAC	*+		; (-increment)
	ADD	*		; (phase) did pointer underflow
	BGEZ	MFL01		; branch if no
	ADLK	PI2		; yes. adjust pointer
MFL01	SACL	*		; save and advance structure pointer
	LAC	*+,8		; scale for table offset
	SACH	STS
;
; I channel
;
	LAC	STS		; get sin argument
	ADLK	SINTAB
	TBLR	TMP
	LT	TMP		; sin * signal
	MPY	FLTSIG
	LTP	GAIN		; adjust gain (256+-)
	SACH	TMP
	MPY	TMP
	LTP	MFGAIN
	SACH	TMP,7
	LARP	AR0		; AR0 is MF delay line pointer
	LAR	AR0,MFPTR
	LAC	TMP		; add new value
	PUSH
	SUB	*		; subtract old value
	POPD	*+,AR1		; insert new value in mf
	SACL	TMP
	SAR	AR0,MFPTR
	MPY	TMP		; scale for unity gain
	PAC
	ADDH	*+		; add to accumulator
	ADDS	*-
	SACH	*+
	SACL	*+
;
; Q channel
;
	LAC	STS		; get cos argument
	ADLK	30+SINTAB
	TBLR	TMP
	LT	TMP		; cos * signal
	MPY	FLTSIG
	LTP	GAIN		; adjust gain (256+-)
	SACH	TMP
	MPY	TMP
	LTP	MFGAIN
	SACH	TMP,7
	LARP	AR0		; AR0 is MF delay line pointer
	LAR	AR0,MFPTR
	LAC	TMP		; add new value
	PUSH
	SUB	*		; subtract old value
	POPD	*+,AR1		; insert new value in mf
	SACL	TMP
	SAR	AR0,MFPTR
	MPY	TMP		; scale for unity gain
	PAC
	ADDH	*+		; add to accumulator
	ADDS	*-
	SACH	*+
	SACL	*+
	CALL	SQSQR		; square, sum and root
	RET

;
; Butterworth IIR filter
;
; This routine implements a filter/detector filter optimized for quasi-
; linear detector frequency response. It uses synchronous, second-order
; Butterworth IIR lowpass filters. The bandwidth is determined by the
; coefficients loaded in block B0P.
;
IFILT	LAC	*+		; (-increment)
	ADD	*		; (phase) did pointer underflow
	BGEZ	IFL01		; branch if no
	ADLK	PI2		; yes. adjust pointer
IFL01	SACL	*		; save and advance structure pointer
	LAC	*+,8		; scale for table offset
	SACH	STS
;
; I channel
;
	LAC	STS		; get sin argument
	ADLK	SINTAB
	TBLR	TMP
	LT	TMP		; sin * signal
	MPY	FLTSIG
	LTP	GAIN		; adjust gain (128/512+-)
	SACH	TMP
	MPY	TMP
	PAC
	LARP	AR2		; AR2 is IIR delay line pointer
	MAR	*+		; (AR1->z^-1)
	MPYK	0		; initialize sum and product registers
	MACD	LPFCOP,*-	; process a coefficients
	MACD	LPFCOP+1,*
	APAC
	SACH	*,5-1		; normalize
	MPYK	0		; initialize sum and product registers
	ZAC
	RPTK	3-1		; process b coefficients
	MAC	LPFCOP+3,*+
	APAC
	LARP	AR1		; AR1 is structure pointer
	SACH	*+
	SACL	*+
;
; Q channel
;
	LAC	STS		; get cos argument
	ADLK	30+SINTAB
	TBLR	TMP
	LT	TMP		; sin * signal
	MPY	FLTSIG
	LTP	GAIN		; adjust gain (128/512+-)
	SACH	TMP
	MPY	TMP
	PAC
	LARP	AR2		; AR2 is IIR delay line pointer
	MAR	*+		; (AR1->z^-1)
	MPYK	0		; initialize sum and product registers
	MACD	LPFCOP,*-	; process a coefficients
	MACD	LPFCOP+1,*
	APAC
	SACH	*,5-1		; normalize
	MPYK	0		; initialize sum and product registers
	ZAC
	RPTK	3-1		; process b coefficients
	MAC	LPFCOP+3,*+
	APAC
	LARP	AR1		; AR1 is structure pointer
	SACH	*+
	SACL	*+
	CALL	SQSQR		; square, sum and root
	SACL	TMP

;
; Process the baseband signal in a matched filter matched to the baud
; rate.
;
	LARP	AR0		; AR0 is MF delay line pointer
	LAR	AR0,MFPTR
	LAC	TMP
	PUSH
	SUB	*		; subtract old value
	POPD	*,AR1		; insert new value in mf
	SACL	TMP
	LACK	2		; increment filter pointer
	ADD	MFPTR
	SACL	MFPTR
	LT	MFGAIN
	MPY	TMP		; scale for unity gain
	PAC
	ADRK	2		; add to accumulator
	ADDH	*+
	ADDS	*-
	SACH	*+
	SACL	*-
	LAC	*		; return high-order sum
	RET

;
; Subroutine to square, sum and root. AR1 points just past the
; accumulator
;
SQSQR	SBRK	4		; point to high order I channel sum
	MPYK	0		; initialize sum and product registers
	ZAC
	SQRA	*		; I^2
	ADRK	2
	SQRA	*		; Q^2
	ADRK	2
	APAC			; sum and save
	SACH	*+
	SACL	*-
;
; Find square root using Newton-Raphelson (3 iterations). Initial
; estimate for x assumes the signal is at the limit threshold.
;
	LALK	LIMIT		; initial guess
	SACL	TMP
	ZALH	*+		; (c / x) iteration 1
	ADDS	*-
	RPTK 	15-1
	SUBC	TMP
	SACL	STS
	LAC	STS		; (c / x + x)
	ADD	TMP
	SFR			; (1 / 2 * (c / x + x))
	SACL	TMP
	ZALH	*+		; (c / x) iteration 2
	ADDS	*-
	RPTK 	15-1
	SUBC	TMP
	SACL	STS
	LAC	STS		; (c / x + x)
	ADD	TMP
	SFR			; (1 / 2 * (c / x + x))
	SACL	TMP
	ZALH	*+		; (c / x) iteration 3
	ADDS	*-
	RPTK 	15-1
	SUBC	TMP
	SACL	STS
	LAC	STS		; (c / x + x)
	ADD	TMP
	SFR			; (1 / 2 * (c / x + x))
	RET

;
; The following routine implements a crude, table-driven command decoder
; which can be used to select various system parameters, such as baud
; rate, channel spacing, etc. It operates by sampling the UART receive-
; done bit. Someday this will be driven by interrupts; meanwhile, it
; should be called with interrupts disabled. Note that the UART hardware
; parameters are set up by the downloader.
;
; Variables
; INPBCB	input buffer structure
; OUTBCB	output buffer structure
; UCHAR		character temp
; UPTR		pointer temp
; UADR		address temp
; UCMD		command table pointer
; UMOD		saved command table pointer
;
GETCH	LARP	AR1		; AR1 means business
	DINT			; bolt the windows
	LDPK	B_PAGE
	LALK	UART_SEL_LSR	; select line status register
	SACL	UARTI
	OUT	UARTI,UART_CTRL
	IN	UARTI,UART_READ
	LALK	UART_LSR_RBF	; is input buffer full
	AND	UARTI
	BNZ	GET14		; branch if yes
	LDPK	U_PAGE		; no. allow fresh air
	EINT
	RET

GET14	LALK	UART_SEL_RX	; select input buffer
	SACL	UARTI
	OUT	UARTI,UART_CTRL 
	IN	UARTI,UART_READ	; input character
	LAC	UARTI
	LDPK	U_PAGE
	EINT			; allow fresh air
	ANDK	07Fh		; unparity it
	SACL	UTMP2
	SUBK	060h		; uncase it
	BLZ	GET15
	SUBK	020h
GET15	ADDK	060h
	SACL	UTMP1
	SUBK	EOT		; is char EOT
	BNZ	GET90		; branch if no
	CALL	FLUSH		; yes. flush buffers
	B	GET92
;
GET90	SUBK	ACK-EOT		; is char ACK
	BNZ	GET91		; branch if no
GET92	LALK	~(1<<RTSSTA)	; yes. kill RTS
	AND	STATUS
	SACL	STATUS
	B	GET12
;
GET91	BIT	STATUS,CTSSTA	; is this transmit mode
	BBZ	GET95		; branch if no
	LRLK	AR1,OUTBCB	; yes. send char
	LAC	UTMP1
	CALL	PUTBUF
	BIT	MODE,FDXMOD	; is this full duplex
	BBNZ	GET12		; branch if yes
	BIT	MODE,ECHMOD	; no. is digital loopback echo selected
	BBZ	GET94		; branch if no
	LRLK	AR1,INPBCB	; yes. echo char
	LAC	UTMP2
	CALL	PUTBUF
GET94	LAC	UTMP2		; was last char CR
	SUBK	CR
	BNZ	GET12		; branch if no
	LRLK	AR1,OUTBCB	; yes. send LF
	LACK	LF 
	CALL	PUTBUF
	LRLK	AR1,OUTBCB	; yes. send SYN (Baudot LTRS)
	LACK	SYN
	CALL	PUTBUF
	BIT	MODE,ECHMOD	; is digital loopback echo selected
	BBZ	GET12		; branch if no
	LRLK	AR1,INPBCB	; yes. echo LF
	LACK	LF
	CALL	PUTBUF
	B	GET12
;
GET95	LRLK	AR1,INPBCB	; echo char
	LAC	UTMP1
	CALL	PUTBUF
	LAC	UTMP1		; was last char CR
	SUBK	CR
	BNZ	GET13		; branch if no
	LRLK	AR1,INPBCB	; yes. send LF
	LACK	LF 
	CALL	PUTBUF
GET13	LAC	UCMD		; initialize table pointer
	SACL	UPTR
GET10	LAC	UPTR		; get next routine address
	TBLR	UADR
	ADDK	1		; get argument
	TBLR	UTMP2
	ADDK	1		; get next command character
	TBLR	UCHAR
	ADDK	1
	SACL	UPTR
	LAC	UCHAR		; is this last entry
	BZ	GET11		; branch if yes
	SUB	UTMP1		; no. does character match
	BNZ	GET10		; branch if no
GET11	LAC	UADR		; yes. call routine
	CALA
GET12	RET
;
; Housekeeping staff
;
GETPSH	LAC	UCMD		; save command table pointer
	SACL	UMOD
	LAC	UTMP2
	SACL	UCMD
	RET
;
GETPOP	LAC	UMOD		; restore command table pointer
	SACL	UCMD
	B	GET13		; continue in command decode

;
; Following are command execution segments. For compactness, we assume
; the auxiliary register pointer is set at AR1.
;
; Restore to initial defaults
;
RESET	LARP	AR1		; (in case called directly)
	ZAC			; initialize things
	LRLK	AR1,B1D		; clear block B1 (0300-03ff)
	RPTK	100h-1
	SACL	*+
	LRLK	AR1,USRBGN	; clear external memory (0401-)
	LRLK	AR2,USREND-USRBGN-1
RST10	SACL	*+
	LARP	AR2		; AR2 is repeat counter
	BANZ	RST10,*-,AR1
	LALK	ST00		; initialize command state machine
	SACL	UCMD
	LALK	((BINSIZ/2)+FRMSIZ)<<8 ; set frame interval
	SACL	SYNDEL
	LALK	ALIMIT		; set autostart threshold
	SACL	ATHRESH
	LALK	CLIMIT		; set carrier threshold
	SACL	CTHRESH
	LACK	5		; set limiter gain
	SACL	LGAIN
	LALK	0FFFFh		; set output mark-hold
	SACL	KEY
	LALK	U_ADDR+STS	; select monitoring point
	SACL	OUTSW
	LALK	BAUD2		; select 75 baud
	CALL	CMDB1
	LALK	SHIFT2		; select 600 Hz shift, IIR filter
	CALL	CMDM1
	CALL	TINIT		; build SITOR tables
	RET

;
; Select SITOR mode
;
CMDY	LALK	BAUD1		; select 100 baud
	CALL	CMDB1
	LALK	SHIFT8		; select 170 Hz shift, matched filter
	CALL	CMDM1
	LALK	(1<<TORMOD)+(1<<SYNMOD) ; enable SITOR Mode B
	OR	MODE
	SACL	MODE
	RET

;
; Select mode, filter and shift m{1-8}
;
CMDM	LAC	UTMP2		; get initialization table pointer
CMDM1	LRLK	AR1,MSTRUCT
	TBLR	*+		; set mark filter routine
	ADDK	1
	TBLR	*		; set mark filter frequency
	ADDK	1
	LRLK	AR1,SSTRUCT
	TBLR	*+		; set space filter routine
	ADDK	1
	TBLR	*		; set space filter frequency
	ADDK	1
	TBLR	MODE		; get mode
	ADDK	1		; set filter gain
	TBLR	GAIN
	CALL	CMDO0		; set output shift
	CNFD			; map B0 to data memory
	LRLK	AR1,B0D
	BIT	MODE,NARMOD	; is this narrowband filter
	BBNZ	CMDM2		; branch if yes
	RPTK	FIRSIZ-1	; no. use wideband coefficients
	BLKP	BPF850,*+
	RPTK	6-1
	BLKP	IIR1,*+
	CNFP			; map B0 to program memory
	B	CMD55
;
CMDM2	RPTK	FIRSIZ-1	; use narrowband coefficients
	BLKP	BPF170,*+
	RPTK	6-1
	BLKP	IIR1,*+
	CNFP			; map B0 to program memory
	B	CMD55
;
; Adjust space frequency m{+-}
;
CMDSP	LALK	-TUNEX		; m+ increase frequency
	B	CMD22

CMDSN	LALK	TUNEX		; m- reduce frequency
CMD22	LRLK	AR1,SSTRUCT+1
	ADD	*
	SACL	*
	RET
;
; Select baud rate b{1-4}
;
CMDB	LAC	UTMP2		; get initialization table pointer
CMDB1	TBLR	RANGE		; AIO samples
	ADDK	1
	TBLR	MFSIZE		; MF size
	ADDK	1
	TBLR	MFGAIN		; MF gain
CMD55	ZAC
	LRLK	AR1,MSTRUCT+2	; clear mark filter structure
	RPTK	STRSIZ-2-1
	SACL	*+
	LRLK	AR1,SSTRUCT+2	; clear space filter structure
	RPTK	STRSIZ-2-1
	SACL	*+
	LRLK	AR1,DELLPF	; clear IIR delay line
	RPTK	12-1
	SACL	*
	LRLK	AR1,DELMFR	; clear MF delay line
	LRLK	AR2,MFRSIZ-1
CMD56	SACL	*+
	LARP	AR2		; AR2 is repeat counter
	BANZ	CMD56,*-,AR1	; AR1 is MF delay line pointer
	LALK	DELMFR		; leave MF pointer in handy place
	SACL	MFPTR
	CALL	FLUSH		; flush buffers
	RET
;
; Adjust baud rate b{+-}
;
CMDBP	LAC	RANGE		; b+ increase baud interval
	ADDK	1
	SACL	RANGE
	RET
;
CMDBN	LAC	RANGE		; b- decrease baud interval
	SUBK	1
	SACL	RANGE
	RET
;
; Select output shift o{0-2}
;
CMDO0	BIT	MODE,NARMOD	; o0 is input narrow shift
	BBZ	CMDO2		; branch if no
CMDO1	LALK	-(2125*FAC)	; o1 yes. select narrow shift
	SACL	OMARK
	LALK	-(2295*FAC)
	SACL	OSPACE
	RET
;
CMDO2	LALK	-(1275*FAC)	; o2 select wide shift
	SACL	OMARK
	LALK	-(2125*FAC)
	SACL	OSPACE
	RET
;
; Select limiter gain l{0-5}
;
CMDL	LAC	UTMP2		; select limiter gain
	SACL	LGAIN
	RET
;
; Tune radio t{+-}
;
CMDTP	LALK	FREQ_UP_R1+FREQ_UP_R2 ; t+
	B	CMDT1
;
CMDTN	LALK	FREQ_DN_R1+FREQ_DN_R2 ; t-
CMDT1	SACL	TUNE
	LALK	TUNEW
	SACL	TUNCTR
	RET
;
; Adjust filter gain g{+-}
;
CMDGP	ZALR	GAIN		; g+ decrease gain
	SUB	GAIN,16-5
	SACH	GAIN
	RET
;
CMDGN	ZALR	GAIN		; g- increase gain
	ADD	GAIN,16-5
	SACH	GAIN
	RET
;
; Select autostart gate a{0-2}
;
CMDA	LALK	~((1<<PULMOD)+(1<<SIGMOD)) ; dim tired bits
	AND	MODE
	OR	UTMP2		; light fresh bits
	SACL	MODE
	ZAC			; kill autostart counter
	SACL	AUTCTR
	RET
;
; Adjust autostart threshold a{+-}
;
CMDAP	ZALR	ATHRESH		; a+ decrease autostart threshold
	SUB	ATHRESH,16-4
	SACH	ATHRESH
	RET
;
CMDAN	ZALR	ATHRESH		; a- increase autostart threshold
	ADD	ATHRESH,16-4
	SACH	ATHRESH
	RET
;
; Select channel c{0-2}
;
CMDC	LALK	~((1<<MRKSTA)+(1<<SPCSTA)) ; dim tired bits
	AND	STATUS
	OR	UTMP2		; light fresh bits
	SACL	STATUS
	RET
;
; Adjust carrier threshold c{+-}
;
CMDCP	ZALR	CTHRESH		; c+ decrease carrier threshold
	SUB	CTHRESH,16-4
	SACH	CTHRESH
	RET
;
CMDCN	ZALR	CTHRESH		; c- increase carrier threshold
	ADD	CTHRESH,16-4
	SACH	CTHRESH
	RET
;
; Select display/output d{0-9}
;
CMDD	LAC	UTMP2		; select monitor point
	SACL	OUTSW
	RET
;
; Select analog/digital loopback echo e{1-3}
;
CMDE	LALK	~((1<<FDXMOD)+(1<<ECHMOD)) ; rattle status bits
	AND	MODE
	OR	UTMP2
	SACL	MODE
	RET
;
; Select radio port p{1-2}
;
CMDP1	LALK	~(1<<PORMOD)	; p1 radio port 1
	AND	MODE
	SACL	MODE
	RET
;
CMDP2	LALK	1<<PORMOD	; p2 radio port 2
	OR	MODE
	SACL	MODE
	RET
;
; Toggle bit s, i
;
CMDTOG	LALK	~(1<<SYNSTA)	; kill sync mode
	AND	STATUS
	SACL	STATUS
	LAC	MODE		; toggle mode bit
	XOR	UTMP2
	SACL	MODE
	RET
;
; Begin transmission x
;
CMDX	CALL	FLUSH		; flush buffers
	LALK	PTT_R1+PTT_R2	; transmit mode
	SACL	TUNE
	LACK	RHOLD		; initialize RTS delay
	SACL	RTSCTR
	LALK	(1<<RTSSTA)+(1<<CTSSTA) ; set transmit mode
	OR	STATUS
	SACL	STATUS
	RET

;
; UART output watcher
;
; Variables
; UARTO		UART output buffer
; INPBCB	input buffer structure
; 
PUTCH	LARP	AR1		; AR1 means business
	DINT			; bolt the windows
	LDPK	B_PAGE
	LALK	UART_SEL_MCR	; select modem control register
	SACL	XTMP
	OUT	XTMP,UART_CTRL	; output modem control
	LAC	UARTC
	OUT	XTMP,UART_WRITE
	LALK	UART_SEL_LSR	; select line status register
	SACL	XTMP
	OUT	XTMP,UART_CTRL
	IN	XTMP,UART_READ	; is output buffer empty
	LACK	UART_LSR_THRE
	AND	XTMP
	BZ	URX10		; branch if no
	LALK	UART_LSR_THRE	; yes. select output buffer
	SACL	XTMP
	OUT	XTMP,UART_CTRL
	LDPK	U_PAGE
	EINT
	LRLK	AR1,INPBCB	; are characters hiding in the buffer
	CALL	GETBUF
	DINT
	LDPK	B_PAGE
	BZ	URX10		; branch if no
	SACL	UARTO		; yes. output character
	OUT	UARTO,UART_WRITE
URX10	LDPK	U_PAGE
	EINT			; allow fresh air
	RET

;
; Input character processing. We stash the character in the input
; buffer, unless transmitting in digital loopback echo mode. We stash
; the character in the output buffer only if receiving in half-duplex
; modes.
;
INCHAR	SACL	TMP		; translate Baudot char to ASCII
	CALL	BDT2ASC
	BZ	INCEND		; suppress nonprint characters
	SACL	TMP
	BIT	STATUS,CTSSTA	; is this digital loopback echo
	BBNZ	INC12		; branch if no
	BIT	MODE,FDXMOD
	BBZ	INC12		; branch if no
	BIT	MODE,ECHMOD
	BBNZ	INC11		; branch if yes
INC12	LRLK	AR1,INPBCB	; no. copy to input buffer
	LAC	TMP
	CALL	PUTBUF
INC11	BIT	STATUS,CTSSTA	; is this half-duplex receive
	BBNZ	INCEND		; branch if no
	BIT	MODE,FDXMOD
	BBNZ	INCEND		; branch if no
	LRLK	AR1,OUTBCB	; yes. copy to output buffer
	LAC	TMP
	CALL	PUTBUF
INCEND	RET			; declare victory

;
; Routine to convert Baudot to ASCII
;
; On entry, AC contains a 5-bit Baudot code. On exit, AC contains the 7-
; bit ASCII code or zero if no corresponding ASCII code exists.
;
BDT2ASC	SUBK	FIGS		; is char FIGS
	BNZ	BDT83		; branch if no
	LALK	1<<RSHSTA	; yes. set shift bit
	OR	STATUS
	SACL	STATUS
	ZAC
	RET
;
BDT83	SUBK	LTRS-FIGS	; is char LTRS
	BNZ	BDT84		; branch if no
	LALK	~(1<<RSHSTA)	; yes. clear shift bit
	AND	STATUS
	SACL	STATUS
	ZAC
	RET
;
BDT84	ADDK	LTRS		; restore char
	ADLK	BAUDOT		; translate Baudot to ASCII
	TBLR	TMP
	BIT	STATUS,RSHSTA	; is this LTRS shift
	BBNZ	BDT85		; branch if no
	LAC	TMP,16-8	; yes. use MSB
	SACH	TMP
BDT85	LAC	TMP
	ANDK	7Fh		; mask off junque
	RET

;
; Routine to convert ASCII to Baudot
;
; On entry, AC contains a 7-bit ASCII code. On exit, AC contains the 5-
; bit Baudot code and LTRS/FIGS shift code, if needed, as a 16-bit word.
; AC is zero if no corresponding Baudot code exists.
;
ASC2BDT	ANDK	07Fh		; unparity char
	SUBK	060h		; uncase char
	BLZ	ASC11
	SUBK	020h
ASC11	ADDK	060h
	SACL	STS
	SFR			; translate to Baudot
	ADLK	ASCII
	TBLR	TMP
	BIT	STS,0		; is this MSB 
	BBNZ	ASC13		; branch if no
	LAC	TMP,16-8	; yes. shift MSB to LSB
	SACH	TMP
ASC13	LAC	TMP		; mask off MSB bits
	ANDK	01Fh
	SACL	STS
	BIT	STATUS,XSHSTA	; is print in FIGS shift
	BBZ	ASC15		; branch if no (LTRS shift)
	BIT	TMP,LCTST	; yes. does char need LTRS shift
	BBZ	ASC17		; branch if no
	LALK	~(1<<XSHSTA)	; yes. set LTRS shift
	AND	STATUS
	SACL	STATUS
	LAC	STS,8		; send LTRS before char
	ORK	LTRS
	B	ASC17
;
ASC15	BIT	TMP,UCTST	; LTRS shift. does char need FIGS shift
	BBZ	ASC17		; branch if no
	LALK	1<<XSHSTA	; yes. set FIGS shift
	OR	STATUS
	SACL	STATUS
	LAC	STS,8		; send FIGS before char
	ORK	FIGS
ASC17	RET

;
; Routine to put a word in a circular buffer
;
; On entry, AR1 points to the buffer structure and the data word is in
; AC. Zero words are not stored. ARP must point to AR1.
;
PUTBUF	BZ	BUF10		; brook no zeros
	LAR	AR0,*+		; get upper limit
	MAR	*+		; get put pointer
	LAR	AR2,*,AR2
	MAR	*+		; increment put pointer
	CMPR			; is it at upper limit
	LARP	AR1
	BBZ	BUF11		; branch if no
	MAR	*-		; yes. wrap to lower limit
	LAR	AR2,*+
BUF11	SAR	AR2,*,AR2	; save put pointer
	SACL	*,,AR1		; put char in circular buffer
BUF10	RET

;
; Routine to get a word from a circular buffer
;
; On entry, AR1 points to the buffer structure. On exit, the word is in
; AC or zero if the buffer is empty. ARP must point to AR1.
;
GETBUF	LAR	AR0,*+		; get upper limit
	MAR	*+		; get put pointer
	LAC	*+
	SUB	*		; get get pointer
	BZ	BUF20		; branch if equal (buffer empty)
	LAR	AR2,*,AR2	; unequal. get get pointer
	MAR	*+		; increment get pointer
	CMPR			; is it at upper limit
	LARP	AR1
	BBZ	BUF21		; branch if no
	SBRK	2		; yes. wrap to lower limit
	LAR	AR2,*
	ADRK	2
BUF21	SAR	AR2,*,AR2	; save get pointer
	LAC	*,,AR1		; get char from circular buffer
BUF20	RET

;
; Routine to flush buffers for initialization and switching between
; transmit and receive. ARP must point to AR1.
;
FLUSH	LRLK	AR1,INPBCB	; preset input buffer structure
	LALK	INPBUF+OUTSIZ
	SACL	*+
	LALK	INPBUF
	SACL	*+
	SACL	*+
	SACL	*+
	LRLK	AR1,OUTBCB	; preset output buffer structure
	LALK	OUTBUF+OUTSIZ
	SACL	*+
	LALK	OUTBUF
	SACL	*+
	SACL	*+
	SACL	*+
	LALK	~((1<<RTSSTA)+(1<<CTSSTA)+(1<<RSHSTA)+(1<<XSHSTA))
	AND	STATUS		; kill shift and transmit bits
	SACL	STATUS
	ZAC
	SACL	CARDET		; reset carrier and noise integrators
	SACL	NOISE
	SACL	AUTSRG		; reset autostart
	SACL	AUTSTR
	SACL	TUNE		; kill radio control bits
	LALK	TORSTR		; initialize SITOR decoder buffer
	SACL	TORPTR
	LALK	ALPHA<<7
	SACL	FECBUF
	SACL	FECBUF+1
	RET

;
; Routine to convert CCIR 476 code to Baudot character
;
; The CCIR 476 code vector developed by the baseband processing routine
; is stored in program memory in the coefficient page, while the
; correlation coefficients are stored in data memory. The routine
; correlates the seven-sample code vector plus the seven-sample repeat
; code vector (delayed five characters) with each of the 35 possible
; CCIR 476 codes and selects the one with the greatest correlation
; function. The corresponding Baudot character is returned in the AC.
; The routine returns in the AC 00h-1Fh for Baudot, 20h-22h for CCIR
; codes with no Baudot equivalent.
;
; Variables
; TORDLD/P	received bit vector (six seven-bit characters)
; CHRDLD/P	vector sum of first and sixth characters
; TORTAB	correlation coefficients for 35 seven-bit characters
;
CCR2BDT	LARP	AR1		; initialize
	CNFD			; map coefficient page to data memory
	LRLK	AR0,7		; initialize for sum
	LRLK	AR1,TORDLD
	LRLK	AR2,TORDLD+(7*5)
	LRLK	AR3,CHRDLD
CCR11	LAC	*+,,AR2		; sum original and repeat vectors
	ADD	*+,,AR3
	SACL	*+
	LARP	AR0
	BANZ	CCR11,*-,AR1	; repeat until done
	CNFP			; map coefficient page to program memory
	LRLK	AR1,TORTAB	; initialize for correlation
	LRLK	AR0,35-1
	LALK	8000h
	SACL	DIST
CCR12	ZAC			; correlate bit vector and weight vector
	MPYK	0
	RPTK	7-1
	MAC	CHRDLP,*+	; multiply and add
	APAC			; accumulate final product
	SUBH	DIST		; is it greatest
	BLZ	CCR13		; branch if no
	ADDH	DIST		; yes. remember things
	SACH	DIST
	SAR	AR0,TMP
CCR13	LARP	AR0
	BANZ	CCR12,*-,AR1	; repeat for 35 vectors
	LACK	35-1		; figure (modified) Baudot char
	SUB	TMP
	RET

;
; Routine to convert Baudot character to CCIR 476 code
;
; The Baudot character is in the AC on entry, the CCIR 476 code is in
; the AC on exit.
;
BDT2CCR	ANDK	1Fh		; determine table address
	ADLK	CCIR
	TBLR	TMP		; get CCIR character
	LAC	TMP
	RET

;
; Routine to construct CCIR 476 decoding coefficient table
;
; The table has 35 vectors of seven words each, in order as the Baudot
; character equivalent, plus three special characters. It is constructed
; from the CCIR table, where each zero bit corresponds to a negative
; coefficient and each one bit corresponds to a positive coefficient.
;
TINIT	LARP	AR1		; AR1 points to coefficient table
	LRLK	AR1,TORTAB+(35*7)-1
	LALK	CCIR+35-1
	SACL	STS
TIN13	LAC	STS		; get code vector
	TBLR	TMP
	SUBK	1
	SACL	STS
	LRLK	AR0,7-1		; loop over seven bits
TIN12	LAC	TMP		; shift code right
	ROR
	SACL	TMP
	LALK	32768/7		; save signed coefficient
	BC	TIN11
	NEG
TIN11	SACL	*-
	LARP	AR0
	BANZ	TIN12,*-,AR1	; continue for seven coefficients
	LAC	STS		; is this last code
	SBLK	CCIR
	BGEZ	TIN13		; branch if no	
	RET			; yes. done here

;
; Subroutine to dazzle LEDs. This runs only when nothing else is waiting
; for processing. It blinks the status LEDs and radio control lines.
;
LIGHTS	LDPK	B_PAGE		; go where the action is
	LALK	~(LED_205+LED_206+LED_207) ; LEDs 5-7 off
	AND	TNC_BUF
	LDPK	U_PAGE
	SACL	UTMP2
	LAC	TUNE		; assemble RADIO_CTRL port bits
	BIT	MODE,PORMOD	; is this port 2
	BBNZ	MAN01		; branch if yes
	ANDK	P1_MSK		; no. enable port 1
	B	MAN01A
;
MAN01	ANDK	P2_MSK		; enable port 2
MAN01A	BIT	STATUS,SIGSTA	; is signal gate enabled
	BBZ	MAN02		; branch if no
	ORK	LED_203		; yes. LED 3 on
MAN02	BIT	STATUS,PULSTA	; is autostart ok
	BBZ	MAN03		; branch if no
	BIT	STATUS,SYNSTA	; yes. is this sync mode
	BBNZ	MAN03A		; branch if yes
	BIT	KEY,0		; no. is output space
	BBNZ	MAN03		; branch if no
MAN03A	ORK	LED_204		; yes. LED 4 on
MAN03	SACL	UTMP1
	LALK	LIMIT		; is mark greater than threshold
	SUB	MARK
	BGZ	MAN04		; branch if no
	LALK	LED_205		; yes. LED 5 on
	OR	UTMP2
	SACL	UTMP2
MAN04	LALK	LIMIT		; is space greater than threshold
	SUB	SPACE
	BGZ	MAN05		; branch if no
	LALK	LED_206		; yes. LED 6 on
	OR	UTMP2
	SACL	UTMP2
MAN05	BIT	KEY,0		; is this mark
	BBZ	MAN06		; branch if no
	LALK	LED_207		; yes. LED 7 on
	OR	UTMP2
	SACL	UTMP2
MAN06	LAC	UTMP2
	LDPK	B_PAGE
	SACL	TNC_BUF
	OUT	TNC_BUF,TNC_OUTPUT ; LEDs 5-7
	LDPK	U_PAGE
	LAC	UTMP1
	LDPK	B_PAGE
	SACL	PDC_BUF
	OUT     PDC_BUF,RADIO_CTRL ; LEDs 3-4
	LDPK	U_PAGE
	RET

;
; Routine to generate scope sync signals. The ACCUM signal can be
; monitored on a DC-coupled scope connected to the AIO output using the
; d5 command. This is a very useful tuning indicator and handy for
; performance assessment. At the speaker output, the AIO output signal
; sounds something like a Teletype Model 12 when piped to a speaker.
;
MODEL12	LARP	AR1
	LAC	BITCTR		; is input in progress
	BZ	MOD11		; branch if no
	LAC	SRVCTR		; yes. is this a bit time
	SUBK	1
	BGZ	MOD10		; branch if no
	LAC	BITCTR		; yes. update bit counter
	SUBK	1
	SACL	BITCTR
	LALK	-LIMIT/2	; make bit strobe
	SACL	SSYNC
	LACK	BINSIZ		; update sample counter
MOD10	SACL	SRVCTR
MOD11	LRLK	AR1,SRVBGN	; point to youngest bit for display
	BIT	MODE,TORMOD	; is this SITOR
	BBNZ	MOD12		; branch if yes
	ADRK	7*BINSIZ	; no. point to oldest bit
MOD12	LAC	*
	ADD	SSYNC
	SACL	ACCUM
	ZAC
	SACL	SSYNC
	RET

;
; Turn selected LED on and select AIO input
;
LEDON	LDPK	U_PAGE
	OR	PIO_BUF		; set LED bit
	B	LED10

;
; Turn selected LED off and select AIO input
;
LEDOFF	CMPL			; clear LED bit
	AND	PIO_BUF
LED10	ANDK	0C0h		; clear mux bits
	BIT	STATUS,CTSSTA	; is this transmit mode
	BBZ	LED12		; branch if no
	BIT	MODE,FDXMOD	; yes. is this full duplex
	BBNZ	LED12		; branch if yes
	ORK	AIO_ENABLE+GAIN0 ; no select transmitter monitor
	BIT	MODE,PORMOD
	BBNZ	LED11
	ORK	XMIT_1		; port 1
	B	LED14
;
LED11	ORK	XMIT_2		; port 2
	B	LED14
;
LED12	ORK	AIO_ENABLE+GAIN4 ; select receiver monitor
	BIT	MODE,PORMOD
	BBNZ	LED13
	ORK	REC_IN_1	; port 1
	B	LED14
;
LED13	ORK	REC_IN_2	; port 2
LED14	SACL	PIO_BUF
	OUT	PIO_BUF,RADIO_GAIN ; (U_PAGE must be 8 for hardware)
	RET

;
; Wait loops
;
WAIT4	LARK	AR2,0FFh
	CALL	WAIT2
	CALL	WAIT2
	RET

WAIT2	LARK	AR2,0FFh
	CALL	WAIT1
	CALL	WAIT1
	RET

WAIT1	LARK	AR2,0FFh
WAIT_A	LARK	AR3,0FFh
WAIT_B	LARP	AR3
	BANZ	WAIT_B		; Loop through count
	LARP	AR2
	BANZ	WAIT_A		; Loop through count
	RET

	.MSFIRST

;
; Command tables
;
ST00	.WORD	GETPSH,ST40,'A'	; adjust autostart threshold and gate
	.WORD	GETPSH,ST20,'B'	; select baud rate
	.WORD	GETPSH,ST41,'C'	; adjust carrier threshold and channel
	.WORD	GETPSH,ST10,'D'	; select display/output
	.WORD	GETPSH,ST60,'E' ; digital/analog echo
	.WORD	GETPSH,ST51,'G'	; adjust filter gain
	.WORD	CMDTOG,1<<REVMOD,'I' ; reverse shift (toggle)
	.WORD	GETPSH,ST30,'L'	; select limiter gain
	.WORD	GETPSH,ST70,'M'	; select mode, filter and shift
	.WORD	GETPSH,ST15,'O'	; select output shift
	.WORD	GETPSH,ST25,'P'	; select radio port
	.WORD	RESET,0,'R'	; master reset
	.WORD	CMDTOG,1<<SYNMOD,'S' ; sync enable (toggle)
	.WORD	GETPSH,ST50,'T'	; tune radio
	.WORD	CMDX,0,'X'	; begin transmission
	.WORD	CMDY,0,'Y'	; enable SITOR mode B (FEC)
	.WORD	GET12,0,0	; default
;
; Select baud rate b{+-1-6}
;
ST20	.WORD	CMDB,BAUD1,'1'	; 100 baud
	.WORD	CMDB,BAUD2,'2'	; 75 baud
	.WORD	CMDB,BAUD3,'3'	; 50 baud
	.WORD	CMDB,BAUD4,'4'	; 45.45 baud
	.WORD	CMDB,BAUD5,'5'	; 25 baud
	.WORD	CMDB,BAUD6,'6'	; 10 baud
	.WORD	CMDBP,0,'+'	; increase
	.WORD	CMDBP,0,'='	; increase (LC)
	.WORD	CMDBN,0,'-'	; decrease
	.WORD	CMDBN,0,'_'	; decrease (UC)
	.WORD	GETPOP,0,0	; default
;
; Select mode, filter and shift m{+-1-8}
;
ST70	.WORD	CMDM,SHIFT1,'1'	; IIR filter, 850 Hz shift
	.WORD	CMDM,SHIFT2,'2'	; IIR filter, 600 Hz shift
	.WORD	CMDM,SHIFT3,'3'	; IIR filter, 400 Hz shift
	.WORD	CMDM,SHIFT4,'4'	; IIR filter, 170 Hz shift
	.WORD	CMDM,SHIFT5,'5'	; MF filter, 850 Hz shift
	.WORD	CMDM,SHIFT6,'6'	; MF filter, 600 Hz shift
	.WORD	CMDM,SHIFT7,'7'	; MF filter, 400 Hz shift
	.WORD	CMDM,SHIFT8,'8'	; MF filter, 170 Hz shift
	.WORD	CMDSP,0,'+'	; increase
	.WORD	CMDSP,0,'='	; increase (LC)
	.WORD	CMDSN,0,'-'	; decrease
	.WORD	CMDSN,0,'_'	; decrease (UC)
	.WORD	GETPOP,0,0	; default
;
; Select limiter gain l{0-5}
;
ST30	.WORD	CMDL,0,'0'	; 0dB
	.WORD	CMDL,1,'1'	; 6dB
	.WORD	CMDL,2,'2'	; 12dB
	.WORD	CMDL,3,'3'	; 18dB
	.WORD	CMDL,4,'4'	; 24dB
	.WORD	CMDL,5,'5'	; 30dB
	.WORD	GETPOP,0,0	; defaultt
;
; Select display (output) d<0-9>
;
ST10	.WORD	CMDD,U_ADDR+STS,'0' ; AFSK generator
	.WORD	CMDD,U_ADDR+INPSIG,'1' ; AIO input signal
	.WORD	CMDD,U_ADDR+FLTSIG,'2' ; filtered/limited input signal
	.WORD	CMDD,U_ADDR+MARK,'3' ; mark signal
	.WORD	CMDD,U_ADDR+SPACE,'4' ; space signal
	.WORD	CMDD,U_ADDR+ACCUM,'5' ; detector output
	.WORD	CMDD,U_ADDR+DIST,'6' ; distance signal
	.WORD	CMDD,U_ADDR+AUTSTR,'7' ; autostart signal
	.WORD	CMDD,U_ADDR+NOISE,'8' ; noise signal
	.WORD	CMDD,U_ADDR+CARDET,'9' ; carrier signal
	.WORD	CMDD,U_ADDR+XPHASE,'-' ; VCO signal (UC)
	.WORD	CMDD,U_ADDR+XPHASE,'_' ; VCO signal
	.WORD	CMDD,U_ADDR+PEEK,'+' ; peekaboo
	.WORD	CMDD,U_ADDR+PEEK,'=' ; peekaboo (LC)
	.WORD	GETPOP,0,0	; default
;
; Select analog/digital loopback echo e{1-3}
;
ST60	.WORD	CMDE,0,'1'	; analog loopback echo
	.WORD	CMDE,1<<ECHMOD,'2' ; digital loopback echo
	.WORD	CMDE,1<<FDXMOD,'3' ; full duplex
	.WORD	GETPOP,0,0	; default
;
; Select radio port {p1-2}
;
ST25	.WORD	CMDP1,0,'1'	; radio port 1
	.WORD	CMDP2,0,'2'	; radio port 2
	.WORD	GETPOP,0,0	; default
;
; Select output shift {o0-2}
;
ST15	.WORD	CMDO0,0,'0'	; follow input shift
	.WORD	CMDO1,0,'1'	; 170-Hz shift
	.WORD	CMDO2,0,'2'	; 850-Hz shift
	.WORD	GETPOP,0,0	; default
;
; Tune radio t{+-}
;
ST50	.WORD	CMDTP,0,'+'	; increase
	.WORD	CMDTP,0,'='	; increase (LC)
	.WORD	CMDTN,0,'-'	; decrease
	.WORD	CMDTN,0,'_'	; decrease (UC)
	.WORD	GETPOP,0,0	; default
;
; Adjust filter gain g{+-}
;
ST51	.WORD	CMDGN,0,'+'	; increase
	.WORD	CMDGN,0,'='	; increase (LC)
	.WORD	CMDGP,0,'-'	; decrease
	.WORD	CMDGP,0,'_'	; decrease (UC)
	.WORD	GETPOP,0,0	; default
;
; Adjust autostart threshold a{+-01}
;
ST40	.WORD	CMDAN,0,'+'	; increase
	.WORD	CMDAN,0,'='	; increase (LC)
	.WORD	CMDAP,0,'-'	; decrease
	.WORD	CMDAP,0,'_'	; decrease (UC)
	.WORD	CMDA,0,'0'	; autostart off
	.WORD	CMDA,1<<SIGMOD,'1' ; autostart signal gate
	.WORD	CMDA,1<<PULMOD,'2' ; autostart pulse gate
	.WORD	GETPOP,0,0	; default
;
; Adjust carrier threshold c{+-}
;
ST41	.WORD	CMDCN,0,'+'	; increase
	.WORD	CMDCN,0,'='	; increase (LC)
	.WORD	CMDCP,0,'-'	; decrease
	.WORD	CMDCP,0,'_'	; decrease (UC)
	.WORD	CMDC,0,'0'	; mark/space
	.WORD	CMDC,1<<SPCSTA,'1' ; mark only (space disable)
	.WORD	CMDC,1<<MRKSTA,'2' ; space only (mark disable)
	.WORD	GETPOP,0,0	; default
;
; Initialization table. This is used to configure the AIO at initial
; startup.
;
INITAB	.WORD	3		; 0 load AIO configuration
	.WORD	CFGC		; 1
	.WORD	3		; 2 load TB register
	.WORD	CFGB		; 3
	.WORD	3		; 4 load TA register
	.WORD	CFGA		; 5
	.WORD	0		; 6 terminator

;
; Baud rate data tables. Note that 74.2 baud is rounded to 75 baud,
; since almost all RTTY today uses electronic UARTs. The first word is
; the number of AIO samples per bit sample, where there are 8 samples
; per bit. The next two words establish the matched filter size and
; gain, respectively. Note that the FAX rates are synchronous with
; submultiples of the AIO sample frequency to minimize aliasing.
;
; 100 baud (132 WPM)
;
BAUD1	.WORD	1250		; AIO samples (1000000 / (r * BINSIZ))
	.WORD	79*4		; MF size * 4 (SFREQ / r)
	.WORD	830		; MF gain (2^16 / size)
;
; 75 Baud (100 WPM)
;
BAUD2	.WORD	1667		; AIO samples (1000000 / (r * BINSIZ))
	.WORD	106*4		; MF size * 4 (SFREQ / r)
	.WORD	618		; MF gain (2^16 / size)
;
; 50 baud (66 WPM)
;
BAUD3	.WORD	2500		; AIO samples (1000000 / (r * BINSIZ))
	.WORD	159*4		; MF size * 4 (SFREQ / r)
	.WORD	412		; MF gain (2^16 / size)
;
; 45.45 baud (60 WPM)
;
BAUD4	.WORD	2750		; AIO samples (1000000 / (r * BINSIZ))
	.WORD	175*4		; MF size * 4 (SFREQ / r)
	.WORD	374		; MF gain (2^16 / size)
;
; 25 baud (33 WPM)
;
BAUD5	.WORD	5000		; AIO samples (1000000 / (r * BINSIZ))
	.WORD	318*4		; MF size * 4 (SFREQ / r)
	.WORD	206		; MF gain (2^16 / size)
;
; 10 baud (13.2 WPM)
;
BAUD6	.WORD	12500		; AIO samples (1000000 / (r * BINSIZ))
	.WORD	745*4		; MF size * 4 (SFREQ / r)
	.WORD	88		; MF gain (2^16 / size)

;
; Tone frequency data tables. These are used to initialize variables for
; various combinations of mode, filter and spacing. The first two words
; control the input mark and space carrier oscillators, respectively.
; The third word controls various mode bits. The fourth word sets the
; filter gain n+m, where n is the calculated gain factor and m is a trim
; value. The fifth and sixth words control the output AFSK mark and
; space carrier oscillators. The MF filter gain is set empirically at
; 2295 Hz with the limiter switched off so that the signal LED (LED 4)
; comes on with the mark LED (LED 5). The FIR filter gains are set for
; 800mV peak mark signal at the input clipper threshold; the IIR filter
; gains are set so the mark signal is the same as the MF value at each
; shift.
;
; Note the narrrow-shift IIR tones are centered 125 Hz wide of the
; working shift, in order to improve linearity; e.g., the mark filter is
; centered on 2420 Hz, the space filter on 2000 Hz. For all shifts, the
; center frequency of the discriminator is half way between the upper
; and lower 3-dB points of the corresponding FIR bandpass filter. This
; bit of nonsense matches the AEA PK-232 TNC.
;
; IIR filter, center frequency 1700 Hz, wide shift 850 Hz
;
;
SHIFT1	.WORD	IFILT		; mark filter
	.WORD	-(1275*FAC)	; mark in
	.WORD	IFILT		; space filter
	.WORD	-(2125*FAC)	; space in
	.WORD	0		; wide filter
	.WORD	34		; filter gain
;
; IIR filter, center frequency 1700 Hz, mid shift 600 Hz
;
SHIFT2	.WORD	IFILT		; mark filter
	.WORD	-(1400*FAC)	; mark in
	.WORD	IFILT		; space filter
	.WORD	-(2000*FAC)	; space in
	.WORD	0		; wide filter
	.WORD	34		; filter gain

;
; IIR filter, center frequency 1700 Hz, mid shift 400 Hz
;
SHIFT3	.WORD	IFILT		; mark filter
	.WORD	-(1500*FAC)	; mark in
	.WORD	IFILT		; space filter
	.WORD	-(1900*FAC)	; space in
	.WORD	0		; wide filter
	.WORD	37		; filter gain
;
; IIR filter, center frequency 2210 Hz, narrow shift 170 Hz
;
SHIFT4	.WORD	IFILT		; mark filter
	.WORD	-(2000*FAC)	; mark in
	.WORD	IFILT		; space filter
	.WORD	-(2420*FAC)	; space in
	.WORD	1<<NARMOD	; narrow filter
	.WORD	42		; filter gain
;
; MF filter, center frequency 1700 Hz, wide shift 850 Hz
;
SHIFT5	.WORD	MFILT		; mark filter
	.WORD	-(1275*FAC)	; mark in
	.WORD	MFILT		; space filter
	.WORD	-(2125*FAC)	; space in
	.WORD	0		; wide filter
	.WORD	256+8		; filter gain
;
; MF filter, center frequency 1700 Hz, mid shift 600 Hz
;
SHIFT6	.WORD	MFILT		; mark filter
	.WORD	-(1400*FAC)	; mark in
	.WORD	MFILT		; space filter
	.WORD	-(2000*FAC)	; space in
	.WORD	0		; wide filter
	.WORD	256+8		; filter gain
;
; MF filter, center frequency 1700 Hz, mid shift 400 Hz
;
SHIFT7	.WORD	MFILT		; mark filter
	.WORD	-(1500*FAC)	; mark in
	.WORD	MFILT		; space filter
	.WORD	-(1900*FAC)	; space in
	.WORD	0		; wide filter
	.WORD	256+11		; filter gain
;
; MF filter, center frequency 2210 Hz, narrow shift 170 Hz
;
SHIFT8	.WORD	MFILT		; mark filter
	.WORD	-(2125*FAC)	; mark in
	.WORD	MFILT		; space filter
	.WORD	-(2295*FAC)	; space in
	.WORD	1<<NARMOD	; narrow filter
	.WORD	256+18		; filter gain

;
; ITA-2 to ASCII translate table (US keyboard). LSB is FIGS shift, MSB
; is LTRS shift. Adapted from ARRL 1995 Handbook Table 30.41. Each line
; defines one word ".BYTE x,y", where x is the MSB and y is the LSB. The
; LTRS-shift Baudot characters are in the MSB, while the FIGS-shift ones
; are in the LSB.
;				     ALL  ITA2 US
;				     LTRS FIGS FIGS
BAUDOT	.BYTE	NUL,NUL		; 00 NUL  NUL  NUL
	.BYTE	'E','3'		; 01 E    3    3
	.BYTE	LF,LF		; 02 LF   LF   LF
	.BYTE	'A','-'		; 03 A    -    -
	.BYTE	' ',' '		; 04 SP   SP   SP
	.BYTE	'S',BEL		; 05 S    '    BELL
	.BYTE	'I','8'		; 06 I    8    8
	.BYTE	'U','7'		; 07 U    7    7
	.BYTE	CR,CR		; 08 CR   CR   CR
	.BYTE	'D','$'		; 09 D    WRU  $
	.BYTE	'R','4'		; 0A R    4    4
	.BYTE	'J',27h		; 0B J    BELL '
	.BYTE	'N',','		; 0C N    ,    ,
	.BYTE	'F','!'		; 0D F    NA   !
	.BYTE	'C',':'		; 0E C    :    :
	.BYTE	'K','('		; 0F K    (    (
	.BYTE	'T','5'		; 10 T    5    5
	.BYTE	'Z',22h		; 11 Z    +    "
	.BYTE	'L',')'		; 12 L    )    )
	.BYTE	'W','2'		; 13 W    2    2
	.BYTE	'H','#'		; 14 H    NA   #
	.BYTE	'Y','6'		; 15 Y    6    6
	.BYTE	'P','0'		; 16 P    0    0
	.BYTE	'Q','1'		; 17 Q    1    1
	.BYTE	'O','9'		; 18 O    9    9
	.BYTE	'B','?'		; 19 B    ?    ?
	.BYTE	'G','&'		; 1A G    NA   &
	.BYTE	NUL,NUL		; 1B FIGS FIGS FIGS
	.BYTE	'M','.'		; 1C M    .    .
	.BYTE	'X','/'		; 1D X    /    /
	.BYTE	'V',';'		; 1E V    =    ;
	.BYTE	NUL,NUL		; 1F LTRS LTRS LTRS

;
; ASCII to ITA-2 translate table (US keyboard). Adapted from ARRL 1995
; Handbook Table 30.44. Each line defines one word ".BYTE x,y", where x
; is the MSB and y is the LSB. The even-numbered ASCII characters are in
; the MSB, while the odd-numbered ones in the LSB. Except for the
; following, all undefined Baudot codes are translated to Baudot NUL.
;
; EOT	LTRS	close down transmitter immediately
; ACK	LTRS	close down transmitter when buffer empty
; ENQ	WRU	trigger answerback at receiver
; SYN	LTRS	use for fill if necessary
;
ASCII	.BYTE	0,0		; 00 NUL 01 SOH
	.BYTE	0,0		; 02 STX 03 ETX
	.BYTE	1Fh,09h+UC	; 04 EOT 05 ENQ (ENQ = Baudot WRU)
	.BYTE	1Fh,05h+UC	; 06 ACK 07 BEL
	.BYTE	0,0		; 08 BS  09 HT
	.BYTE	02h,0		; 0A LF  0B VT
	.BYTE	0,08h		; 0C FF  0D CR
	.BYTE	0,0		; 0E SO  0F SI
	.BYTE	0,0		; 10 DLE 11 DC1
	.BYTE	0,0		; 12 DC2 13 DC3
	.BYTE	0,0		; 14 DC4 15 NAK
	.BYTE	1Fh+LC,0	; 16 SYN 17 ETB (SYN = Baudot LTRS)
	.BYTE	0,0		; 18 CAN 19 EM
	.BYTE	0,0		; 1A SUB 1B ESC
	.BYTE	0,0		; 1C FS  1D GS
	.BYTE	0,0		; 1E RS  1F US
 	.BYTE	04h,0Dh+UC	; 20 SP  21 !
	.BYTE	11h+UC,14h+UC	; 22 "   23 #
	.BYTE	09h+UC,0	; 24 $   25 %
	.BYTE	1A+UC,0Bh+UC	; 26 &   27 '
	.BYTE	0Fh+UC,12h+UC	; 28 (   29 )
	.BYTE	0,0		; 2A *   2B +
	.BYTE	0Ch+UC,03H+UC	; 2C ,   2D -
	.BYTE	1Ch+UC,1Dh+UC	; 2E .   2F /
	.BYTE	16h+UC,17h+UC	; 30 0   31 1
	.BYTE	13h+UC,01h+UC	; 32 2   33 3
	.BYTE	0Ah+UC,10h+UC	; 34 4   35 5
	.BYTE	15h+UC,07h+UC	; 36 6   37 7
	.BYTE	06h+UC,18h+UC	; 38 8   39 9
	.BYTE	0Eh+UC,1Eh+UC	; 3A :   3B ;
	.BYTE	0,0		; 3C <   3D =
	.BYTE	0,19h+UC	; 3E >   3F ?
	.BYTE	0,03h+LC	; 40 @   41 A
	.BYTE	19h+LC,0Eh+LC	; 42 B   43 C
	.BYTE	09h+LC,01h+LC	; 44 D   45 E
	.BYTE	0Dh+LC,1Ah+LC	; 46 F   47 G
	.BYTE	14h+LC,06h+LC	; 48 H   49 I
	.BYTE	0Bh+LC,0Fh+LC	; 4A J   4B K
	.BYTE	12h+LC,1Ch+LC	; 4C L   4D M
	.BYTE	0Ch+LC,18h+LC	; 4E N   4F O
	.BYTE	16h+LC,17h+LC	; 50 P   51 Q
	.BYTE	0Ah+LC,05h+LC	; 52 R   53 S
	.BYTE	10h+LC,07h+LC	; 54 T   55 U
	.BYTE	1Eh+LC,13h+LC	; 56 V   57 W
	.BYTE	1Dh+LC,15h+LC	; 58 X   59 Y
	.BYTE	11h+LC,0	; 5A Z   5B [
	.BYTE	0,0		; 5C \   5D ]
	.BYTE	0,0		; 5E `   5F _

;
; ITA-2 to CCIR 625 translation table. Adapted from ARRL 1995
; Handbook Table 30.41 and Table 12.6.
;
CCIR	.WORD	1101010b	; 00 NUL (CS2)
	.WORD	1010110b	; 01 E
	.WORD	1101100b	; 02 LF
	.WORD	1000111b	; 03 A
	.WORD	1011100b	; 04 SP
	.WORD	1001011b	; 05 S
	.WORD	1001101b	; 06 I
	.WORD	1001110b	; 07 U
	.WORD	1111000b	; 08 CR
	.WORD	1010011b	; 09 D
	.WORD	1010101b	; 0A R
	.WORD	0010111b	; 0B J
	.WORD	1011001b	; 0C N (CS3)
	.WORD	0011011b	; 0D F
	.WORD	0011101b	; 0E C
	.WORD	0011110b	; 0F K
	.WORD	1110100b	; 10 T
	.WORD	1100011b	; 11 Z
	.WORD	1100101b	; 12 L (CS1)
	.WORD	0100111b	; 13 W
	.WORD	1101001b	; 14 H (CS5)
	.WORD	0101011b	; 15 Y
	.WORD	0101101b	; 16 P
	.WORD	0101110b	; 17 Q
	.WORD	1110001b	; 18 O
	.WORD	1110010b	; 19 B
	.WORD	0110101b	; 1A G (CS4)
	.WORD	0110110b	; 1B FIGS
	.WORD	0111001b	; 1C M
	.WORD	0111010b	; 1D X
	.WORD	0111100b	; 1E V
	.WORD	1011010b	; 1F LTRS
CCTRQ	.WORD	1100110b	; 20 signal repetition (RQ)
CCTALF	.WORD	0001111b	; 21 signal alpha )ALPHA)
CCTBET	.WORD	0110011b	; 22 signal beta

;
; The following filter coefficient tables were generated using Math
; Works Inc. Matlab Signal Processing Toolkit. The performance data were
; obtained directly from the rounded and truncated coefficients, in
; order to accurately model the roundoff error.
;
; IIR coefficients for 170-Hz shift LPF - Butterworth, q10/20 format,
; sample freq 8000, order 2, transition 125 Hz, 3dB 150 Hz, 6dB 188 Hz
;
IIR1	.WORD	-1734		; a2 q10
	.WORD	3755		; a1 q10
	.WORD	2048		; a0 q10
	.WORD	6711		; b0 q20
	.WORD	13421		; b1 q20
	.WORD	6711		; b2 q20

;
; The following two tables define the FIR coefficients for a 170-Hz
; bandpass filter and a 850-Hz bandpass filter. Both were produced using
; the Remez method, sample frequency 8000 Hz, order 70, q15 format.
;
; FIR coefficients for 170-Hz shift - design passband 2125-2295 Hz,
; transition band 300 Hz, 3dB 2031-2391 Hz, 6dB 1984-2438 Hz, stopbands
; 0-1828 and 2625-4000 Hz, max stopband ripple -50dB.
;
BPF170	.WORD	   -11		;   0
	.WORD	   -19		;   1
	.WORD	   -25		;   2
	.WORD	   -49		;   3
	.WORD	    97		;   4
	.WORD	    28		;   5
	.WORD	  -167		;   6
	.WORD	    19		;   7
	.WORD	   243		;   8
	.WORD	  -119		;   9
	.WORD	  -267		;  10
	.WORD	   237		;  11
	.WORD	   216		;  12
	.WORD	  -310		;  13
	.WORD	  -102		;  14
	.WORD	   270		;  15
	.WORD	    -2		;  16
	.WORD	   -88		;  17
	.WORD	   -15		;  18
	.WORD	  -189		;  19
	.WORD	   256		;  20
	.WORD	   432		;  21
	.WORD	  -752		;  22
	.WORD	  -465		;  23
	.WORD	  1420		;  24
	.WORD	   138		;  25
	.WORD	 -2064		;  26
	.WORD	   591		;  27
	.WORD	  2438		;  28
	.WORD	 -1613		;  29
	.WORD	 -2336		;  30
	.WORD	  2681		;  31
	.WORD	  1692		;  32
	.WORD	 -3491		;  33
	.WORD	  -618		;  34
	.WORD	  3792		;  35
	.WORD	  -618		;  36
	.WORD	 -3491		;  37
	.WORD	  1692		;  38
	.WORD	  2681		;  39
	.WORD	 -2336		;  40
	.WORD	 -1613		;  41
	.WORD	  2438		;  42
	.WORD	   591		;  43
	.WORD	 -2064		;  44
	.WORD	   138		;  45
	.WORD	  1420		;  46
	.WORD	  -465		;  47
	.WORD	  -752		;  48
	.WORD	   432		;  49
	.WORD	   256		;  50
	.WORD	  -189		;  51
	.WORD	   -15		;  52
	.WORD	   -88		;  53
	.WORD	    -2		;  54
	.WORD	   270		;  55
	.WORD	  -102		;  56
	.WORD	  -310		;  57
	.WORD	   216		;  58
	.WORD	   237		;  59
	.WORD	  -267		;  60
	.WORD	  -119		;  61
	.WORD	   243		;  62
	.WORD	    19		;  63
	.WORD	  -167		;  64
	.WORD	    28		;  65
	.WORD	    97		;  66
	.WORD	   -49		;  67
	.WORD	   -25		;  68
	.WORD	   -19		;  69
	.WORD	   -11		;  70

;
; FIR coefficients for 170-Hz shift - design passband 1275-2125 Hz,
; transition band 300 Hz, 3dB 1172-2234 Hz, 6dB 1141-2266 Hz, stopbands
; 0-984 and 2422-4000 Hz, max stopband ripple -48dB.
;
BPF850	.WORD	    24		;   0
	.WORD	   -14		;   1
	.WORD	    79		;   2
	.WORD	    27		;   3
	.WORD	  -128		;   4
	.WORD	  -104		;   5
	.WORD	    42		;   6
	.WORD	    11		;   7
	.WORD	     1		;   8
	.WORD	   197		;   9
	.WORD	   113		;  10
	.WORD	  -292		;  11
	.WORD	  -259		;  12
	.WORD	    99		;  13
	.WORD	    29		;  14
	.WORD	    -3		;  15
	.WORD	   467		;  16
	.WORD	   316		;  17
	.WORD	  -606		;  18
	.WORD	  -594		;  19
	.WORD	   181		;  20
	.WORD	    45		;  21
	.WORD	   -41		;  22
	.WORD	  1024		;  23
	.WORD	   810		;  24
	.WORD	 -1257		;  25
	.WORD	 -1410		;  26
	.WORD	   334		;  27
	.WORD	    57		;  28
	.WORD	  -218		;  29
	.WORD	  2876		;  30
	.WORD	  2917		;  31
	.WORD	 -4355		;  32
	.WORD	 -7261		;  33
	.WORD	  2122		;  34
	.WORD	  9423		;  35
	.WORD	  2122		;  36
	.WORD	 -7261		;  37
	.WORD	 -4355		;  38
	.WORD	  2917		;  39
	.WORD	  2876		;  40
	.WORD	  -218		;  41
	.WORD	    57		;  42
	.WORD	   334		;  43
	.WORD	 -1410		;  44
	.WORD	 -1257		;  45
	.WORD	   810		;  46
	.WORD	  1024		;  47
	.WORD	   -41		;  48
	.WORD	    45		;  49
	.WORD	   181		;  50
	.WORD	  -594		;  51
	.WORD	  -606		;  52
	.WORD	   316		;  53
	.WORD	   467		;  54
	.WORD	    -3		;  55
	.WORD	    29		;  56
	.WORD	    99		;  57
	.WORD	  -259		;  58
	.WORD	  -292		;  59
	.WORD	   113		;  60
	.WORD	   197		;  61
	.WORD	     1		;  62
	.WORD	    11		;  63
	.WORD	    42		;  64
	.WORD	  -104		;  65
	.WORD	  -128		;  66
	.WORD	    27		;  67
	.WORD	    79		;  68
	.WORD	   -14		;  69
	.WORD	    24		;  70

;
; Sine table in 3-degree increments (450 degrees), q15 format
;
SINTAB	.WORD	     0		;   0
	.WORD	  1715		;   3
	.WORD	  3425		;   6
	.WORD	  5126		;   9
	.WORD	  6813		;  12
	.WORD	  8481		;  15
	.WORD	 10126		;  18
	.WORD	 11743		;  21
	.WORD	 13328		;  24
	.WORD	 14876		;  27
	.WORD	 16384		;  30
	.WORD	 17847		;  33
	.WORD	 19261		;  36
	.WORD	 20622		;  39
	.WORD	 21926		;  42
	.WORD	 23170		;  45
	.WORD	 24351		;  48
	.WORD	 25466		;  51
	.WORD	 26510		;  54
	.WORD	 27482		;  57
	.WORD	 28378		;  60
	.WORD	 29197		;  63
	.WORD	 29935		;  66
	.WORD	 30592		;  69
	.WORD	 31164		;  72
	.WORD	 31651		;  75
	.WORD	 32052		;  78
	.WORD	 32365		;  81
	.WORD	 32588		;  84
	.WORD	 32723		;  87
	.WORD	 32767		;  90
	.WORD	 32723		;  93
	.WORD	 32588		;  96
	.WORD	 32365		;  99
	.WORD	 32052		; 102
	.WORD	 31651		; 105
	.WORD	 31164		; 108
	.WORD	 30592		; 111
	.WORD	 29935		; 114
	.WORD	 29197		; 117
	.WORD	 28378		; 120
	.WORD	 27482		; 123
	.WORD	 26510		; 126
	.WORD	 25466		; 129
	.WORD	 24351		; 132
	.WORD	 23170		; 135
	.WORD	 21926		; 138
	.WORD	 20622		; 141
	.WORD	 19261		; 144
	.WORD	 17847		; 147
	.WORD	 16384		; 150
	.WORD	 14876		; 153
	.WORD	 13328		; 156
	.WORD	 11743		; 159
	.WORD	 10126		; 162
	.WORD	  8481		; 165
	.WORD	  6813		; 168
	.WORD	  5126		; 171
	.WORD	  3425		; 174
	.WORD	  1715		; 177
	.WORD	     0		; 180
	.WORD	 -1715		; 183
	.WORD	 -3425		; 186
	.WORD	 -5126		; 189
	.WORD	 -6813		; 192
	.WORD	 -8481		; 195
	.WORD	-10126		; 198
	.WORD	-11743		; 201
	.WORD	-13328		; 204
	.WORD	-14876		; 207
	.WORD	-16384		; 210
	.WORD	-17847		; 213
	.WORD	-19261		; 216
	.WORD	-20622		; 219
	.WORD	-21926		; 222
	.WORD	-23170		; 225
	.WORD	-24351		; 228
	.WORD	-25466		; 231
	.WORD	-26510		; 234
	.WORD	-27482		; 237
	.WORD	-28378		; 240
	.WORD	-29197		; 243
	.WORD	-29935		; 246
	.WORD	-30592		; 249
	.WORD	-31164		; 252
	.WORD	-31651		; 255
	.WORD	-32052		; 258
	.WORD	-32365		; 261
	.WORD	-32588		; 264
	.WORD	-32723		; 267
	.WORD	-32768		; 270
	.WORD	-32723		; 273
	.WORD	-32588		; 276
	.WORD	-32365		; 279
	.WORD	-32052		; 282
	.WORD	-31651		; 285
	.WORD	-31164		; 288
	.WORD	-30592		; 291
	.WORD	-29935		; 294
	.WORD	-29197		; 297
	.WORD	-28378		; 300
	.WORD	-27482		; 303
	.WORD	-26510		; 306
	.WORD	-25466		; 309
	.WORD	-24351		; 312
	.WORD	-23170		; 315
	.WORD	-21926		; 318
	.WORD	-20622		; 321
	.WORD	-19261		; 324
	.WORD	-17847		; 327
	.WORD	-16384		; 330
	.WORD	-14876		; 333
	.WORD	-13328		; 336
	.WORD	-11743		; 339
	.WORD	-10126		; 342
	.WORD	 -8481		; 345
	.WORD	 -6813		; 348
	.WORD	 -5126		; 351
	.WORD	 -3425		; 354
	.WORD	 -1715		; 357
	.WORD	    -0		; 360
	.WORD	  1715		; 363
	.WORD	  3425		; 366
	.WORD	  5126		; 369
	.WORD	  6813		; 372
	.WORD	  8481		; 375
	.WORD	 10126		; 378
	.WORD	 11743		; 381
	.WORD	 13328		; 384
	.WORD	 14876		; 387
	.WORD	 16384		; 390
	.WORD	 17847		; 393
	.WORD	 19261		; 396
	.WORD	 20622		; 399
	.WORD	 21926		; 402
	.WORD	 23170		; 405
	.WORD	 24351		; 408
	.WORD	 25466		; 411
	.WORD	 26510		; 414
	.WORD	 27482		; 417
	.WORD	 28378		; 420
	.WORD	 29197		; 423
	.WORD	 29935		; 426
	.WORD	 30592		; 429
	.WORD	 31164		; 430
	.WORD	 31651		; 435
	.WORD	 32052		; 438
	.WORD	 32365		; 441
	.WORD	 32588		; 444
	.WORD	 32723		; 447
	.WORD	 32767		; 450
;
	.END			; end of program
