;  File:  MEASGAIN.ASM Version 1.3 December 10, 1994 Copyright Ron Parsons W5RKN 1994
;
;  To assemble, use the command "asm measgain.asm -dTONES" to produce audio loop-back
;    or "asm measgain.asm -dINPUT" to measure gain from input on radio 1
;
;  Program to measure the gain with the software selectable gain settings.
;
;
;
;
;Audio loop-back mode:
;  Sweeps the audio input through each gain setting and prints a value to the terminal.
;
;  Loops the audio output back to radio port 1 (unplug the cable from port 1).
;     Steps the audio output/input frequency, prints the frequency,
;     then steps through each gain setting.
;
;Both modes:
;  For each step, the program prints the step number and the mean square
;     of the 128 samples at each step. Uses the upper 10 bits of each sample. 
;     The samples at each step are averaged and adjusted to zero
;     average before the mean square is calculated.
;
;  When the program starts, LED 201 is turned off.  When the program is running,
;     LED 205 (MSB), LED 206, and LED 207 (LSB) indicate the gain step.
;
;  If the sampling rate is too fast for your input signal, adjust CMDA and CMDB
;
;  The program waits for a character to be typed from the keyboard before proceeding
;     to allow time to set up a terminal program to observe the mean square numbers.
;     LED 204 lights and LED 207, LED 206, and LED 205 begin to cycle.
;
;  The program runs until the DSP-93 is reset.
;
;Revision history
;  
;  1.3 Changes due to Monitor memory usage
;  
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Uses radio port #1
;  LED208 is fixed active as power indicator

;  Portions copyright Frank H Perkins, 1993 and Lon Cecil, 1993
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
; 
; This document, and it executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
; 
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
; 
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
; 
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;

#include  "REGS.INC"
#include  "PORTS.INC"
#include  "MONITOR.INC"

;  global variables
TEMP      .EQU      PRVAL_MEM ;  temporary storage
TEMP2     .EQU      OUTBIT2_MEM ;  temporary storage

CFG       .EQU      6Bh       ;  AIO CFG buffer (data mem page 8)
XO        .EQU      6Ch       ;  Gain/LED control
LCT1      .EQU      6Dh       ;  loop counter
LCT2      .EQU      6Eh       ;  loop counter
SO        .EQU      6Fh       ;  sig buffer 
H         .EQU      70h       ;  storage for H part of ACC
L         .EQU      71h       ;  storage for L part of ACC
ADJUST    .EQU      72h       ;  adjustment for zero offset
LCTF      .EQU      73h       ;  audio freq loop counter
RB        .EQU      74h       ;  ramp buffer
TS        .EQU      75h       ;  sine buffer
LCTF1     .EQU      76h       ;  audio pre-sound loop counter

BUFI      .EQU      77h       ;  AIO input buffer
BUFO      .EQU      78h       ;  AIO output buffer
FLAG      .EQU      79h       ;  data ready flag

G1        .EQU      1000h     ;  port 7 values array (absolute address)
L1        .EQU      1020h     ;  port 6 values array (absolute address)
S1        .EQU      1040h     ;  sample values array (absolute address)

NSAMPS    .EQU      128       ;  number of samples (power of 2)
LOG2NS    .EQU      6         ;  (Log (2) number of samples) minus 1
NSAMPM1   .EQU      NSAMPS-1  ;  number of samples minus 1
GLOOPS    .EQU      8         ;  number of gain loops (usually 8)
FLOOPS    .EQU      13        ;  number of frequency loops 
NPRESND   .EQU      6000      ;  number of pre-sound loops 

CMDA      .EQU      CMDA_VAL(10)     ;  AIO load RA/TA = 10 for SCF @ 500.0 kHz
CMDB      .EQU      CMDB_VAL(14)     ;  AIO load RB/TB = 14 for 35714 samples/sec 
;CMDA      .EQU      CMDA_VAL(10)     ;  AIO load RA/TA = 10 for SCF @ 500.0 kHz
;CMDB      .EQU      CMDB_VAL(5)      ;  AIO load RB/TB = 5 for 100,000 samples/sec 
;CMDA      .EQU      CMDA_VAL(10)     ;  AIO load RA/TA = 10 for SCF @ 500.0 kHz
;CMDB      .EQU      CMDB_VAL(15)     ;  AIO load RB/TB = 15 for 33,333.33 samples/sec 
;CMDA      .EQU      CMDA_VAL(16)     ;  AIO load RA/TA = 16 for SCF @ 312.5 kHz
;CMDB      .EQU      CMDB_VAL(22)     ;  AIO load RB/TB = 22 for 14,204.55 samples/sec 
;  AIO configuration: HPF in, Sync, 3 V half scale, sin x/x
CMDC      .EQU      AIO_CONFIG|AIO_INSADHPF|AIO_SYNC|AIO_GAIND60|AIO_GAIND70|AIO_INSSINX

TOP       .EQU      04000h    ;  ramp top value
;INCF      .EQU      (500*16384)/14205     ;  incr value for 500 Hz @ 14,204.55 samples/sec
;INCF      .EQU      (500*16384)/33333     ;  incr value for 500 Hz @ 33,333.33 samples/sec
;INCF      .EQU      (5000*16384)/100000   ;  incr value for 5000 Hz @ 100,000 samples/sec
INCF      .EQU      (500*16384)/35714     ;  incr value for 500 Hz @ 35714 samples/sec
INCP      .EQU      500       ;  printing incr value for 500 Hz
;INCP      .EQU      5         ;  printing incr value for 5 MHz

;  locations used by PRVAL
KSTART    .EQU      7Ah       ;  address of powers of 2 konstants (data mem page 8)
HI        .EQU      7Bh       ;  storage for HI part of ACC
LO        .EQU      7Ch       ;  storage for LO part of ACC
CRYDIG    .EQU      7Dh       ;  storage for carry digit/print digit
LCT3      .EQU      7Eh       ;  loop counter
LCT4      .EQU      7Fh       ;  loop counter

DANS      .EQU      2000h     ;  answer and powers of 2 array (absolute address)

NDIGITS   .EQU      10        ;  number of digits allowed in answer
NPOWOF2   .EQU      32        ;  number of powers of 2 (incl 2^0)
NACCBITS  .EQU      32        ;  number of bits in ACC to look at

; ---------------------------------------------------------------------
          .ORG      1000h

          B         GO        ;  branch to program start (TINT)
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)

GO        DINT                ;  shut off interrupts
          LDPK      USER_PAGE ;  data mem page 8 (external)

LED_INI   LALK      0000h     ;  sel AUX in, gain = 1, LEDs OFF
          SACL      XO        ;  into XO
          OUT       XO,UART_CTRL    ;  out to ANA latch, to disable AIO chip

          LALK      0000h     ;  load DSP start
          SACL      XO        ;  into XO
          OUT       XO,RADIO_GAIN
          OUT       XO,TNC_OUTPUT
          OUT       XO,RADIO_CTRL
          CALL      WAIT4     ;  only POWER LED should be lighted

          LARP      AR0
          LRLK      AR0,2000h ;  Set up a known memory area pointer
          CALL      INBIT     ;  Force program to wait for keyboard input before proceeding

          LARP      AR0       ;  Write welcome string
          LRLK      AR0,2000h
          RPTK      S_END-STR1
          BLKP      STR1,*+
          LARP      AR0
          LRLK      AR0,2000h
          CALL      SD_STR
          CALL      SD_CRLF

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       0         ;  set P shift = 0

LOADTBL   LARP      AR0       ;  used by PRVAL
          LRLK      AR0,DANS  ;  Set up a known memory area pointer
          LALK      ANS       ;     
          RPTK      249       ;  transfer first 250 values from ANS
          TBLR      *+        ;  move it to DANS
          LRLK      AR0,DANS+250
          LALK      ANS+250   ;     
          RPTK      79        ;  transfer remaining 80 values from ANS+250
          TBLR      *+        ;  move it to DANS+250

AIO_RST   LALK      AIO_RESET|GAIN4|REC_IN_1     ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 100 ms
 
AIO_INI   LALK      LED_202|GAIN4|REC_IN_1       ;  LED 202 on, radio 1, gain = 16
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 100 ms

          LALK      AIO_ENABLE|GAIN4|REC_IN_1     ;  load AIO run, radio 1, gain = 16
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 100 ms


          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      IMR       ;  store INT mask
          LDPK      USER_PAGE ;  data mem page 8 (external)

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts


LED204    LALK      LED_204   ;  load LED 204 BRIGHT as active light
          SACL      XO        ;  into XO
          OUT       XO,RADIO_CTRL    ;  out to LED 204
; ---------------------------------------------------------------------
                              ;  transfer port values to data memory
          LRLK      AR7,G1    ;  G1 is port values start
          LARP      AR7       ;  make AR7 pointer @ G1
          RPTK      7         ;  repeat (7 + 1) times
          BLKP      G1A,*+    ;  to xfer 8 values

          LRLK      AR6,L1    ;  L1 is port values start
          LARP      AR6       ;  make AR6 pointer @ G1
          RPTK      7         ;  repeat (7 + 1) times
          BLKP      L1A,*+    ;  to xfer 8 values

          SACL      RB        ;  zero ramp buffer
          SACL      TS        ;  zero sine buffer

STARTF    LALK      FLOOPS    ;  number of loops
          SACL      LCTF      ;  loop counter

FLOOP
          CALL      SD_CRLF
#ifdef    TONES
          LT        LCTF
          MPYK      INCP
          PAC
          CALL      PRDEC     ;  print frequency
          CALL      SD_CRLF
#endif

START     LALK      GLOOPS    ;  number of loops
          SACL      LCT1      ;  loop counter

                              ;  sequence of gain steps 
          LRLK      AR7,G1    ;  G1 is port values start
          LRLK      AR6,L1    ;  L1 is LED values start

GAIN      LALK      GLOOPS
          SUB       LCT1
          CALL      HEXOUT    ;  print the gain loop number
          CALL      SP2

          LRLK      AR5,S1    ;  S1 is sample values start
          LARP      AR7       ;  make AR0 pointer @ G1
          LAC       *+        ;  load Radio #1 REC
          SACL      XO

;+++++++++++++++++++++
;          SACL      TEMP
;          CALL      PRVAL16   ;  print it
;          CALL      SD_CRLF
;---------------------

          OUT       XO,07h    ;  Write change to Radio #1 REC - Gain varies
          LARP      AR6       ;  make AR0 pointer @ L1
          LAC       *+        ;  load LED values
          SACL      XO
          OUT       XO,06h    ;  Set LEDs
          CALL      WAIT4

          LALK      NSAMPS    ;  number of loops
          SACL      LCT2      ;  loop counter

          LALK      NPRESND   ;  number of loops
          SACL      LCTF1     ;  for pre-sounds (so you hear something besides clicks)

AGAIN

#ifdef    TONES
          LT        LCTF      ;  get frequency loop counter
          MPYK      INCF      ;  multiply by increment
          PAC                 ;  get product into ACC
          ADD       RB        ;  add to last ramp value

          SACL      RB        ;  store back in RB
          SBLK      TOP       ;  subtract TOP value

          BLEZ      GETS      ;  ramp not > top, go GETSC

          SACL      RB        ;  else put wrapped value in ramp buffer

GETS      ZAC                 ;  zero A
          ADD       RB,9      ;  get ramp offset * 2^7, shift 2^9
          SACH      TS        ;  store high for 2^-16 normalize
          LALK      TRIG_TBL  ;  point to trig table
          ADD       TS        ;  add normalized ramp offset
          TBLR      TS        ;  read trig value * 2^10 into sine buff

TX_OUT
;  Choose among the following lines to adjust the amplitude
;     of the  audio tones <---------
;          LAC       TS,2      ;  get sine value, shift 2^2
;          LAC       TS,1      ;  get sine value, shift 2^1
          LAC       TS        ;  get sine value, shift 2^0
          SFR                 ;  get sine value, shift 2^-1
          ANDK      0FFFCh    ;  mask out AIO control bits
          SACL      BUFO      ;  AIO out

          LALK      0020h     ;  mask SIO TX interrupt active
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      IMR       ;  store mask in IMR
          LDPK      USER_PAGE ;  data mem page 8 (external)
          EINT                ;  enable interrupts 
          IDLE                ;  wait for TX interrupt

          LAC       LCTF1
          SUBK      1
          SACL      LCTF1     ;  pre-sound loop counter
          BGZ       AGAIN
#endif

RX_IN
          LALK      0010h     ;  mask SIO RX interrupt active
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      IMR       ;  store mask in IMR
          LDPK      USER_PAGE ;  data mem page 8 (external)
          EINT                ;  enable interrupts 
          IDLE                ;  wait for RX interrupt

          LAC       BUFI      ;  get analog value

          SACL      SO        ;  save in SO

          LAC       SO,10
          SACH      SO        ;  10 bits in SO
;+++++++++++++++++++++
;          LAC       SO
;          SACL      TEMP
;          CALL      PRVAL16   ;  print it
;          CALL      SP1
;---------------------

;  accumulate square of values
          LARP      AR5       ;  make AR0 pointer @ S1
          LAC       SO
          SACL      *+        ;  store sample

          LAC       LCT2
          SUBK      1
          SACL      LCT2      ;  loop counter
          BGZ       AGAIN

;+++++++++++++++++++++
;          CALL      SD_CRLF
;---------------------

GETAVE    ZAC                 ;  zero A
          LRLK      AR5,S1    ;  S1 is sample values start
          LARP      AR5       ;  make AR0 pointer @ S1
          RPTK      NSAMPM1   ;  repeat (NSAMPM1 + 1) times
          ADD       *+        ;  accumulate sum of samples
          RPTK      LOG2NS    ;  repeat (LOG2NS + 1) times
          SFR                 ;  shift right to get average of samples squared
          SACL      ADJUST    ;  save average of gain for normalization
;+++++++++++++++++++++
;          SACL      TEMP
;          CALL      PRVAL16   ;  print it
;          CALL      SP2
;          CALL      SD_CRLF
;---------------------

          LALK      NSAMPS    ;  number of loops
          SACL      LCT2      ;  loop counter
          LRLK      AR5,S1    ;  S1 is sample values start
          LARP      AR5       ;  make AR0 pointer @ S1
ADJSAM    LAC       LCT2
          SUBK      1
          SACL      LCT2
          BLZ       SUMSQS    ;  we are done
          LAC       *
          SUB       ADJUST    ;  adjust sample
          SACL      *+        ;  save adjusted samples
;+++++++++++++++++++++
;          SACL      TEMP
;          CALL      PRVAL16   ;  print it
;          CALL      SP1
;---------------------
          B         ADJSAM

SUMSQS    MPYK      0         ;  zero P
          ZAC                 ;  zero A
          LRLK      AR5,S1    ;  S1 is sample values start
          LARP      AR5       ;  make AR0 pointer @ S1
          RPTK      NSAMPM1   ;  repeat (NSAMPM1 + 1) times
          SQRA      *+        ;  square and accumulate product
          APAC                ;  add in last product
          RPTK      LOG2NS    ;  repeat (LOG2NS + 1) times
          SFR                 ;  shift right to get average of samples squared

OUTPUT    SACH      H         ;  save H word of ACC
          SACL      L         ;  save L word of ACC

          CALL      PRDEC     ;  print mean square in decimal

;+++++++++++++++++++++
;          CALL      SP4
;          LAC       H         ;  display H word of sum of squares
;          SACL      TEMP
;          CALL      PRVAL16   ;  print it
;          CALL      SP1
;
;          LAC       L         ;  display L word of sum of squares
;          SACL      TEMP
;          CALL      PRVAL16   ;  print it
;---------------------

          CALL      WAIT4
          CALL      SD_CRLF

          LAC       LCT1
          SUBK      1
          SACL      LCT1      ;  loop counter
          BGZ       GAIN

          LAC       LCTF
          SUBK      1
          SACL      LCTF      ;  loop counter
          BGZ       FLOOP

          B         STARTF    ;  start over, and over ...

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
;  Prints the contents of ACC in Decimal.
;  This is a very crude function, but it works. Adds in BCD.
;  It uses the method of addition you learned in first grade.
PRDEC
          SACH      HI        ;  save ACC in HI and LO
          SACL      LO
          BGEZ      POS
          NEG                 ;  ACC was negative, make it pos
          SACH      HI        ;  save ACC in HI and LO
          SACL      LO
          LALK      2Dh       ;  minus sign (ACSII)
          SACL      TEMP2
          CALL      OUTBIT2   ;  print minus sign
POS       LALK      NACCBITS  ;  number of bits in ACC to work with
          SACL      LCT3      ;  loop counter

          ZAC
          LARP      AR0       ;  Zero answer digits
          LRLK      AR0,DANS 
          RPTK      NDIGITS-1
          SACL      *+

LOOP32
          RSXM                ;  reset sign extension mode
          ZALH      HI        ;  restore hi ACC
          ADD       LO        ;  restore lo ACC
          SFR                 ;  shift lowest bit into C
          SACH      HI        ;  save shifted ACC in HI and LO
          SACL      LO
          SSXM                ;  set sign extension mode
          BNC       ELOOP32   ;  nothing to do

          LRLK      AR4,DANS  ;  DANS is answer values start
          LT        LCT3      ;  calculate offset to power of 2 konstants
          MPYK      NDIGITS
          PAC
          ADLK      DANS
          SACL      KSTART
          LAR       AR3,KSTART ;  KSTART is power of 2 konstants start

;  add power of 2 konstant to answer
          LALK      NDIGITS   ;  number of digits in answer
          SACL      LCT4      ;  loop counter

          LARP      AR4
;LOOPDIG  LAC       *,0,AR3   ;  Assembler 2.8.3 won't do this  <--
;         ADD       *+,0,AR4  ;  Assembler 2.8.3 won't do this  <--
LOOPDIG   LAC       *         ;  Get digit of answer
          LARP      AR3
          ADD       *+        ;  add in digit of constant
          LARP      AR4
          SACL      *+    
          LAC       LCT4
          SUBK      1
          SACL      LCT4
          BGZ       LOOPDIG

;  adjust answer digits for base 10 carry
          LALK      NDIGITS   ;  number of digits in answer
          SACL      LCT4      ;  loop counter

          ZAC    
          SACL      CRYDIG    ;  zero carry digit
          LRLK      AR4,DANS+NDIGITS-1  ;  DANS is answer values start
          LARP      AR4
LOOPCRY   LAC       *         ;  Get digit of answer
          ADD       CRYDIG    ;  add in carry digit
          SACL      *         ;  save adjusted digit
          SUBK      10        ;  see if answer digit is greater than 9
          BLZ       NOCRY     ;  no carry digit needed

          SACL      *-        ;  save adjusted digit
          LALK      1
          SACL      CRYDIG    ;  set carry digit to 1
          B         ELOOPCRY

NOCRY     SBRK      1         ;  decrement aux register 
          ZAC    
          SACL      CRYDIG    ;  zero carry digit

ELOOPCRY  LAC       LCT4
          SUBK      1
          SACL      LCT4
          BGZ       LOOPCRY

ELOOP32   LAC       LCT3
          SUBK      1
          SACL      LCT3
          BGZ       LOOP32

;  print the answer supressing leading zeros
          LALK      NDIGITS   ;  number of digits in the answer
          SACL      LCT3      ;  loop counter

          LRLK      AR4,DANS  ;  DANS is answer values start
          LARP      AR4
LOOPANZ   LAC       LCT3
          SUBK      1
          SACL      LCT3
          BZ        LOOPAN1   ;  catch last zero if ACC is zero

          LAC       *+        ;  Get digit of answer
          BZ        LOOPANZ   ;  skip leading zeros
          B         LOOPPRT

LOOPANS
          LAC       LCT3
          SUBK      1
          SACL      LCT3
          BLZ       DONE

LOOPAN1   LAC       *+        ;  Get digit of answer
LOOPPRT   CALL      HEXOUT
          B         LOOPANS

DONE      RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
RINT
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          LAC       DRR       ;  get received AIO data sample from DRR
          LDPK      USER_PAGE ;  data mem page 8 (external)
          SACL      BUFI      ;  store DDR in AIO input buffer
          RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
XINT
          LAC       BUFO      ;  get data from AIO output buffer
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      DXR       ;  store data in DXR to transmit to AIO
          LDPK      USER_PAGE ;  data mem page 8 (external)
          RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
TAB1      LACK      9         ; Print tab
          SACL      TEMP2
          CALL      OUTBIT2
          RET
; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
WAIT4     CALL      WAIT2
          CALL      WAIT2
          RET

WAIT2     CALL      WAIT1
          CALL      WAIT1
          RET

WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ;  Loop through count
          LARP      AR2
          BANZ      WAIT_A    ;  Loop through count
          RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
STR1      .TEXT     " M e a s u r e   G a i n"
          .WORD     00
S_END

          .MSFIRST

#ifdef    TONES
G1A
          .WORD        AIO_ENABLE|GAIN0|XMIT_1    ;Radio #1 XMIT Monitor - Gain=1
          .WORD        AIO_ENABLE|GAIN1|XMIT_1    ;Radio #1 XMIT Monitor - Gain=2
          .WORD        AIO_ENABLE|GAIN2|XMIT_1    ;Radio #1 XMIT Monitor - Gain=4
          .WORD        AIO_ENABLE|GAIN3|XMIT_1    ;Radio #1 XMIT Monitor - Gain=8
          .WORD        AIO_ENABLE|GAIN4|XMIT_1    ;Radio #1 XMIT Monitor - Gain=16
          .WORD        AIO_ENABLE|GAIN5|XMIT_1    ;Radio #1 XMIT Monitor - Gain=32
          .WORD        AIO_ENABLE|GAIN6|XMIT_1    ;Radio #1 XMIT Monitor - Gain=64
          .WORD        AIO_ENABLE|GAIN7|XMIT_1    ;Radio #1 XMIT Monitor - comparator
#endif

#ifdef    INPUT
G1A
          .WORD        AIO_ENABLE|GAIN0|REC_IN_1    ;Radio #1 REC - Gain=1
          .WORD        AIO_ENABLE|GAIN1|REC_IN_1    ;Radio #1 REC - Gain=2
          .WORD        AIO_ENABLE|GAIN2|REC_IN_1    ;Radio #1 REC - Gain=4
          .WORD        AIO_ENABLE|GAIN3|REC_IN_1    ;Radio #1 REC - Gain=8
          .WORD        AIO_ENABLE|GAIN4|REC_IN_1    ;Radio #1 REC - Gain=16
          .WORD        AIO_ENABLE|GAIN5|REC_IN_1    ;Radio #1 REC - Gain=32
          .WORD        AIO_ENABLE|GAIN6|REC_IN_1    ;Radio #1 REC - Gain=64                                       
          .WORD        AIO_ENABLE|GAIN7|REC_IN_1    ;Radio #1 REC - comparator
#endif

L1A       .WORD        0000h    ;  L202,L203,L204 DARK
          .WORD        0400h    ;  L202 BRIGHT
          .WORD        0200h    ;  L203 BRIGHT
          .WORD        0600h    ;  L203,L204 BRIGHT
          .WORD        0100h    ;  L204 BRIGHT  L202,L203 DARK
          .WORD        0500h    ;  L202,L204 BRIGHT   L203 DARK
          .WORD        0300h    ;  L203,L204 BRIGHT   L202 DARK
          .WORD        0700h    ;  L202,L203,L204 BRIGHT


;  Table of powers of 2 in decimal
ANS       .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 0
K31       .WORD        2, 1, 4, 7, 4, 8, 3, 6, 4, 8
K30       .WORD        1, 0, 7, 3, 7, 4, 1, 8, 2, 4 
K29       .WORD        0, 5, 3, 6, 8, 7, 0, 9, 1, 2
K28       .WORD        0, 2, 6, 8, 4, 3, 5, 4, 5, 6
K27       .WORD        0, 1, 3, 4, 2, 1, 7, 7, 2, 8
K26       .WORD        0, 0, 6, 7, 1, 0, 8, 8, 6, 4
K25       .WORD        0, 0, 3, 3, 5, 5, 4, 4, 3, 2
K24       .WORD        0, 0, 1, 6, 7, 7, 7, 2, 1, 6
K23       .WORD        0, 0, 0, 8, 3, 8, 8, 6, 0, 8
K22       .WORD        0, 0, 0, 4, 1, 9, 4, 3, 0, 4
K21       .WORD        0, 0, 0, 2, 0, 9, 7, 1, 5, 2
K20       .WORD        0, 0, 0, 1, 0, 4, 8, 5, 7, 6
K19       .WORD        0, 0, 0, 0, 5, 2, 4, 2, 8, 8
K18       .WORD        0, 0, 0, 0, 2, 6, 2, 1, 4, 4
K17       .WORD        0, 0, 0, 0, 1, 3, 1, 0, 7, 2
K16       .WORD        0, 0, 0, 0, 0, 6, 5, 5, 3, 6
K15       .WORD        0, 0, 0, 0, 0, 3, 2, 7, 6, 8
K14       .WORD        0, 0, 0, 0, 0, 1, 6, 3, 8, 4
K13       .WORD        0, 0, 0, 0, 0, 0, 8, 1, 9, 2
K12       .WORD        0, 0, 0, 0, 0, 0, 4, 0, 9, 6
K11       .WORD        0, 0, 0, 0, 0, 0, 2, 0, 4, 8
K10       .WORD        0, 0, 0, 0, 0, 0, 1, 0, 2, 4
K9        .WORD        0, 0, 0, 0, 0, 0, 0, 5, 1, 2
K8        .WORD        0, 0, 0, 0, 0, 0, 0, 2, 5, 6
K7        .WORD        0, 0, 0, 0, 0, 0, 0, 1, 2, 8
K6        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 6, 4
K5        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 3, 2
K4        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 1, 6
K3        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 8
K2        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 4
K1        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 2
K0        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 1
END_2     .WORD        0


TRIG_TBL  .WORD    00h        ;  sine table, scaled 2^10, 128 + 1 entries
          .WORD    032h
          .WORD    064h
          .WORD    096h
          .WORD    0C8h
          .WORD    0F9h
          .WORD    0129h
          .WORD    0159h
          .WORD    0188h
          .WORD    01B6h
          .WORD    01E3h
          .WORD    020Eh
          .WORD    0239h
          .WORD    0262h
          .WORD    028Ah
          .WORD    02B0h
          .WORD    02D4h
          .WORD    02F7h
          .WORD    0318h
          .WORD    0336h
          .WORD    0353h
          .WORD    036Eh
          .WORD    0387h
          .WORD    039Eh
          .WORD    03B2h
          .WORD    03C4h
          .WORD    03D4h
          .WORD    03E1h
          .WORD    03ECh
          .WORD    03F5h
          .WORD    03FBh
          .WORD    03FFh
          .WORD    0400h
          .WORD    03FFh
          .WORD    03FBh
          .WORD    03F5h
          .WORD    03ECh
          .WORD    03E1h
          .WORD    03D4h
          .WORD    03C4h
          .WORD    03B2h
          .WORD    039Eh
          .WORD    0387h
          .WORD    036Eh
          .WORD    0353h
          .WORD    0336h
          .WORD    0318h
          .WORD    02F7h
          .WORD    02D4h
          .WORD    02B0h
          .WORD    028Ah
          .WORD    0262h
          .WORD    0239h
          .WORD    020Eh
          .WORD    01E3h
          .WORD    01B6h
          .WORD    0188h
          .WORD    0159h
          .WORD    0129h
          .WORD    0F9h
          .WORD    0C8h
          .WORD    096h
          .WORD    064h
          .WORD    032h
          .WORD    00h
          .WORD    0FFCEh
          .WORD    0FF9Ch
          .WORD    0FF6Ah
          .WORD    0FF38h
          .WORD    0FF07h
          .WORD    0FED7h
          .WORD    0FEA7h
          .WORD    0FE78h
          .WORD    0FE4Ah
          .WORD    0FE1Dh
          .WORD    0FDF2h
          .WORD    0FDC7h
          .WORD    0FD9Eh
          .WORD    0FD76h
          .WORD    0FD50h
          .WORD    0FD2Ch
          .WORD    0FD09h
          .WORD    0FCE8h
          .WORD    0FCCAh
          .WORD    0FCADh
          .WORD    0FC92h
          .WORD    0FC79h
          .WORD    0FC62h
          .WORD    0FC4Eh
          .WORD    0FC3Ch
          .WORD    0FC2Ch
          .WORD    0FC1Fh
          .WORD    0FC14h
          .WORD    0FC0Bh
          .WORD    0FC05h
          .WORD    0FC01h
          .WORD    0FC00h
          .WORD    0FC01h
          .WORD    0FC05h
          .WORD    0FC0Bh
          .WORD    0FC14h
          .WORD    0FC1Fh
          .WORD    0FC2Ch
          .WORD    0FC3Ch
          .WORD    0FC4Eh
          .WORD    0FC62h
          .WORD    0FC79h
          .WORD    0FC92h
          .WORD    0FCADh
          .WORD    0FCCAh
          .WORD    0FCE8h
          .WORD    0FD09h
          .WORD    0FD2Ch
          .WORD    0FD50h
          .WORD    0FD76h
          .WORD    0FD9Eh
          .WORD    0FDC7h
          .WORD    0FDF2h
          .WORD    0FE1Dh
          .WORD    0FE4Ah
          .WORD    0FE78h
          .WORD    0FEA7h
          .WORD    0FED7h
          .WORD    0FF07h
          .WORD    0FF38h
          .WORD    0FF6Ah
          .WORD    0FF9Ch
          .WORD    0FFCEh
          .WORD    00h


          .END
