;
; Memory test program for DSP-93                10-29-94 TCM
;
; Tests Data RAM, then Program RAM
;
; Tests all ones, all zeros, address-stuck
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
;
; This document, and its executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
;
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
;
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
;
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;
;
;  Revision history
;
;  Version   Date     Author     Reason
;  ------- -------- -----------  ---------------------------------------------------------
;  0.0.1   10-29-94 TCMcDermott  Original
;  1.0     12-11-94 TCMcDermott  Production #1 release.
;
;
;

#include        "regs.inc"
#include        "monitor.inc"

; ------------------ Internal memory usage page 0 -------------------
; ------------------ only valid for 60h-7Fh = B2  -------------------

; ------------------ Internal memory usage page 4 -------------------
; ---------  (200h-27Fh = B0 lower half when B0 is data)  -----------

; ------------------ Internal memory usage page 5 -------------------
; ---------  (280h-2FFh = B0 upper half when B0 is data)  -----------

; ------------------ Internal memory usage page 6 -------------------
; ------------------  (300h-37Fh = B1 lower half) -------------------

; ------------------ Internal memory usage page 7 -------------------
; ------------------  (380h-3FFh = B1 upper half) -------------------

TEMP1     .EQU      70h       ;  temporary storage


; ------------------ External memory usage page 8 -------------------

; most of the memory accesses go to this page while running

CFG       .EQU      02h       ;  hold AIO configuration

MON0      .EQU      06h       ;  monitor scratch register

MON1      .EQU      060h      ;  monitor scratch register
MON2      .EQU      061h      ;  monitor scratch register
MON3      .EQU      062h      ;  monitor scratch register
PGMPTR    .EQU      063h      ;  pointer to program memory
TEMPDAT   .EQU      064h      ;  temp data for program ram test


; ----------------- Parameter values and constants -------------------
;
;
; Change these 4 values if you have 64k of memory installed
;

PROGBOT   .EQU      1200h       ; bottom of PROGRAM RAM.  Memtest program
				;   occupies 1008h..11FFh
PPAGCNT   .EQU      6Dh         ; PROG PAGE COUNT. 6Eh pages of PROG RAM
				;   are testable from 1200h..7FFFh
				; pages = PPAGCNT+1
DATATOP   .EQU      07FFFh      ; top of DATA RAM
DATABOT   .EQU      0300h       ; bot of external DATA RAM

	  .ORG      1000h


	  B         GO          ; branch to program start (Timer Intrpt)
	  B         RINT        ; SIO receive interrupt (AIO-rx'd value)
	  B         XINT        ; SIO transmit interrupt (AIO-tx'd value)
	  B         GO          ; branch to program start (SW Trap)


GO        DINT                  ; disable interrupts
	  LDPK      08h         ; data mem page 8 (external)
	  RSXM                  ; unsigned mode

AIO_RST   LALK      8025h       ; reset D15 to disconnect AIO (DX&DR)
	  SACL      CFG         ; clear D14 to hold AIO in RESET mode
	  OUT       CFG,07h     ; OUT to config port
	  CALL      WAIT4       ; delay 400 ms


START

;--- all zeros to external RAM

	  LARP      AR0         ; memory pointer
	  LRLK      AR0,DATATOP ; start at top of equipped ram
	  LRLK      AR1, DATATOP-DATABOT ; to bottom of RAM
	  ZAC                   ; zero accum
MC01      SACL      *-,0,AR1    ; write zero in RAM
	  BANZ      MC01,*-,AR0 ; do all lower RAM to 300

;--- read back RAM, verify all zeros

	  LARP      AR0
	  LRLK      AR0,DATATOP ; start at top of RAM again
	  LRLK      AR1,DATATOP-DATABOT ; to bottom of RAM
MC02      LAC       *-,0,AR1    ; load accum
	  BNZ       ERR01       ; ERR01 if not zero
	  BANZ      MC02,*-,AR0 ; do all lower RAM to 300

	  LACK      '1'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 1

;--- all ones to external RAM

	  LARP      AR0
	  LRLK      AR0,DATATOP ; start at top of equipped ram
	  LRLK      AR1,DATATOP-DATABOT ; to bottom of RAM
	  LALK      0FFFFh      ; all-ones to accum
MC03      SACL      *-,0,AR1    ; write zero in RAM
	  BANZ      MC03,*-,AR0 ;  do all lower RAM to 300

;--- read back external RAM, verify all ones

	  LARP      AR0
	  LRLK      AR0,DATATOP ; start at top of RAM again
	  LRLK      AR1,DATATOP-DATABOT ; to bottom of RAM
MC04      LAC       *-,0,AR1
	  XORK      0FFFFh      ; invert all accum bits     
	  BNZ       ERR02       ; ERR02 if not zero
	  BANZ      MC04,*-,AR0 ; do all lower RAM to 300

	  LACK      '2'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 2

; --- verify address uniqueness by putting a different value in
; ---   each memory location.  Value = Address of that location.
;
;   First, go from top of external RAM to bottom
	  
	  LDPK      07h         ; use on-chip RAM for storage
	  LARP      AR0
	  LRLK      AR0,DATATOP ; start at top of external RAM
	  LRLK      AR1,DATATOP-DATABOT ; to bottom of RAM
MC05      SAR       AR0,TEMP1   ; store value of address pointer
	  LAC       TEMP1       ; get value of address to Accum
	  SACL      *-,0,AR1    ; write accum to RAM
	  BANZ      MC05,*-,AR0 ;  do all lower RAM to 300

; --- read back memory, verify correct 

	  LARP      AR0
	  LRLK      AR0,DATATOP ; start at top of RAM again
	  LRLK      AR1,DATATOP-DATABOT ; to bottom of RAM
MC06      SAR       AR0,TEMP1   ; store value of address pointer
	  LAC       TEMP1       ; get addr pointer to Accum
	  SUB       *-,0,AR1    ; compare ptr to memory
	  BNZ       ERR03       ;   jif error
	  BANZ      MC06,*-,AR0 ; do all lower RAM to 300

	  LDPK      08h         ; get page pointer back to monitor page
	  LACK      '3'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 3

;   Now, go store from bottom of RAM to top
	  
	  LDPK      07h         ; use on-chip RAM for storage
	  LARP      AR0
	  LRLK      AR0,DATABOT ; start at bottom of external RAM
	  LRLK      AR1,DATATOP-DATABOT ; length of RAM
MC07      SAR       AR0,TEMP1   ; store value of address pointer
	  LAC       TEMP1       ; get value of address to Accum
	  SACL      *+,0,AR1    ; write accum to RAM
	  BANZ      MC07,*-,AR0 ;  do all lower RAM to 300

; --- read back memory, verify correct 

	  LARP      AR0
	  LRLK      AR0,DATABOT ; start at bottom of RAM again
	  LRLK      AR1,DATATOP-DATABOT ; length of RAM
MC08      SAR       AR0,TEMP1   ; store value of address pointer
	  LAC       TEMP1       ; get addr pointer to Accum
	  SUB       *+,0,AR1    ; compare ptr to memory
	  BNZ       ERR04       ;   jif error
	  BANZ      MC08,*-,AR0 ; do all lower RAM to 300

	  LDPK      08h         ; get page pointer back to monitor page
	  LACK      '4'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 4


; -------------------- Test external PROGRAM RAM ------------------------
;
; ---- write a block 0f 256 zeros to RAM, then transfer that one block
; ----   to many pages in program memory.
; ---- Then read back each page from program memory to data memory,
; ----   and see if it is all zeros.
;

	  LRLK      AR0, 02000h ; from 2000..20FF
	  LRLK      AR1, 0FFh   ; size of block = 256
	  ZAC                   ; zero accum
MC10      SACL      *+,0,AR1    ; write zero in RAM
	  BANZ      MC10,*-,AR0 

	  LALK      PROGBOT
	  SACL      PGMPTR      ; initialize program pointer to 1200h
	  LRLK      AR1,PPAGCNT ; do 6Eh pages of 256 words
	  LARP      AR0
MC11      LRLK      AR0,2000h   ; AR0 points to 2000h in data memory
	  LAC       PGMPTR      ; Accum points to PROG memory, set AR0
	  RPTK      0ffh        ; repeat instruction 256 times
	  TBLW      *+          ; Move from data-mem to program-mem

	  ADLK      100h        ; point to next page
	  SACL      PGMPTR      ; save pointer
	  LARP      AR1         ; get page counter
	  BANZ      MC11,*-,AR0 ; see if all pages (of 256) are done

; now read back program RAM, and see if it's all-zeros

	  LALK      PROGBOT
	  SACL      PGMPTR      ; initialize program pointer to bottom
	  LRLK      AR1,PPAGCNT ; do pages of 256 words
	  LARP      AR0
MC12      LRLK      AR0,3000h   ; AR0 points to 3000h in data memory
	  LAC       PGMPTR      ; Accum points to PROG memory, set AR0
	  RPTK      0ffh        ; repeat instruction 256 times
	  TBLR      *+          ; Move from program-mem to data-mem

; (see if 3000h..30ffh in data RAM is all zeros)

	  LARP      AR2
	  LRLK      AR2,3000h   ; start at 3000h
	  LRLK      AR3, 0FFh   ; for 100h words
MC13      LAC       *+,0,AR3    ; load accum
	  BNZ       ERR05       ; ERR05 if not zero
	  BANZ      MC13,*-,AR2 ; do 256 times

	  LAC       PGMPTR
	  ADLK      100h        ; point to next program RAM page
	  SACL      PGMPTR      ; save pointer
	  LARP      AR1         ; get page counter
	  BANZ      MC12,*-,AR0 ; see if all pages (of 256) are done

	  LACK      '5'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 5

; now do all-ones to program RAM

	  LRLK      AR0, 02000h ; from 2000..20FF
	  LRLK      AR1, 0FFh   ; size of block = 256
	  LALK      0FFFFh      ; all ones to accum
MC14      SACL      *+,0,AR1    ; write all ones to PGM RAM
	  BANZ      MC14,*-,AR0 ; 

	  LALK      PROGBOT
	  SACL      PGMPTR      ; initialize program pointer to bottom
	  LRLK      AR1,PPAGCNT ; do pages of 256 words
	  LARP      AR0
MC15      LRLK      AR0,2000h   ; AR0 points to 2000h in data memory
	  LAC       PGMPTR      ; Accum points to PROG memory, set AR0
	  RPTK      0ffh        ; repeat instruction 256 times
	  TBLW      *+          ; Move from data-mem to program-mem

	  ADLK      100h        ; point to next page
	  SACL      PGMPTR      ; save pointer
	  LARP      AR1         ; get page counter
	  BANZ      MC15,*-,AR0 ; see if all pages (of 256) are done

; now read back program RAM, and see if it's all-ones

	  LALK      PROGBOT
	  SACL      PGMPTR      ; initialize program pointer to bottom
	  LRLK      AR1,PPAGCNT ; do pages of 256 words
	  LARP      AR0
MC16      LRLK      AR0,3000h   ; AR0 points to 3000h in data memory
	  LAC       PGMPTR      ; Accum points to PROG memory, set AR0
	  RPTK      0ffh        ; repeat instruction 256 times
	  TBLR      *+          ; Move from program-mem to data-mem

; (see if 3000h..30ffh in data RAM is all ones)

	  LARP      AR2
	  LRLK      AR2, 03000h       ; start at 3000h
	  LRLK      AR3, 0FFh         ; for 100h words
MC17      LAC       *+,0,AR3          ; load accum
	  SBLK      0FFFFh            ; compare against all-ones
	  BNZ       ERR06             ; ERR06 if not the same
	  BANZ      MC17,*-,AR2       ; do 256 times

	  LAC       PGMPTR
	  ADLK      100h        ; point to next program RAM page
	  SACL      PGMPTR      ; save pointer
	  LARP      AR1         ; get page counter
	  BANZ      MC16,*-,AR0 ; see if all pages (of 256) are done

	  LACK      '6'
	  SACL      MON3
	  CALL      OUTBIT2   ; print 6

; address uniqueness test.  Write to PROGRAM RAM with data value
;   equal to address of the location.
;
;  AR0 - points to data RAM block of 256 words
;  AR1 - counts #words within each page of 256
;  AR2 - counts up within each page of 256
;  AR3 - counts program ram pages
;  PGMPTR - holds page# of PROGRAM RAM
;  TEMPDAT - holds data while building value

	  LALK      PROGBOT
	  SACL      PGMPTR            ; initialize program pointer to bottom
	  LRLK      AR3,PPAGCNT       ; do pages of 256 words

; fill data RAM with data = address of program RAM

MC18      LRLK      AR0, 02000h       ; from 2000..20FF in data RAM
	  LRLK      AR1, 0FFh         ; size of block = 256
	  LRLK      AR2, 0h           ; counts position within block
MC19      SAR       AR2,TEMPDAT
	  LAC       PGMPTR            ; page counter to upper byte
	  ADD       TEMPDAT           ; data value = address location
	  SACL      *+,0,AR2          ; write to data RAM buffer, incr AR0
	  MAR       *+,AR1            ; increment AR2
	  BANZ      MC19,*-,AR0       ; decrement counter AR1

; move the data ram block to program RAM

	  LRLK      AR0,2000h         ; AR0 points to 2000h in data memory
	  LAC       PGMPTR            ; Accum points to PROG memory, set AR0
	  RPTK      0ffh              ; repeat instruction 256 times
	  TBLW      *+                ; Move from data-mem to program-mem

; do next page

	  ADLK      100h              ; point to next page
	  SACL      PGMPTR            ; save pointer
	  LARP      AR3               ; get page counter
	  BANZ      MC18,*-,AR0       ; see if all pages (of 256) are done

;
; now read back program RAM, and check value
;

	  LALK      PROGBOT
	  SACL      PGMPTR      ; initialize program pointer to bottom
	  LRLK      AR3,PPAGCNT ; do pages of 256 words
	  LARP      AR0

; read each page in program RAM to 3000..30FF in data RAM

MC20      LRLK      AR0,3000h   ; AR0 points to 3000h in data memory
	  LAC       PGMPTR      ; Accum points to PROGRAM memory
	  RPTK      0ffh        ; repeat instruction 256 times
	  TBLR      *+          ; Move from program-mem to data-mem

; see if 3000h..30ffh in data RAM is correct value

	  LRLK      AR0, 03000h       ; start at 3000h
	  LRLK      AR1, 0FFh         ; for 100h words
	  LRLK      AR2, 0h           ; counts from 0..FF in page
MC21      SAR       AR2, TEMPDAT
	  LAC       PGMPTR            ; page counter to upper byte
	  ADD       TEMPDAT           ; data value = address location     
	  SUB       *+,0,AR2          ; load accum, incr pointer AR0
	  BNZ       ERR07             ; ERR07 if not zero
	  MAR       *+,AR1            ; incr AR2
	  BANZ      MC21,*-,AR0       ; do 256 times

; do next page of program RAM

	  LAC       PGMPTR
	  ADLK      100h              ; point to next program RAM page
	  SACL      PGMPTR            ; save pointer
	  LARP      AR3               ; get page counter
	  BANZ      MC20,*-,AR0       ; see if all pages (of 256) are done

	  LACK      '7'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 7


	  CALL      SD_CRLF     ; no errors - all tests OK
	  B         START       ; loop through tests forever

; --- errors in memory test come here

ERR01     LACK      '1'
	  B         ERRPRNT
ERR02     LACK      '2'
	  B         ERRPRNT
ERR03     LACK      '3'
	  B         ERRPRNT
ERR04     LACK      '4'
	  B         ERRPRNT
ERR05     LACK      '5'
	  B         ERRPRNT
ERR06     LACK      '6'
	  B         ERRPRNT
ERR07     LACK      '7'
	  B         ERRPRNT
ERR08     LACK      '8'
	  B         ERRPRNT

ERRPRNT   LDPK      08h         ; page context that makes Monitor happy
	  PUSH                  ; save error code
	  LACK      'E'
	  SACL      MON3
	  CALL      OUTBIT2     ; print 'E'
	  LACK      ':'
	  SACL      MON3
	  CALL      OUTBIT2     ; print ':'
	  POP                   ; restore error code
	  SACL      MON3
	  CALL      OUTBIT2     ; print error code
	  CALL      SD_CRLF

	  CALL      RESET       ; grind to a halt on error
	  

; ---------------- rcv, tx interrupts do nothing ----------------------

RINT      EINT
	  RET

XINT      EINT
	  RET

; --------------------- Time delay subroutines ----------------------

WAIT4     LARK      AR2,0FFh
	  CALL      WAIT2
	  CALL      WAIT2

	  RET

WAIT2     LARK      AR2,0FFh
	  CALL      WAIT1
	  CALL      WAIT1

	  RET

WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
	  BANZ      WAIT_B,*-  ; Loop through count

	  LARP      AR2
	  BANZ      WAIT_A,*-  ; Loop through count

	  RET


	  .END
