;  Experimental 1200 bps AFSK modem (Bell 202)
;
;  To assemble, use the command "asm pkt.asm -dRn -dGn"
;    where Rn is R1 or R2 for radio ports 1 or 2 and
;    where Gn is one of G0, G1, G2, G3, G4, G5, or G6 to set the gain
;
;  LED203 is DCD
;  LED204 is PTT
;  LED205 is ~DCD
;  LED206 is level indicator
;  LED207 is duty cycle
;  LED208 is power
;
;  Version for TAPR/AMSAT DSP-93, File - PKT.ASM
;  Derived from File - PKT_93A.ASM
;  Updates for conditional assembly and includes by Ron Parsons W5RKN
;  TLC32044 AIO @ 14,204.55 samples/sec (70.4 sec/sample), 40 MHz CPU clock
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Uses TNC disconnect header (set TNC to 1200 bps!!)
;  Updates:
;    TX PTT lock out added
;    Make TX audio only if PTT on (RGP)
;    Added LED204 as PTT indicator (RGP)
;    Added LED203 as DCD indicator (RGP)
;    Added reset command (RGP)
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1991, 1992, 1993 
; and 1994, Frank H. Perkins, Jr., WB5IPM. All rights reserved.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact Greg Jones at TAPR or the author for information on
; commercial licenses of this software.
; 
; Frank H. Perkins, Jr.
;
#include  "REGS.INC"
#include  "PORTS.INC"
#include  "MONITOR.INC"
#INCLUDE  "SERIAL.INC"

#ifdef    R1
REC_IN    .EQU      REC_IN_1
#endif
#ifdef    R2
REC_IN    .EQU      REC_IN_2
#endif

#ifdef    G0
GAIN      .EQU      GAIN0
#endif
#ifdef    G1
GAIN      .EQU      GAIN2
#endif
#ifdef    G2
GAIN      .EQU      GAIN2
#endif
#ifdef    G3
GAIN      .EQU      GAIN3
#endif
#ifdef    G4
GAIN      .EQU      GAIN4
#endif
#ifdef    G5
GAIN      .EQU      GAIN5
#endif
#ifdef    G6
GAIN      .EQU      GAIN6
#endif

;  global variables
BUFI      .EQU      060h      ;  AIO input buffer
BUFO      .EQU      061h      ;  AIO output buffer

DO        .EQU      062h      ;  data output buffer
SO        .EQU      063h      ;  sig buffer 

DL0       .EQU      064h      ;  delay buffer 0
DL1       .EQU      065h      ;  delay buffer 1
DL2       .EQU      066h      ;  delay buffer 2     

RB        .EQU      067h      ;  ramp buffer
TS        .EQU      068h      ;  sine buffer
DI        .EQU      069h      ;  TX data in buffer
DC        .EQU      070h      ;  DCD count down buffer
RO        .EQU      071h      ;  radio control buffer (PTT)
TXE       .EQU      072h      ;  TX enable counter buffer

TEMPBYTE  .EQU      078h      ;  temporary location used by CK_RESET

CMDA      .EQU      CMDA_VAL(16)    ;  AIO load RA/TA = 16 for SCF @ 312.5 kHz
CMDB      .EQU      CMDB_VAL(22)    ;  AIO load RB/TB = 22 for 14,204.55 samples/sec
;  AIO configuration: HPF in, Sync, 3 V half scale, sin x/x
CMDC      .EQU      AIO_CONFIG|AIO_INSADHPF|AIO_SYNC|AIO_GAIND60|AIO_GAIND70|AIO_INSSINX

TOP       .EQU      04000h    ;  ramp top value
INCL      .EQU      0563h     ;  incr value for 1.2 kHz
INCH      .EQU      09E0h     ;  incr value for 2.2 kHz

DCDC      .EQU      02C9h     ;  50 ms count for DCD
LEV       .EQU      96        ;  level threshold
TXEC      .EQU      400       ;  TX enable count


CFG       .EQU      02h       ;  AIO CFG buffer (data mem page 8)


;  TASM does not assume the necessary 1000h starting address
          .ORG      1000h


          B         GO        ;  branch to program start (TINT)
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)


GO        DINT                ;  disable interrupts
          LDPK      USER_PAGE ;  data mem page 8 (external)

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       0         ;  set P shift = 0

AIO_RST   LALK      AIO_RESET|GAIN|REC_IN     ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      LED_202|GAIN|REC_IN     ;  LED 202 on
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LALK      AIO_ENABLE|GAIN|REC_IN     ;  load AIO run
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LDPK      BASE_PAGE ;  data mem page 0 (internal)

          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          SACL      IMR       ;  store INT mask

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts
          LALK      0030h     ;  mask SIO RX and TX interrupts active
          SACL      IMR       ;  store mask in IMR

DSP_INI   ZAC                 ;  zero A
          SACL      DO        ;  zero data out buffer
          SACL      SO        ;  zero sig buffer

          SACL      DL0       ;  zero delay buffer 0
          SACL      DL1       ;  zero delay buffer 1
          SACL      DL2       ;  zero delay buffer 2

          SACL      RB        ;  zero ramp buffer
          SACL      TS        ;  zero sine buffer
          SACL      DI        ;  zero TX data in buffer
          SACL      DC        ;  zero DC buffer

          LALK      TXEC      ;  get TX enable count
          SACL      TXE       ;  store in TX enable buffer

          ZAC                 ;  zero A

FIR_INI   CNFD                ;  make B0 data mem for coefs load

          LARP     AR0        ;  AR0 indirect addr pointer
          LRLK     AR0,0200h  ;  point ARO to block B0
          RPTK     47         ;  repeat (47 + 1) times
          BLKP     BPF,*+     ;  to xfer 48 BPF coeffs to B0

          LRLK     AR0,0230h  ;  point ARO to block B0
          RPTK     31         ;  repeat (31 + 1) times
          BLKP     LPF,*+     ;  to xfer 32 LPF coeffs to B0

          CNFP                ;  make B0 prgm mem

          EINT                ;  enable interrupts             


LOOP      NOP
          B         LOOP      ;  loop here between interrupts


RINT      LAC       DRR       ;  get received AIO data sample from DRR
          SACL      BUFI      ;  store DDR in AIO input buffer
          CALL      DSP       ;  call DSP process

          EINT
          RET


XINT      LAC       BUFO      ;  get data from AIO output buffer
          SACL      DXR       ;  store data in DXR to transmit to AIO

          EINT
          RET

DSP       LAC       DO        ;  load DO
          ORK       KEYI_207  ;  LED 207 is duty cycle
          SACL      DO        ;  store DO
          OUT       DO,TNC_OUTPUT    ;  OUT to TNC

          LAC       BUFI,9    ;  3 V nominal signal input
          SACH      SO        ;  7 bits in SO

LEVEL     LAC       SO        ;  get SO
          SBLK      LEV       ;  subtract LEV

          BLZ       L_LO      ;  if < 0 goto L_LO

L_HI      LAC       DO        ;  load DO
          ORK       KEYI_206  ;  OR to set level LED
          SACL      DO        ;  store DO

          B         BP_FIR    ;  goto BP_FIR

L_LO      LAC       DO        ;  load DO
          ANDK      KEYI_206_OFF    ;  AND to reset level LED
          SACL      DO        ;  store DO

BP_FIR    LAC       SO        ;  get SO
          LRLK      AR2,03CFh ;  03CFh is start of delay line
          LARP      AR2       ;  AR2 pointer to 03CFh
          SACL      *         ;  store SO 

          LRLK     AR1,03FEh  ;  03FEh is end of delay line 
          LARP     AR1        ;  AR1 pointer to 03FEh  

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     47         ;  repeat (47 + 1) times to do FIR
          MACD     0FF00h,*-  ;  MACD up thru FIR coeffs, down thru data
          APAC                ;  accum last product
          SACH     SO,03h     ;  store hi, 2^13 * 2^3 * 2^-16 norm

          DMOV      DL1       ;  copy DL1 into DL2
          DMOV      DL0       ;  copy DL0 into DL1
          LAC       SO        ;  get SO
          SACL      DL0       ;  copy SO into DL0
          NEG                 ;  * -1
          SACL      SO        ;  store SO

          LT        DL2       ;  copy DL2 into T
          MPY       SO        ;  DL2 * SO into P
          PAC                 ;  P into A

LP_FIR    LRLK      AR2,03AEh ;  03AEh is start of delay line
          LARP      AR2       ;  AR2 pointer to start of delay line
          SACL      *         ;  store SO 

          LRLK     AR1,03CDh  ;  03CDh is end of data delay line 
          LARP     AR1        ;  AR1 pointer to data delay line start

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     31         ;  repeat (31 + 1) times to do FIR
          MACD     0FF30h,*-  ;  MACD, up thru B0 coeffs, down thru B1 data
          APAC                ;  accum last product
          SACH     SO,03h     ;  store A hi -> 2^13 * 2^3 * 2^-16 norm

          LAC       SO        ;  get SO
          ADLK      0080h     ;  add slicer offset
          SACL      SO        ;  store SO (diagnostic)  

          BGEZ      HI        ;  if => 0 goto HI

LO        LAC       DO        ;  else get DO
          ANDK      ~RDI      ;  AND DATA OUT to 0
          SACL      DO        ;  store in data out

          LALK      DCDC      ;  get DCD count
          SACL      DC        ;  back into buffer

          B         TX_ONOF   ;  branch to TX_ONOF

HI        LAC       DO        ;  get DO
          ORK       RDI       ;  OR DATA OUT to 1
          SACL      DO        ;  store in data out

TX_ONOF   LAC       TXE       ;  get count in TXE

          BLZ       TX_OFF    ;  if < 0 goto TX_OFF

          IN        DI,TNC_INPUT    ;  get TNC input port
          LAC       DI        ;  load data
          ANDK      RTSO      ;  isolate PTT (*RTS) bit

          BNZ       TX_OFF    ;  if <> 0 goto TX_OFF

TX_ON     LAC       RO        ;  get RO

#ifdef    R1
          ORK       PTT_R1    ;  turn PTT on for radio 1
#endif
#ifdef    R2
          ORK       PTT_R2    ;  turn PTT on for radio 2
#endif
          SACL      RO        ;  store RO

          ORK       LED_204   ;  turn PTT LED 204 on
          SACL      RO        ;  store RO

          B         TX        ;  goto TX

TX_OFF    LAC       RO        ;  get RO
#ifdef    R1
          ANDK      PTT_R1_OFF    ;  turn PTT off for radio 1
#endif
#ifdef    R2
          ANDK      PTT_R2_OFF    ;  turn PTT off for radio 2
#endif
          SACL      RO        ;  store RO

          ANDK      LED_204_OFF   ;  turn PTT LED 204 off
          SACL      RO        ;  store RO

          B         DOUT      ;  Make TX audio only if PTT on

TX        IN        DI,TNC_INPUT    ;  get input TNC port
          LAC       DI        ;  load data
          ANDK      TDO       ;  isolate TRANSMIT DATA bit

          BNZ       THI       ;  if <> 0 goto THI

TLO       LAC       TXE       ;  get count in TXE

          BLZ       TL        ;  if < 0 goto TL

          SUBK      1         ;  else decrement count in TXE
          SACL      TXE       ;  and store

TL        LAC       RB        ;  get last ramp value
          ADLK      INCL      ;  add space incr value to ramp

          B         WRAP      ;  goto WRAP

THI       LALK      TXEC      ;  get TXEC count
          SACL      TXE       ;  reset TXE
          LAC       RB        ;  get last ramp value
          ADLK      INCH      ;  add mark incr value to ramp

WRAP      SACL      RB        ;  store back in RB
          SBLK      TOP       ;  subtract TOP value

          BLEZ      GETS      ;  ramp not > top, go GETSC

          SACL      RB        ;  else put wrapped value in ramp buffer

GETS      ZAC                 ;  zero A
          ADD       RB,9      ;  get ramp offset * 2^7, shift 2^9
          SACH      TS        ;  store high for 2^-16 normalize
          LALK      TRG_TBL   ;  point to trig table
          ADD       TS        ;  add normalized ramp offset
          TBLR      TS        ;  read trig value * 2^10 into sine buff

TX_OUT    LAC       TS,2      ;  get sine value, shift 2^2
          ANDK      0FFFCh    ;  mask out AIO control bits
          SACL      BUFO      ;  AIO out

DOUT      LAC       DC        ;  get DC

          BLEZ      NODCD     ;  if <= 0 goto NODCD

          SUBK      1         ;  else decrement DC
          SACL      DC        ;  store

          B         DCD       ;  goto DCD

NODCD     LAC       DO        ;  get DO
          ORK       CDI       ;  set DCD bit high
          SACL      DO        ;  store DO

          LAC       RO
          ANDK      LED_203_OFF   ;  set DCD LED 203 off
          SACL      RO        ;  store RO

          B         END_DSP   ;  goto END_DSP
          
DCD       LAC       DO        ;  get DO
          ANDK      CDI_OFF   ;  set DCD bit low
          SACL      DO        ;  store DO

          LAC       RO
          ORK       LED_203   ;  set DCD LED 203 on
          SACL      RO        ;  store RO

END_DSP
          CALL      CK_RESET  ; Should we reset?

          LAC       DO        ;  load DO
          ANDK      KEYI_207_OFF    ;  reset duty cycle
          SACL      DO        ;  store DO
          OUT       DO,TNC_OUTPUT    ;  output to TNC port

          OUT       RO,RADIO_CTRL    ;  output PTT to port

          RET                 ;  done!!

;
; This is a modification of the ROM based INBIT code that is used to 
; reset if a 'R' is received, or to return if no key is waiting.

CK_RESET
        LALK    UART_SEL_LSR    ; Select the line status register
        SACL    TEMPBYTE        ; Use temp memory
        OUT     TEMPBYTE,UART_CTRL   ; Select the register to read
        ZAC
        ZAC

        IN      TEMPBYTE,UART_READ   ; Get the Line CTRL on SIO (get the selected reg)
        LACK    UART_LSR_RBF    ; Set bit for receiver holding register
        AND     TEMPBYTE        ; Test the 0th bit Line CTRL 
        BZ      CK_RET          ; Receive Holding Register (not) Empty

        ZAC
        SACL    TEMPBYTE
        OUT     TEMPBYTE,UART_CTRL
        IN      TEMPBYTE,UART_READ
        LAC     TEMPBYTE
        ANDK    05Fh            ; match upper or lower case
        SUBK    'R'             ; Reset DSP-93
        BZ      RESET

CK_RET
        RET                     ; Continue


WAIT4
          CALL      WAIT2
          CALL      WAIT2

          RET


WAIT2
          CALL      WAIT1
          CALL      WAIT1

          RET


WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ; Loop through count

          LARP      AR2
          BANZ      WAIT_A    ; Loop through count

          RET


          .MSFIRST

BPF       .WORD         2     ;  hamming bpf, 48 taps, scaled 2^13, 2.8% <-> 20.8%
          .WORD        -2
          .WORD         9
          .WORD        21
          .WORD        12
          .WORD       -18
          .WORD       -27
          .WORD         8
          .WORD        31
          .WORD       -31
          .WORD      -136
          .WORD      -144
          .WORD       -34
          .WORD        -7
          .WORD      -213
          .WORD      -448
          .WORD      -366
          .WORD       -36
          .WORD       -16
          .WORD      -587
          .WORD     -1110
          .WORD      -560
          .WORD      1144
          .WORD      2710
          .WORD      2710
          .WORD      1144
          .WORD      -560
          .WORD     -1110
          .WORD      -587
          .WORD       -16
          .WORD       -36
          .WORD      -366
          .WORD      -448
          .WORD      -213
          .WORD        -7
          .WORD       -34
          .WORD      -144
          .WORD      -136
          .WORD       -31
          .WORD        31
          .WORD         8
          .WORD       -27
          .WORD       -18
          .WORD        12
          .WORD        21
          .WORD         9
          .WORD        -2
          .WORD         2

LPF       .WORD        13     ; hamming lpf, 32 taps, scaled 2^13, 8.4%
          .WORD        16
          .WORD        17
          .WORD        10
          .WORD       -11
          .WORD       -50
          .WORD      -100
          .WORD      -141
          .WORD      -143
          .WORD       -75
          .WORD        83
          .WORD       330
          .WORD       637
          .WORD       952
          .WORD      1211
          .WORD      1357
          .WORD      1357
          .WORD      1211
          .WORD       952
          .WORD       637
          .WORD       330
          .WORD        83
          .WORD       -75
          .WORD      -143
          .WORD      -141
          .WORD      -100
          .WORD       -50
          .WORD       -11
          .WORD        10
          .WORD        17
          .WORD        16
          .WORD        13

TRG_TBL   .WORD    00h        ;  sine table, scaled 2^10, 128 + 1 entries
          .WORD    032h
          .WORD    064h
          .WORD    096h
          .WORD    0C8h
          .WORD    0F9h
          .WORD    0129h
          .WORD    0159h
          .WORD    0188h
          .WORD    01B6h
          .WORD    01E3h
          .WORD    020Eh
          .WORD    0239h
          .WORD    0262h
          .WORD    028Ah
          .WORD    02B0h
          .WORD    02D4h
          .WORD    02F7h
          .WORD    0318h
          .WORD    0336h
          .WORD    0353h
          .WORD    036Eh
          .WORD    0387h
          .WORD    039Eh
          .WORD    03B2h
          .WORD    03C4h
          .WORD    03D4h
          .WORD    03E1h
          .WORD    03ECh
          .WORD    03F5h
          .WORD    03FBh
          .WORD    03FFh
          .WORD    0400h
          .WORD    03FFh
          .WORD    03FBh
          .WORD    03F5h
          .WORD    03ECh
          .WORD    03E1h
          .WORD    03D4h
          .WORD    03C4h
          .WORD    03B2h
          .WORD    039Eh
          .WORD    0387h
          .WORD    036Eh
          .WORD    0353h
          .WORD    0336h
          .WORD    0318h
          .WORD    02F7h
          .WORD    02D4h
          .WORD    02B0h
          .WORD    028Ah
          .WORD    0262h
          .WORD    0239h
          .WORD    020Eh
          .WORD    01E3h
          .WORD    01B6h
          .WORD    0188h
          .WORD    0159h
          .WORD    0129h
          .WORD    0F9h
          .WORD    0C8h
          .WORD    096h
          .WORD    064h
          .WORD    032h
          .WORD    00h
          .WORD    0FFCEh
          .WORD    0FF9Ch
          .WORD    0FF6Ah
          .WORD    0FF38h
          .WORD    0FF07h
          .WORD    0FED7h
          .WORD    0FEA7h
          .WORD    0FE78h
          .WORD    0FE4Ah
          .WORD    0FE1Dh
          .WORD    0FDF2h
          .WORD    0FDC7h
          .WORD    0FD9Eh
          .WORD    0FD76h
          .WORD    0FD50h
          .WORD    0FD2Ch
          .WORD    0FD09h
          .WORD    0FCE8h
          .WORD    0FCCAh
          .WORD    0FCADh
          .WORD    0FC92h
          .WORD    0FC79h
          .WORD    0FC62h
          .WORD    0FC4Eh
          .WORD    0FC3Ch
          .WORD    0FC2Ch
          .WORD    0FC1Fh
          .WORD    0FC14h
          .WORD    0FC0Bh
          .WORD    0FC05h
          .WORD    0FC01h
          .WORD    0FC00h
          .WORD    0FC01h
          .WORD    0FC05h
          .WORD    0FC0Bh
          .WORD    0FC14h
          .WORD    0FC1Fh
          .WORD    0FC2Ch
          .WORD    0FC3Ch
          .WORD    0FC4Eh
          .WORD    0FC62h
          .WORD    0FC79h
          .WORD    0FC92h
          .WORD    0FCADh
          .WORD    0FCCAh
          .WORD    0FCE8h
          .WORD    0FD09h
          .WORD    0FD2Ch
          .WORD    0FD50h
          .WORD    0FD76h
          .WORD    0FD9Eh
          .WORD    0FDC7h
          .WORD    0FDF2h
          .WORD    0FE1Dh
          .WORD    0FE4Ah
          .WORD    0FE78h
          .WORD    0FEA7h
          .WORD    0FED7h
          .WORD    0FF07h
          .WORD    0FF38h
          .WORD    0FF6Ah
          .WORD    0FF9Ch
          .WORD    0FFCEh
          .WORD    00h


          .END
