;  Experimental 1200 bps PSK modem, 95.03.31 @ 22:00 CST
;  Version for TAPR/AMSAT DSP-93, File - PSK93R2J.ASM
;  TLC32044 AIO @ 15,432.1 samples/sec (64.8 sec/sample), 40 MHz CPU clock
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Uses radio port #2, TNC disconnect header
;  Set TNC to FULLDUP ON, at 1200 bps !!!
;  Radio tuning via "pulse" method
;  Target radio use - Icom 275/475 etc...
;  Updates:
;    TX PTT lock out added when TNC turned off
;    SmartTune lock detector added
;    reduced level threshold from 850 to 650
;    added lock detect and "hot" DCD
;    added PTT LED
;    added reset via UART
;    660 Hz Bessel IIR
;  Trig table set up for cos 45 deg
;  This software offered AS IS, without warranty or guarantee of any kind!
;  No cost, non-exclusive license granted to all users of the TAPR/AMSAT
;  DPS-93 kit (Amateur Radio use only)
;  Phone/FAX (817) 461 0780, CI$ 76610.170, Internet 76610.170@compuserve.com
;  Copyright Frank H. Perkins, Jr., WB5IPM, 1991, 1992, 1993, 1994, 1995
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1991, 1992, 1993 
; and 1994, Frank H. Perkins, Jr., WB5IPM. All rights reserved.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact Greg Jones at TAPR or the author for information on
; commercial licenses of this software.
; 
; Frank H. Perkins, Jr.

AR0       .EQU      00h       ;  aux register 0 (equate used by TASM)
AR1       .EQU      01h       ;  aux register 1 (equate used by TASM)
AR2       .EQU      02h       ;  aux register 2 (equate used by TASM)
AR3       .EQU      03h       ;  aux register 3 (equate used by TASM)
AR4       .EQU      04h       ;  aux register 4 (equate used by TASM)
AR5       .EQU      05h       ;  aux register 5 (equate used by TASM)
AR6       .EQU      06h       ;  aux register 6 (equate used by TASM)
AR7       .EQU      07h       ;  aux register 7 (equate used by TASM)

DRR       .EQU      00h       ;  SIO (AIO) receive buffer
DXR       .EQU      01h       ;  SIO (AIO) transmit buffer
TIM       .EQU      02h       ;  TIMER countdown buffer
PRD       .EQU      03h       ;  TIMER period buffer
IMR       .EQU      04h       ;  INT mask buffer

BUFI      .EQU      060h      ;  AIO input buffer
BUFO      .EQU      061h      ;  AIO output buffer

IS0       .EQU      062h      ;  IIR buffer IS0
IS1       .EQU      063h      ;  IIR buffer IS1
IS2       .EQU      064h      ;  IIR buffer IS2

QS0       .EQU      065h      ;  IIR buffer QS0
QS1       .EQU      066h      ;  IIR buffer QS1
QS2       .EQU      067h      ;  IIR buffer QS2

S0        .EQU      068h      ;  PLL IIR buffer S0
S1        .EQU      069h      ;  PLL IIR buffer S1

RB        .EQU      06Ah      ;  RX VCO ramp buffer
TS        .EQU      06Bh      ;  sine buffer
TC        .EQU      06Ch      ;  cosine buffer

PD        .EQU      06Dh      ;  phase det buffer
TN        .EQU      06Eh      ;  tuning buffer

STS       .EQU      06Fh      ;  sig buffer; 1.5 nominal 
TI        .EQU      070h      ;  I channel buffer
TQ        .EQU      071h      ;  Q channel buffer 

DO        .EQU      072h      ;  data out buffer

DI        .EQU      073h      ;  TNC data in buffer            
TXD       .EQU      074h      ;  TX data buffer
LTXD      .EQU      075h      ;  last TX data buffer
TRB       .EQU      076h      ;  TX ramp buffer
TXS       .EQU      077h      ;  TX signal buffer

TF0       .EQU      078h      ;  IIR tune filter buffer TF0
TF1       .EQU      079h      ;  IIR tune filter buffer TF1

DF0       .EQU      07Ah      ;  IIR decm filter buffer TF0
DF1       .EQU      07Bh      ;  IIR decm filter buffer TF1
DCNT      .EQU      07Ch      ;  decimation count buffer

PCNT      .EQU      07Dh      ;  tuning pulse count buffer
RO        .EQU      07Eh      ;  radio control buffer (PTT)

TXE       .EQU      07Fh      ;  TX enable counter buffer

LIS0      .EQU      020h       ;  IIR buffer LIS0
LIS1      .EQU      021h       ;  IIR buffer LIS1
LIS2      .EQU      022h       ;  IIR buffer LIS2

LQS0      .EQU      023h       ;  IIR buffer LQS0
LQS1      .EQU      024h       ;  IIR buffer LQS1
LQS2      .EQU      025h       ;  IIR buffer LQS2

LTS       .EQU      026h       ;  L (cos - 45) buffer
LTC       .EQU      027h       ;  L (cos + 45) buffer
LPD       .EQU      028h       ;  L phase det buffer

LSI       .EQU      029h       ;  L sig in buffer 
LTI       .EQU      02Ah       ;  L -45 channel buffer
LTQ       .EQU      02Bh       ;  L +45 channel buffer 

LS0       .EQU      02Ch       ;  IIR buffer LS0
LS1       .EQU      02Dh       ;  IIR buffer LS1
LS        .EQU      02Eh       ;  buffer LS

UBUF      .EQU      02Fh      ;  UART buffer
SYMBI     .EQU      030h      ;  input symbol buffer
EYE       .EQU      031h      ;  eye patten buffer

CMDA      .EQU      02448h    ;  AIO load RA/TA = 18 for SCF @ 277.777 kHz
CMDB      .EQU      0244Ah    ;  AIO load RB/TB = 18 for 15,432.1 samples/sec
CMDC      .EQU      0263h     ;  AIO config cmd,  3 V input, AIO sync, no HPF

TOP       .EQU      04000h    ;  ramp top value = 2^14
INC       .EQU      0688h     ;  RX VCO ramp incr value for 1575 Hz
TINC      .EQU      04FAh     ;  TX ramp incr value for 1200 Hz

G0        .EQU      02Eh      ;  IIR COEF G0 (scaled 2^11)
G1        .EQU      05Bh      ;  IIR COEF G1 (scaled 2^11)
G2        .EQU      02Eh      ;  IIR COEF G2 (scaled 2^11)
H1        .EQU      0BB1h     ;  IIR COEF H1 (scaled 2^11)
H2        .EQU      01B98h    ;  IIR COEF H2 (scaled 2^11, 13 bit 2's comp)

GA        .EQU      0799h     ;  PLL COEF GA (scaled 2^13)
GB        .EQU      01907h    ;  PLL COEF GB (scaled 2^13, 13 bit 2's comp)
H         .EQU      07D4h     ;  PLL COEF H (scaled 2^11)

TFGA      .EQU      034h      ;  TUNE FIL COEF TFGA (scaled 2^12)
TFGB      .EQU      034h      ;  TUNE FIL COEF TFGB (scaled 2^12)
TFH       .EQU      07DDh     ;  TUNE FIL COEF TFH (scaled 2^11)

CNT       .EQU      10        ;  decimation count

RTL       .EQU      020h      ;  radio low tune limit (tune)
RTH       .EQU      00h       ;  radio high tune limit (no tune)
LEV       .EQU      650       ;  level threshold
LLEV      .EQU      60        ;  lock threshold

CNTH      .EQU      288       ;  tuning pulse load counter value
CNTL      .EQU      186       ;  tuning pulse hi -> lo transition value
TXEC      .EQU      400       ;  TX enable count

CFG       .EQU      02h       ;  AIO CFG buffer (data mem page 8)


          .ORG      1000h


          B         GO        ;  branch to program start (TINT)
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)


GO        DINT                ;  disable interrupts
          LDPK      08h       ;  data mem page 8 (external)

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       00h       ;  set P shift = 0

AIO_RST   LALK      8037h     ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      0077h     ;  load AIO reset, radio 2, gain -> 50
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LALK      4037h     ;  load AIO run, radio 2, gain -> 50
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,07h   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LDPK      00h       ;  data mem page 0 (internal)

          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          SACL      IMR       ;  store INT mask

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      0003h     ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts
          LALK      0010h     ;  mask SIO RX interrupt active
          SACL      IMR       ;  store mask in IMR

DSP_INI   ZAC                 ;  zero A

          SACL      IS0       ;  zero buffer
          SACL      IS1       ;  zero buffer
          SACL      IS2       ;  zero buffer

          SACL      QS0       ;  zero buffer
          SACL      QS1       ;  zero buffer
          SACL      QS2       ;  zero buffer

          SACL      S0        ;  zero buffer
          SACL      S1        ;  zero buffer

          SACL      RB        ;  zero buffer
          SACL      TS        ;  zero buffer
          SACL      TC        ;  zero buffer

          SACL      PD        ;  zero buffer
          SACL      TN        ;  zero buffer

          SACL      STS       ;  zero buffer
          SACL      TI        ;  zero buffer
          SACL      TQ        ;  zero buffer

          SACL      DO        ;  zero buffer

          SACL      DI        ;  zero buffer
          SACL      TXD       ;  zero buffer
          SACL      LTXD      ;  zero buffer
          SACL      TRB       ;  zero buffer
          SACL      TXS       ;  zero buffer
          SACL      RO        ;  zero buffer

          SACL      TF0       ;  zero buffer
          SACL      TF1       ;  zero buffer

          SACL      DF0       ;  zero buffer
          SACL      DF1       ;  zero buffer

          LACK      CNT       ;  get CNT value
          SACL      DCNT      ;  preset DCNT

          LALK      CNTH      ;  get CNTH value
          SACL      PCNT      ;  preset PCNT

          LALK      TXEC      ;  get TX enable count
          SACL      TXE       ;  store in TX enable buffer

          LDPK      06h       ;  data mem page 6

          ZAC                 ;  zero A

          SACL      LIS0      ;  zero buffer
          SACL      LIS1      ;  zero buffer
          SACL      LIS2      ;  zero buffer

          SACL      LQS0      ;  zero buffer
          SACL      LQS1      ;  zero buffer
          SACL      LQS2      ;  zero buffer

          SACL      LTS       ;  zero buffer
          SACL      LTC       ;  zero buffer

          SACL      LTI       ;  zero buffer
          SACL      LTQ       ;  zero buffer
          SACL      LSI       ;  zero buffer

          SACL      LS0       ;  zero buffer
          SACL      LS1       ;  zero buffer
          SACL      LS        ;  zero buffer

          SACL      UBUF      ;  zero buffer
          SACL      SYMBI     ;  zero buffer

          LDPK      00h       ;  data mem page 0

PNTR_INI  LRLK      AR0,07Eh  ;  point AR0 to 07Eh

          ZAC                 ;  zero A

UART_INI  LDPK      06h       ;  data mem page 6
          ZAC                 ;  UART RX/TX buffer addr
          SACL      UBUF      ;  into UBUF
          OUT       UBUF,03   ;  and OUT to UART
          IN        SYMBI,05h ;  read UART RX (clean)
          LDPK      00h       ;  data mem page 0

          EINT                ;  enable interrupts             


LOOP      NOP
          B         LOOP      ;  loop here between interrupts


RINT      LAC       DRR       ;  get received AIO data sample from DRR
          SACL      BUFI      ;  store DDR in AIO input buffer

          CALL      DSP       ;  call DSP process

          EINT
          RET


XINT      LAC       BUFO      ;  get data from AIO output buffer
          SACL      DXR       ;  store data in DXR to transmit to AIO

          EINT
          RET


DSP       LAC       DO        ;  load DO
          ORK       0400h     ;  LED 202 is duty cycle
          SACL      DO        ;  store DO
          OUT       DO,06h    ;  OUT to TNC

SIG       LAC       BUFI,0Ch  ;  1.5 V nominal signal input
          SACH      STS       ;  normalize 2^10

LEVEL     LAC       STS       ;  get STS
          SBLK      LEV       ;  subtract LEV

          BLZ       L_LO      ;  if < 0 goto L_LO

L_HI      LAC       DO        ;  load DO
          ORK       0200h     ;  OR to set level LED
          SACL      DO        ;  store DO

          B         VCO       ;  goto VCO

L_LO      LAC       DO        ;  load DO
          ANDK      0FDFFh    ;  AND to reset level LED
          SACL      DO        ;  store DO

VCO       LAC       RB        ;  get last ramp value
          ADLK      INC       ;  add incr value
          ADD       TN        ;  add tuning value
          SACL      RB        ;  store RB
          SBLK      TOP       ;  subtract TOP value

          BLEZ      GETSC     ;  ramp <= TOP, goto GETSC

          SACL      RB        ;  else wrap ramp

GETSC     LAC       RB,9      ;  get ramp offset * 2^7, shift 2^9
          SACH      TS        ;  store high to normalize
          LALK      TRG_TBL   ;  point to trig table
          ADD       TS        ;  add normalized ramp offset
          TBLR      TS        ;  read trig value * 2^10 into sine buff
          ADDK      020h      ;  add cosine offset
          TBLR      TC        ;  read trig value * 2^10 into cosine buff

IMUL      LT        STS       ;  load T from STS
          MPY       TS        ;  STS * TS -> P
          PAC                 ;  P -> A
          SACH      TI,6      ;  normalize

QMUL      LT        STS       ;  load T from STS
          MPY       TC        ;  STS * TC -> P
          PAC                 ;  P -> A
          SACH      TQ,6      ;  normalize

I_LPF     ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          DMOV      IS1       ;  copy IS1 up to IS2
          DMOV      IS0       ;  copy IS0 up to IS1
          LT        IS2       ;  load T from IS2
          MPYK      H2        ;  S2 * H2 to P
          LTA       IS1       ;  add P to A, load T from IS1
          MPYK      H1        ;  IS1 * H1 to P
          APAC                ;  add P to A
          ADD       TI,0Bh    ;  shift input sample 2^11, add 
          SACH      IS0,5     ;  normalize

          ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          LT        IS2       ;  load T from IS2
          MPYK      G0        ;  IS2 * G0 to P
          LTA       IS1       ;  add P to A, load T from IS1
          MPYK      G1        ;  IS1 * G1 to P
          LTA       IS0       ;  add P to A, load T from IS0
          MPYK      G2        ;  IS0 * G2 to P
          APAC                ;  add P to A
          SACH      TI,5      ;  normalize

          LAC       TI        ;  get TI
          LDPK      06h       ;  data mem page 6
          SACL      EYE       ;  store TI
          LDPK      00h       ;  data mem page 0

Q_LPF     ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          DMOV      QS1       ;  copy QS1 up to QS2
          DMOV      QS0       ;  copy QS0 up to QS1
          LT        QS2       ;  load T from QS2
          MPYK      H2        ;  QS2 * H2 to P
          LTA       QS1       ;  add P to A, load T from QS1
          MPYK      H1        ;  QS1 * H1 to P
          APAC                ;  add P to A
          ADD       TQ,0Bh    ;  shift input sample 2^11, add 
          SACH      QS0,5     ;  normalize

          ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          LT        QS2       ;  load T from QS2
          MPYK      G0        ;  IS2 * G0 to P
          LTA       QS1       ;  add P to A, load T from QS1
          MPYK      G1        ;  QS1 * G1 to P
          LTA       QS0       ;  add P to A, load T from QS0
          MPYK      G2        ;  QS0 * G2 to P
          APAC                ;  add P to A
          SACH      TQ,5      ;  normalize

PHDET     LT        TI        ;  load T from TI
          MPY       TQ        ;  TI * TQ -> P
          PAC                 ;  P -> A
          SACH      PD,6      ;  normalize

PLL       DMOV      S0        ;  copy S0 up to S1
          LT        S1        ;  load T from S1
          MPYK      H         ;  S1 * (H * 2^11) -> P
          PAC                 ;  P -> A
          SACH      S0,5      ;  normalize

          LAC       PD        ;  load PD
          ADD       S0        ;  add S0
          SACL      S0        ;  store sum in S0

          ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          LT        S0        ;  load T from S0
          MPYK      GA        ;  S0 * (G * 2^13) -> P
          LTA       S1        ;  add P to A, load T from S1
          MPYK      GB        ;  S1 * (G1 * 2^13) to P
          APAC                ;  add P to A
          SACH      S1,4      ;  use S1 as temp buffer here

          LAC       S1        ;  to normalize output (2^17 * 2^-16)
          SACL      TN        ;  and store in TN

LOCK      CALL      C_LOCK    ;  indicate lock condition

DATA      LAC      TI         ;  get LPF'd I channel value

          BLZ      DA_LO      ;  if less than 0, go DA_LO

DA_HI     LAC      DO         ;  else load DO
          ORK      4000h      ;  OR with data high mask
          SACL     DO         ;  store DO

          LALK     1          ;  load 1
          SACL     TI         ;  into TI

          B        FLAG_T     ;  on to TUNE

DA_LO     LAC      DO         ;  load DO
          ANDK     0BFFFh     ;  AND with data low mask
          SACL     DO         ;  store DO

          LALK     -1         ;  load -1
          SACL     TI         ;  into TI

FLAG_T    LAC       TI        ;  get TI
          LRLK      AR2,0398h ;  0398h is start of delay line
          LARP      AR2       ;  AR2 point to DL start
          SACL      *         ;  store TI 

          LRLK     AR1,03FEh  ;  03FEh is end of delay line 
          LARP     AR1        ;  AR1 pointer to 03FEh  

          MPYK     0          ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK     066h       ;  repeat (102 + 1) times to do correlation
          MACD     FLAG_C,*-  ;  MACD up thru coeffs, down thru data
          APAC                ;  accum last product
          SACL     TI         ;  store low

TFL_H     LAC       TI        ;  load TI
          SBLK      103       ;  TI - 103

          BGEZ      FLG       ;  TI => 103, goto FLG

TFL_L     LAC       TI        ;  load TI
          ADLK      103       ;  TI + 103

          BLEZ      FLG       ;  TI <= -110, goto FLG

NOFLG     LDPK      08h       ;  mem page 8
          LAC       CFG       ;  get CFG
          ANDK      0FF7Fh    ;  LED 208 OFF
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  mem page 0

          B         TUNE      ;  goto TUNE

FLG       LDPK      08h       ;  else mem page 8
          LAC       CFG       ;  load CFG
          ORK       0080h     ;  LED208 ON
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  mem page 0

TUNE      CALL     R_TUNE     ;  tune radio via PLL

TX_ONOFF  LAC       TXE       ;  get count in TXE

          BLZ       TX_OFF    ;  if < 0 goto TX_OFF

          IN        DI,0Ah    ;  else input TNC port
          LAC       DI        ;  load TNC input
          ANDK      0200h     ;  strip garbage around PTT bit

          BNZ       TX_OFF    ;  if <> 0 goto TX_OFF

TX_ON     LAC       RO        ;  get RO
          ORK       0008h     ;  turn PTT ON
          SACL      RO        ;  store RO

          LDPK      08h       ;  else mem page 8
          LAC       CFG       ;  load CFG
          ORK       0040h     ;  LED207 ON
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  mem page 0

          B         TX

TX_OFF    LAC       RO        ;  get RO
          ANDK      0FFF7h    ;  turn PTT OFF
          SACL      RO        ;  store RO

          LDPK      08h       ;  mem page 8
          LAC       CFG       ;  get CFG
          ANDK      0FFBFh    ;  LED 207 OFF
          SACL      CFG       ;  store CFG
          LDPK      00h       ;  mem page 0

TX        IN        DI,0Ah    ;  TNC into DI
          DMOV      TXD       ;  copy TXD up to LTXD
          LAC       DI        ;  load DI  
          SFR                 ;  shift off sign bit!!
          ANDK      4000h     ;  strip garbage around TX data bit
          SACL      TXD       ;  store TX bit

          BNZ       THI       ;  if <> 0 goto THI

TLO       LAC       TXE       ;  get count in TXE

          BLZ       EDGE       ;  if < 0 goto EDGE

          SUBK      01h       ;  else decrement count in TXE
          SACL      TXE       ;  and store

          B         EDGE      ;  goto EDGE

THI       LALK      TXEC      ;  get TXEC count
          SACL      TXE       ;  reset TXE

EDGE      LAC       TXD       ;  get TX bit
          XOR       LTXD      ;  TXD XOR LTXD

          BZ        TXRAMP    ;  0 if no data edge

          ZAC                 ;  else load zero
          SACL      TRB       ;  and reset ramp

TXRAMP    LAC       TRB       ;  get last ramp value
          ADLK      TINC      ;  add incr value to ramp
          SACL      TRB       ;  store back in TRB
          SBLK      TOP       ;  subtract TOP value

          BLEZ      GETTXS    ;  if ramp <= TOP goto GETTXS

          SACL      TRB       ;  else put wrapped value in TX ramp buffer

GETTXS    LAC       TRB,9     ;  get ramp offset * 2^7, shift 2^9
          SACH      TXS       ;  normalize
          LALK      TRG_TBL   ;  point to trig table
          ADD       TXS       ;  add normalized ramp offset
          TBLR      TXS       ;  read trig value * 2^10 into sine buff

          LAC       TXD       ;  get TX bit

          BZ        TX_OUT    ;  if 0 goto TX_OUT  

          LAC       TXS       ;  else get TXS
          NEG                 ;  -1 * TXS
          SACL      TXS       ;  store TXS

; use the following to output the TX signal (normal operation)

TX_OUT    LAC       TXS,2     ;  get TX signal, shift 2^2

; use the following to output eye instead of TX signal

;  TX_OUT    LDPK      06h       ;  xx
;            LAC       EYE,4     ;  get filtered PLL signal, shift 2^6
;            LDPK      00h       ;  xx

            ANDK      0FFFCh    ;  mask out AIO control bits
            SACL      DXR       ;  AIO out

END_DSP   LAC       DO        ;  load DO
          ANDK      0FBFFh    ;  reset duty cycle
          SACL      DO        ;  store DO
          OUT       DO,06h    ;  output to TNC port 06h 

          LDPK      08h       ;  data mem page 8
          OUT       CFG,07h   ;  output to TNC port 07h 
          LDPK      00h       ;  data mem page 0

          OUT       RO,0Bh    ;  output PTT to radio_1 port 0Bh

RESET     CALL      RST       ;  process RST

          RET                 ;  done!


RST       LDPK      06h       ;  data mem page 6
          LALK      0500h     ;  UART LINE STATUS buffer addr
          SACL      UBUF      ;  into UBUF
          OUT       UBUF,03h  ;  and OUT to UART

          IN        UBUF,05h  ;  read UART LINE STATUS
          LACK      01h       ;  UART RX status mask
          AND       UBUF      ;  test UART RX status

          BZ        RST_DN    ;  if no read goto RST_DN

          ZAC                 ;  UART TX/RX buffer addr
          SACL      UBUF      ;  into UBUF
          OUT       UBUF,03   ;  and OUT to UART

          IN        SYMBI,05h ;  read UART RX
          LAC       SYMBI     ;  get symbol
          ANDK      005Fh     ;  zero upper byte!!

          SACL      SYMBI     ;  store clean symbol

TST_RST   LAC       SYMBI     ;  get symbol
          SUBK      'R'       ;  compare with "R"

          BNZ       RST_DN    ;  branch if not "R"

          B         0414h     ;  else reset

RST_DN    LDPK      00h       ;  data mem page 0
          RET                 ;  RST done !!


R_TUNE    DMOV     TF0        ;  move TF0 to TF1

          ZAC                 ;  zero A
          MPYK     00h        ;  zero P

          LT       TF1        ;  load T from TF1
          MPYK     TFH        ;  (TF1 * TFH * 2^11) to P
          APAC                ;  add P to A
          SACH     TF0,5      ;  store A high in TF0 (2^-16 normalize)

          LAC      TN         ;  load A with TN
          ADD      TF0        ;  add TF0 to A
          SACL     TF0        ;  store TF0 for next time

          ZAC                 ;  zero A
          MPYK     00h        ;  zero P

          LT       TF0        ;  load T from TF0
          MPYK     TFGA       ;  (S0 * TFGA * 2^12) to P
          LTA      TF1        ;  add P to A, load T from TF1
          MPYK     TFGB       ;  (TF1 * TFGB * 2^12) to P
          APAC                ;  add P to A
          SACH     TF1,4      ;  use TF1 as temp output buffer

          LAC      DCNT       ;  get decimation conter
          SBLK     1          ;  decrement
          SACL     DCNT       ;  and store

          BGEZ     TN_DONE    ;  if => 0 goto TN_DONE

          LACK     CNT        ;  else get CNT
          SACL     DCNT       ;  and reset DCNT

DCM_FIL   DMOV     DF0        ;  move DF0 to DF1

          ZAC                 ;  zero A
          MPYK     00h        ;  zero P

          LT       DF1        ;  load T from DF1
          MPYK     TFH        ;  (DF1 * TFH * 2^11) to P
          APAC                ;  add P to A
          SACH     DF0,5      ;  store A high in DF0 (2^-16 normalize)

          LAC      TF1        ;  load A with TF1
          ADD      DF0        ;  add DF0 to A
          SACL     DF0        ;  store DF0 for next time

          ZAC                 ;  zero A
          MPYK     00h        ;  zero P

          LT       DF0        ;  load T from S0
          MPYK     TFGA       ;  (DF0 * TFGA * 2^12) to P
          LTA      DF1        ;  add P to A, load T from DF1
          MPYK     TFGB       ;  (DF1 * TFGB * 2^12) to P
          APAC                ;  add P to A
          SACH     DF1,4      ;  use DF1 as temp output buffer

PULSE     LAC      PCNT       ;  get PCNT

          BGEZ     P_TUNE     ;  if PCNT => 0 goto P_TUNE

          LAC      DF1        ;  else get decm filter output
          ADDK     RTL        ;  add receiver tuning low limit

          BGEZ     TN_DONE    ;  if => 0, goto TN_DONE

          LALK     CNTH       ;  else load CNTH
          SACL     PCNT       ;  and reset PCNT

          B        TN_DONE    ;  goto TN_DONE

P_TUNE    LAC      PCNT       ;  get PCNT
          SBLK     01h        ;  decrement
          SACL     PCNT       ;  store PCNT
          SBLK     CNTL       ;  subtract CNTL

          BLZ      NO_TUNE    ;  if < 0 goto NO_TUNE

DO_TUNE   LAC      RO         ;  else load RO
          ORK      0080h      ;  OR with LED205 ON mask
          SACL     RO         ;  store RO

          LDPK     06h        ;  data mem page 6
          LAC      LS         ;  get LS
          LDPK     00h        ;  data mem page 0

          BZ       TN_DONE    ;  don't tune if LS = 0

          LAC      RO         ;  else load RO
          ORK      0020h      ;  OR with tune-low mask
          SACL     RO         ;  store RO

          B        TN_DONE    ;  on to TN_DONE

NO_TUNE   LAC      PCNT       ;  get decm filter output

          BLZ      TN_DONE    ;  if < 0, goto TN_DONE

          LAC      RO         ;  else load RO
          ANDK     0FF5Fh     ;  AND with no tune-low/LED205 OFF mask 
          SACL     RO         ;  store RO

TN_DONE   RET                 ;  tuning done


C_LOCK    LAC       STS       ;  get STS
          LDPK      06h       ;  data mem page 6
          SACL      LSI       ;  store LSI

GET_TRG   LDPK      00h       ;  data mem page 0
          LAC       RB,9      ;  get ramp offset * 2^7, shift 2^9
          LDPK      06h       ;  data mem page 6
          SACH      LTS       ;  store high to normalize
          LALK      TRG_TBL   ;  point to trig table
          ADD       LTS       ;  add normalized ramp offset
          ADDK      010h      ;  offset 010h
          TBLR      LTS       ;  read trig value * 2^10 into sine buff
          ADDK      020h      ;  add cosine offset
          TBLR      LTC       ;  read trig value * 2^10 into cosine buff

LIMUL     LT        LSI       ;  load T from LSI
          MPY       LTS       ;  LSI * LTS -> P
          PAC                 ;  P -> A
          SACH      LTI,6     ;  normalize

LQMUL     LT        LSI       ;  load T from LSI
          MPY       LTC       ;  LSI * LTC -> P
          PAC                 ;  P -> A
          SACH      LTQ,6     ;  normalize

LI_LPF    ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          DMOV      LIS1      ;  copy LIS1 up to LIS2
          DMOV      LIS0      ;  copy LIS0 up to LIS1
          LT        LIS2      ;  load T from LIS2
          MPYK      H2        ;  LIS2 * H2 to P
          LTA       LIS1      ;  add P to A, load T from LIS1
          MPYK      H1        ;  LIS1 * H1 to P
          APAC                ;  add P to A
          ADD       LTI,0Bh   ;  shift input sample 2^11, add 
          SACH      LIS0,5    ;  normalize

          ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          LT        LIS2      ;  load T from LIS2
          MPYK      G0        ;  LIS2 * G0 to P
          LTA       LIS1      ;  add P to A, load T from LIS1
          MPYK      G1        ;  LIS1 * G1 to P
          LTA       LIS0      ;  add P to A, load T from LIS0
          MPYK      G2        ;  IS0 * G2 to P
          APAC                ;  add P to A
          SACH      LTI,5     ;  normalize

LQ_LPF    ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          DMOV      LQS1      ;  copy LQS1 up to LQS2
          DMOV      LQS0      ;  copy LQS0 up to LQS1
          LT        LQS2      ;  load T from LQS2
          MPYK      H2        ;  LQS2 * H2 to P
          LTA       LQS1      ;  add P to A, load T from LQS1
          MPYK      H1        ;  LQS1 * H1 to P
          APAC                ;  add P to A
          ADD       LTQ,0Bh   ;  shift input sample 2^11, add 
          SACH      LQS0,5    ;  normalize

          ZAC                 ;  zero A!!
          MPYK      00h       ;  zero P!!
          LT        LQS2      ;  load T from LQS2
          MPYK      G0        ;  LIS2 * G0 to P
          LTA       LQS1      ;  add P to A, load T from LQS1
          MPYK      G1        ;  LQS1 * G1 to P
          LTA       LQS0      ;  add P to A, load T from LQS0
          MPYK      G2        ;  LQS0 * G2 to P
          APAC                ;  add P to A
          SACH      LTQ,5     ;  normalize

LPHDET    LT        LTI       ;  load T from LTI
          MPY       LTQ       ;  LTI * LTQ -> P
          PAC                 ;  P -> A
          SACH      LPD,6     ;  normalize

LP_LCK    DMOV     LS0        ;  move LS0 to LS1
          ZAC                 ;  zero A
          MPYK     00h        ;  zero P

          LT       LS1        ;  load T from LS1
          MPYK     TFH        ;  (LS1 * TFH * 2^11) to P
          APAC                ;  add P to A
          SACH     LS0,5      ;  store A high in LS0 (2^-16 normalize)

          LAC      LPD        ;  load A with LPD
          ADD      LS0        ;  add LS0 to A
          SACL     LS0        ;  store LS0 for next time

          ZAC                 ;  zero A
          MPYK     00h        ;  zero P

          LT       LS0        ;  load T from LS0
          MPYK     TFGA       ;  (lS0 * TFGA * 2^12) to P
          LTA      LS1        ;  add P to A, load T from LS1
          MPYK     TFGB       ;  (LS1 * TFGB * 2^12) to P
          APAC                ;  add P to A
          SACH     LS1,4      ;  use LS1 as temp output buffer

LLEVL     LAC       LS1       ;  get LS1
          LDPK      00h       ;  back to data mem page 0
          ADLK      LLEV      ;  add LLEV

          BGEZ      LL_LO     ;  if < 0 goto LL_LO

LL_HI     LAC       RO        ;  load RO
          ORK       0040h     ;  OR to set lock LED
          SACL      RO        ;  store RO

          LAC       DO        ;  load DO
          ANDK      0FEFFh    ;  AND to set DCD ON (low)
          SACL      DO        ;  store DO

          LACK      01h       ;  load 1
          LDPK      06h       ;  data mem page 6
          SACL      LS        ;  store in LS
          LDPK      00h       ;  data mem page 0

          B         LOCK_DN   ;  goto LOCK_DN

LL_LO     LAC       RO        ;  load RO
          ANDK      0FFBFh    ;  AND to reset lock LED
          SACL      RO        ;  store RO

          LAC       DO        ;  load DO
          ORK       0100h     ;  OR set DCD OFF (high)
          SACL      DO        ;  store DO

          ZAC                 ;  load 0
          LDPK      06h       ;  data mem page 6
          SACL      LS        ;  store in LS
          LDPK      00h       ;  data mem page 0


LOCK_DN   RET                 ;  lock done


WAIT4     LARK      AR2,0FFh
          CALL      WAIT2
          CALL      WAIT2

          RET


WAIT2     LARK      AR2,0FFh
          CALL      WAIT1
          CALL      WAIT1

          RET


WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ; Loop through count

          LARP      AR2
          BANZ      WAIT_A    ; Loop through count

          RET


          .MSFIRST

FLAG_C    .WORD        -1     ;  flag signature
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD        -1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1
          .WORD         1

TRG_TBL   .WORD    00h        ;  start sine * 2^10
          .WORD    032h
          .WORD    064h
          .WORD    096h
          .WORD    0C8h
          .WORD    0F9h
          .WORD    0129h
          .WORD    0159h
          .WORD    0188h
          .WORD    01B6h
          .WORD    01E3h
          .WORD    020Eh
          .WORD    0239h
          .WORD    0262h
          .WORD    028Ah
          .WORD    02B0h
          .WORD    02D4h
          .WORD    02F7h
          .WORD    0318h
          .WORD    0336h
          .WORD    0353h
          .WORD    036Eh
          .WORD    0387h
          .WORD    039Eh
          .WORD    03B2h
          .WORD    03C4h
          .WORD    03D4h
          .WORD    03E1h
          .WORD    03ECh
          .WORD    03F5h
          .WORD    03FBh
          .WORD    03FFh
          .WORD    0400h      ;  start cosine * 2^10
          .WORD    03FFh
          .WORD    03FBh
          .WORD    03F5h
          .WORD    03ECh
          .WORD    03E1h
          .WORD    03D4h
          .WORD    03C4h
          .WORD    03B2h
          .WORD    039Eh
          .WORD    0387h
          .WORD    036Eh
          .WORD    0353h
          .WORD    0336h
          .WORD    0318h
          .WORD    02F7h
          .WORD    02D4h
          .WORD    02B0h
          .WORD    028Ah
          .WORD    0262h
          .WORD    0239h
          .WORD    020Eh
          .WORD    01E3h
          .WORD    01B6h
          .WORD    0188h
          .WORD    0159h
          .WORD    0129h
          .WORD    0F9h
          .WORD    0C8h
          .WORD    096h
          .WORD    064h
          .WORD    032h
          .WORD    00h
          .WORD    0FFCEh
          .WORD    0FF9Ch
          .WORD    0FF6Ah
          .WORD    0FF38h
          .WORD    0FF07h
          .WORD    0FED7h
          .WORD    0FEA7h
          .WORD    0FE78h
          .WORD    0FE4Ah
          .WORD    0FE1Dh
          .WORD    0FDF2h
          .WORD    0FDC7h
          .WORD    0FD9Eh
          .WORD    0FD76h
          .WORD    0FD50h
          .WORD    0FD2Ch
          .WORD    0FD09h
          .WORD    0FCE8h
          .WORD    0FCCAh
          .WORD    0FCADh
          .WORD    0FC92h
          .WORD    0FC79h
          .WORD    0FC62h
          .WORD    0FC4Eh
          .WORD    0FC3Ch
          .WORD    0FC2Ch
          .WORD    0FC1Fh
          .WORD    0FC14h
          .WORD    0FC0Bh
          .WORD    0FC05h
          .WORD    0FC01h
          .WORD    0FC00h
          .WORD    0FC01h
          .WORD    0FC05h
          .WORD    0FC0Bh
          .WORD    0FC14h
          .WORD    0FC1Fh
          .WORD    0FC2Ch
          .WORD    0FC3Ch
          .WORD    0FC4Eh
          .WORD    0FC62h
          .WORD    0FC79h
          .WORD    0FC92h
          .WORD    0FCADh
          .WORD    0FCCAh
          .WORD    0FCE8h
          .WORD    0FD09h
          .WORD    0FD2Ch
          .WORD    0FD50h
          .WORD    0FD76h
          .WORD    0FD9Eh
          .WORD    0FDC7h
          .WORD    0FDF2h
          .WORD    0FE1Dh
          .WORD    0FE4Ah
          .WORD    0FE78h
          .WORD    0FEA7h
          .WORD    0FED7h
          .WORD    0FF07h
          .WORD    0FF38h
          .WORD    0FF6Ah
          .WORD    0FF9Ch
          .WORD    0FFCEh
          .WORD    00h
          .WORD    032h
          .WORD    064h
          .WORD    096h
          .WORD    0C8h
          .WORD    0F9h
          .WORD    0129h
          .WORD    0159h
          .WORD    0188h
          .WORD    01B6h
          .WORD    01E3h
          .WORD    020Eh
          .WORD    0239h
          .WORD    0262h
          .WORD    028Ah
          .WORD    02B0h
          .WORD    02D4h
          .WORD    02F7h
          .WORD    0318h
          .WORD    0336h
          .WORD    0353h
          .WORD    036Eh
          .WORD    0387h
          .WORD    039Eh
          .WORD    03B2h
          .WORD    03C4h
          .WORD    03D4h
          .WORD    03E1h
          .WORD    03ECh
          .WORD    03F5h
          .WORD    03FBh
          .WORD    03FFh
          .WORD    0400h
          .WORD    03FFh
          .WORD    03FBh
          .WORD    03F5h
          .WORD    03ECh
          .WORD    03E1h
          .WORD    03D4h
          .WORD    03C4h
          .WORD    03B2h
          .WORD    039Eh
          .WORD    0387h
          .WORD    036Eh
          .WORD    0353h
          .WORD    0336h
          .WORD    0318h
          .WORD    02F7h
          .WORD    02D4h
          .WORD    02B0h
          .WORD    028Ah
          .WORD    0262h
          .WORD    0239h
          .WORD    020Eh
          .WORD    01E3h
          .WORD    01B6h
          .WORD    0188h
          .WORD    0159h
          .WORD    0129h
          .WORD    0F9h
          .WORD    0C8h
          .WORD    096h
          .WORD    064h
          .WORD    032h
          .WORD    00h

          .END
