;  File:  SPEEDTST.ASM Version 1.3 December 10, 1994 Copyright Ron Parsons W5RKN 1994
;
;  To assemble, use the command "asm speedtst.asm
;
;  Program to probe maximum conversion frequency.
;
;
;  The program waits for a character to be typed from the keyboard before 
;    proceeding to allow time to set up a terminal program.
;
;  It then loops through values of TA and TB sorted by increasing conversion 
;    frequency. These values are stored in BOTH_TBL (TA, TB, freq and cycles)
;
;  The "inner loop" is less than 60 cycles long which puts a processor 
;    limit of about 160,000 cycles per second which is well above the 
;    D/A conversion limit.
;
;  The program generates a tone at each step with the values of TA, TB, 
;    conversion frequency and cycles sent to the terminal before each tone.
;
;  The calculated number of processor cycles and the measured number are
;    printed for each combination.
;
;  At some steps, the tone will not be generated. At some point, 
;    the program will hang.
;
;  Some conversion frequencies in the table exceed the 16 bit limit of a word.
;    No attempt has been made to allow for this (my DSP-93 dies before it gets there).
;
;  Revision history
;  
;  1.3 Fixed PTT bug, changes due to Monitor usage
;  
;  1.2 Added separate TA and TB columns in data table, calculated table size.
;      Added timer interrupt to measure conversion frequency.
;
;  1.1 Added table for TA/TB and conversion frequencies.
;
;  1.0 Calculated TA and TB with program loops.
;
;
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Uses radio port #1
;  L201 is fixed active as power indicator

;  Portions copyright Frank H Perkins, 1993 and Lon Cecil, 1993
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
; 
; This document, and it executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
; 
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
; 
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
; 
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;

#include  "REGS.INC"
#include  "PORTS.INC"
#include  "MONITOR.INC"

;  global variables
TEMP2     .EQU      OUTBIT2_MEM ;  temporary storage


CFG       .EQU      20h       ;  AIO CFG buffer (data mem page 8)
XO        .EQU      21h       ;  Gain/LED control
LCTA      .EQU      22h       ;  loop counter
LCTB      .EQU      23h       ;  loop counter
SO        .EQU      24h       ;  sig buffer 
RB        .EQU      25h       ;  ramp buffer
TS        .EQU      26h       ;  sine buffer
CMDA      .EQU      27h       ;  AIO load RA/TA
CMDB      .EQU      28h       ;  AIO load RB/TB
LCTF1     .EQU      29h       ;  audio sound loop counter
LCTF      .EQU      2Ah       ;  audio multiplier
BUFI      .EQU      2Bh       ;  AIO input buffer
BUFO      .EQU      2Ch       ;  AIO output buffer
LCTBL     .EQU      2Dh       ;  loop counter for BOTH_TBL
TATB      .EQU      2Eh       ;  
; Memory Storage
RINT_TMR_H .EQU     33h       ;  Upper 16 bits of time interval
RINT_TMR_L .EQU     34h       ;  Lower 16 bits of time interval
TINTCTR   .EQU      35h       ;  TINT counter
TBSIZE    .EQU      36h       ;  number of pairs of values in BOTH_TBL
TIM_TEMP1 .EQU      3Bh       ;  A memory location for temporary use
TIM_TEMP2 .EQU      3Ch       ;  A memory location for temporary use
TIM_TEMP3 .EQU      3Dh       ;  A memory location for temporary use
NEW_TIME  .EQU      3Eh       ;  New time flag


; Constants
TMR_ON    .EQU      8         ; Timer Int on
TMR_OFF   .EQU      0         ; Timer Int off
NPRESND   .EQU      20000     ;  number of sound loops 

CMDA_L    .EQU      CMDA_VAL(24)    ;  "Large" TA
;  AIO configuration: HPF in, Sync, 3 V half scale, sin x/x
;CMDC      .EQU      AIO_CONFIG|AIO_INSADHPF|AIO_SYNC|AIO_GAIND60|AIO_GAIND70|AIO_INSSINX
;  AIO cofig:  A/D gain 2, sync, no HPF or sin corr
CMDC      .EQU      AIO_CONFIG|AIO_GAIND71|AIO_GAIND60|AIO_SYNC 

TOP       .EQU      04000h    ;  ramp top value

;  locations used by PRVAL
KSTART    .EQU      10h       ;  address of powers of 2 konstants (data mem page 8)
HI        .EQU      11h       ;  storage for HI part of ACC
LO        .EQU      12h       ;  storage for LO part of ACC
CRYDIG    .EQU      13h       ;  storage for carry digit/print digit
LCT3      .EQU      14h       ;  loop counter
LCT4      .EQU      15h       ;  loop counter

DANS      .EQU      2000h     ;  answer and powers of 2 array (absolute address)

NDIGITS   .EQU      10        ;  number of digits allowed in answer
NPOWOF2   .EQU      32        ;  number of powers of 2 (incl 2^0)
NACCBITS  .EQU      32        ;  number of bits in ACC to look at

; ---------------------------------------------------------------------
          .ORG      1000h

          B         TINT      ;  branch to Timer interrupt
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)

GO        DINT                ;  shut off interrupts
          LDPK      USER_PAGE ;  data mem page 8 (external)

LED_INI   LALK      0000h     ;  sel AUX in, gain = 1, LEDs OFF
          SACL      XO        ;  into XO
          OUT       XO,UART_CTRL    ;  out to ANA latch, to disable AIO chip

          LALK      0000h     ;  load DSP start
          SACL      XO        ;  into XO
          OUT       XO,RADIO_GAIN
          OUT       XO,TNC_OUTPUT
          OUT       XO,RADIO_CTRL
          CALL      WAIT4     ;  only POWER LED should be lighted

          LARP      AR0
          LRLK      AR0,2000h ;  Set up a known memory area pointer
          CALL      INBIT     ;  Force program to wait for keyboard input before proceeding

          LARP      AR0       ;  Write welcome string
          LRLK      AR0,2000h
          RPTK      S_END-STR1
          BLKP      STR1,*+
          LARP      AR0
          LRLK      AR0,2000h
          CALL      SD_STR
          CALL      SD_CRLF

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       0         ;  set P shift = 0

          LALK      BOTH_TBL_END
          SBLK      BOTH_TBL
          SFR
          SFR                 ;  divide by 4
          SACL      TBSIZE

;       Interrupt masking
;
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          LACK      00h       ;   0000 0000 0000 0000
          SACL      IMR,0     ;   XINT________|| ||||
;  ???    SACL      IMRST,0   ;   RINT_________| ||||
                              ;   TINT___________||||
                              ;   INT2____________|||
                              ;   INT1_____________||
                              ;   INT0______________|
          LALK      0FFFFh    ;  Value for Timer Period (Max)
          SACL      PRD       ;  Set up period
          LDPK      USER_PAGE ;  data mem page 8 (external)

LOADTBL   LARP      AR0       ;  used by PRVAL
          LRLK      AR0,DANS  ;  Set up a known memory area pointer
          LALK      ANS       ;     
          RPTK      249       ;  transfer first 250 values from ANS
          TBLR      *+        ;  move it to DANS
          LRLK      AR0,DANS+250
          LALK      ANS+250   ;     
          RPTK      79        ;  transfer remaining 80 values from ANS+250
          TBLR      *+        ;  move it to DANS+250

          ZAC                 ;  initialize loop counter
          SACL      LCTBL

TATBLOOP  
          LAC       XO
          ANDK      LED_204_OFF   ;  mask LED 204 OFF
          SACL      XO        ;  into XO
          OUT       XO,RADIO_CTRL    ;  out to LED 204

          LALK      BOTH_TBL  ;  point to TA/TB table
          ADD       LCTBL,2   ;  add offset (four words per entry)
          TBLR      TATB      ;  read TA values
          LAC       TATB
          SACL      LCTA

          LALK      BOTH_TBL  ;  point to TA/TB table
          ADD       LCTBL,2   ;  add offset (four words per entry)
          ADDK      1     
          TBLR      TATB      ;  read TB values
          LAC       TATB
          SACL      LCTB

          LAC       LCTA
          CALL      PRDEC     ;  print TA
          CALL      SP4
          LAC       LCTB
          CALL      PRDEC     ;  print TB
          CALL      SP4
          
          LALK      BOTH_TBL  ;  point to TA/TB table
          ADD       LCTBL,2   ;  add offset (four words per entry)
          ADDK      2     
          TBLR      TATB      ;  read conversion freq values
          ZALS      TATB      ;  load as unsigned 16 bit value
          CALL      PRDEC     ;  print conversion freq
          CALL      SP4

          LALK      BOTH_TBL  ;  point to TA/TB table
          ADD       LCTBL,2   ;  add offset (four words per entry)
          ADDK      3     
          TBLR      TATB      ;  read computed cycle values
          ZALS      TATB      ;  load as unsigned 16 bit value
          CALL      PRDEC     ;  print computed cycle
          CALL      SP4

          LAC       LCTA,9    ;  compute CMDA = (LCTA<<9)+(LCTA<<2)
          SACL      CMDA
          LAC       LCTA,2
          ADD       CMDA
          SACL      CMDA

          LAC       LCTB,9    ;  compute CMDB = (LCTB<<9)+(LCTB<<2)+2
          SACL      CMDB
          LAC       LCTB,2
          ADD       CMDB
          ADDK      2
          SACL      CMDB


AIO_RST   LALK      AIO_RESET|GAIN4|REC_IN_1     ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 100 ms
 
AIO_INI   LALK      LED_202|GAIN4|REC_IN_1       ;  LED 202 on, radio 1, gain = 16
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 100 ms

          LALK      AIO_ENABLE|GAIN4|REC_IN_1     ;  load AIO run, radio 1, gain = 16
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 100 ms


          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      IMR       ;  store INT mask
          LDPK      USER_PAGE ;  data mem page 8 (external)

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt
; load "large TA" first
          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      CMDA_L    ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt
; then load TB
          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LAC       CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          LAC       CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          EINT
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts

          ZAC
          SACL      NEW_TIME  ;  zero flag
          LALK      0018h     ;  mask SIO RX and Timer interrupt active
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      IMR       ;  store mask in IMR
          LDPK      USER_PAGE ;  data mem page 8 (external)
          EINT                ;  enable interrupts 
RXLOOP    LAC       NEW_TIME  ;  wait for RX interrupt
          SUBK      1
          BLEZ      RXLOOP

          DINT                ;  disable interrupts

          RSXM                ;  reset sign extension mode
          ZALH      RINT_TMR_H  ;  restore hi ACC
          ADD       RINT_TMR_L  ;  restore lo ACC
          SSXM                ;  set sign extension mode
          CALL      PRDEC     ;  print measured conversion freq
          CALL      SD_CRLF


LED204    LALK      LED_204   ;  load LED 204 BRIGHT as active light
          SACL      XO        ;  into XO
          OUT       XO,RADIO_CTRL    ;  out to LED 204
; ---------------------------------------------------------------------

          SACL      RB        ;  zero ramp buffer
          SACL      TS        ;  zero sine buffer


          LALK      NPRESND   ;  number of loops
          SACL      LCTF1     ;  for sounds (so you hear something besides clicks)

SOUND
          LT        LCTA      ;  get TA
          MPYK      2         ;  multiply by 2 for approx 600 Hz tone
          PAC                 ;  get product into ACC
          SACL      LCTF
          LT        LCTF      ;  get product
          MPY       LCTB      ;  multiply by TB
          PAC                 ;  get product into ACC

          ADD       RB        ;  add to last ramp value

          SACL      RB        ;  store back in RB
          SBLK      TOP       ;  subtract TOP value

          BLEZ      GETS      ;  ramp not > top, go GETSC

          SACL      RB        ;  else put wrapped value in ramp buffer

GETS      ZAC                 ;  zero A
          ADD       RB,9      ;  get ramp offset * 2^7, shift 2^9
          SACH      TS        ;  store high for 2^-16 normalize
          LALK      TRIG_TBL  ;  point to trig table
          ADD       TS        ;  add normalized ramp offset
          TBLR      TS        ;  read trig value * 2^10 into sine buff

TX_OUT
;  Choose among the following lines to adjust the amplitude
;     of the  audio tones <---------
          LAC       TS,2      ;  get sine value, shift 2^2
;          LAC       TS,1      ;  get sine value, shift 2^1
;          LAC       TS        ;  get sine value, shift 2^0
;          SFR                 ;  get sine value, shift 2^-1
          ANDK      0FFFCh    ;  mask out AIO control bits
          SACL      BUFO      ;  AIO out

          LALK      0020h     ;  mask SIO TX interrupt active
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      IMR       ;  store mask in IMR
          LDPK      USER_PAGE ;  data mem page 8 (external)
          EINT                ;  enable interrupts 
          IDLE                ;  wait for TX interrupt

          LAC       LCTF1
          SUBK      1
          SACL      LCTF1     ;  sound loop counter
          BGZ       SOUND


          LAC       LCTBL
          ADDK      1
          SACL      LCTBL     ;  loop counter
          SUB       TBSIZE    ;  maximum value
          BLZ       TATBLOOP

          B         RESET     ;  stop

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
;  Prints the contents of ACC in Decimal.
;  This is a very crude function, but it works. Adds in BCD.
;  It uses the method of addition you learned in first grade.
PRDEC
          SACH      HI        ;  save ACC in HI and LO
          SACL      LO
          BGEZ      POS
          NEG                 ;  ACC was negative, make it pos
          SACH      HI        ;  save ACC in HI and LO
          SACL      LO
          LALK      2Dh       ;  minus sign (ACSII)
          SACL      TEMP2
          CALL      OUTBIT2   ;  print minus sign
POS       LALK      NACCBITS  ;  number of bits in ACC to work with
          SACL      LCT3      ;  loop counter

          ZAC
          LARP      AR0       ;  Zero answer digits
          LRLK      AR0,DANS 
          RPTK      NDIGITS-1
          SACL      *+

LOOP32
          RSXM                ;  reset sign extension mode
          ZALH      HI        ;  restore hi ACC
          ADD       LO        ;  restore lo ACC
          SFR                 ;  shift lowest bit into C
          SACH      HI        ;  save shifted ACC in HI and LO
          SACL      LO
          SSXM                ;  set sign extension mode
          BNC       ELOOP32   ;  nothing to do

          LRLK      AR4,DANS  ;  DANS is answer values start
          LT        LCT3      ;  calculate offset to power of 2 konstants
          MPYK      NDIGITS
          PAC
          ADLK      DANS
          SACL      KSTART
          LAR       AR3,KSTART ;  KSTART is power of 2 konstants start

;  add power of 2 konstant to answer
          LALK      NDIGITS   ;  number of digits in answer
          SACL      LCT4      ;  loop counter

          LARP      AR4
;LOOPDIG  LAC       *,0,AR3   ;  Assembler 2.8.3 won't do this  <--
;         ADD       *+,0,AR4  ;  Assembler 2.8.3 won't do this  <--
LOOPDIG   LAC       *         ;  Get digit of answer
          LARP      AR3
          ADD       *+        ;  add in digit of constant
          LARP      AR4
          SACL      *+    
          LAC       LCT4
          SUBK      1
          SACL      LCT4
          BGZ       LOOPDIG

;  adjust answer digits for base 10 carry
          LALK      NDIGITS   ;  number of digits in answer
          SACL      LCT4      ;  loop counter

          ZAC    
          SACL      CRYDIG    ;  zero carry digit
          LRLK      AR4,DANS+NDIGITS-1  ;  DANS is answer values start
          LARP      AR4
LOOPCRY   LAC       *         ;  Get digit of answer
          ADD       CRYDIG    ;  add in carry digit
          SACL      *         ;  save adjusted digit
          SUBK      10        ;  see if answer digit is greater than 9
          BLZ       NOCRY     ;  no carry digit needed

          SACL      *-        ;  save adjusted digit
          LALK      1
          SACL      CRYDIG    ;  set carry digit to 1
          B         ELOOPCRY

NOCRY     SBRK      1         ;  decrement aux register 
          ZAC    
          SACL      CRYDIG    ;  zero carry digit

ELOOPCRY  LAC       LCT4
          SUBK      1
          SACL      LCT4
          BGZ       LOOPCRY

ELOOP32   LAC       LCT3
          SUBK      1
          SACL      LCT3
          BGZ       LOOP32

;  print the answer supressing leading zeros
          LALK      NDIGITS   ;  number of digits in the answer
          SACL      LCT3      ;  loop counter

          LRLK      AR4,DANS  ;  DANS is answer values start
          LARP      AR4
LOOPANZ   LAC       LCT3
          SUBK      1
          SACL      LCT3
          BZ        LOOPAN1   ;  catch last zero if ACC is zero

          LAC       *+        ;  Get digit of answer
          BZ        LOOPANZ   ;  skip leading zeros
          B         LOOPPRT

LOOPANS
          LAC       LCT3
          SUBK      1
          SACL      LCT3
          BLZ       DONE

LOOPAN1   LAC       *+        ;  Get digit of answer
LOOPPRT   CALL      HEXOUT
          B         LOOPANS

DONE      RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
TINT
          LAC        TINTCTR     ; Get timer counter
          ADDK       1
          SACL       TINTCTR     ; Store the new value
          EINT
          RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
; The code in the RINT routine assumes the PRD is 0FFFh
; 
 
RINT
          RSXM                ;  reset sign extension mode
          LAC       NEW_TIME
          BNZ       END_TIME
START_TIME
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          NOP                 ;  to equalize timing from beginning of RINT
          NOP                 ;  to equalize timing from beginning of RINT
          NOP                 ;  to equalize timing from beginning of RINT
          NOP                 ;  to equalize timing from beginning of RINT
          NOP                 ;  to equalize timing from beginning of RINT
          LAC       TIM       ;  Get the current time count
 
          LDPK      USER_PAGE ;  data mem page 8 (external)
          SACL      RINT_TMR_L  ; Store the remaining counts in register 
          SACL      TIM_TEMP1   ; Store the remaining counts in register 
          LACK      1
          SACL      NEW_TIME  ;  set flag to end time
          ZAC
          SACL      RINT_TMR_H  ;  Clear high count
          SACL      TINTCTR   ;  Clear the timer counter

          EINT
          B         RINT_RET     
 
END_TIME
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          LAC       IMR
          ANDK      ~TMR_ON
          SACL      IMR       ;  Stop Timer interrupts
          LAC       TIM       ;  Get the current time count

          LDPK      USER_PAGE ;  data mem page 8 (external)
          SACL      TIM_TEMP2 ;  Save the current timer
          LAC       TINTCTR   ;  Has a TINT ocurred?
          BZ        NO_INTR   ;  No TINT ocurred

          SUBK      1
          SACL      RINT_TMR_H  ;  save high count (adjusted)
          LALK      0FFFFh    ;  Value for Timer Period (Max)
          SUB       TIM_TEMP2 ;  Get used Timer count this period
          ADD       RINT_TMR_L  ;  Add to initial Timer count
          SACL      RINT_TMR_L  ;  save low count
          SACH      TIM_TEMP3   ;  store 16 bit overflow

          LAC       RINT_TMR_H  ;  get high count
          ADD       TIM_TEMP3
          SACL      RINT_TMR_H  ;  save high count with carry
          B         RINT_EX

NO_INTR
          LAC       RINT_TMR_L ;  Get initial ramaining count
          SUB       TIM_TEMP2  ;  Subtract remaining count
          SACL      RINT_TMR_L ;  save low count

RINT_EX
          LACK      2
          SACL      NEW_TIME  ;  set flag to print time
          DINT
RINT_RET  SSXM                ;  set sign extension mode
          RET


; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
XINT
          LAC       BUFO      ;  get data from AIO output buffer
          LDPK      BASE_PAGE ;  data mem page 0 (internal)
          SACL      DXR       ;  store data in DXR to transmit to AIO
          LDPK      USER_PAGE ;  data mem page 8 (external)
          RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
WAIT4     CALL      WAIT2
          CALL      WAIT2
          RET

WAIT2     CALL      WAIT1
          CALL      WAIT1
          RET

WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ;  Loop through count
          LARP      AR2
          BANZ      WAIT_A    ;  Loop through count
          RET

; ---------------------------------------------------------------------
; ---------------------------------------------------------------------
STR1      .TEXT     " T A         T B       F r e q ( H z )   C y c l e s     C y c l e s ( M e a s )"
          .WORD     00
S_END

          .MSFIRST


;  Table of powers of 2 in decimal
ANS       .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 0
K31       .WORD        2, 1, 4, 7, 4, 8, 3, 6, 4, 8
K30       .WORD        1, 0, 7, 3, 7, 4, 1, 8, 2, 4 
K29       .WORD        0, 5, 3, 6, 8, 7, 0, 9, 1, 2
K28       .WORD        0, 2, 6, 8, 4, 3, 5, 4, 5, 6
K27       .WORD        0, 1, 3, 4, 2, 1, 7, 7, 2, 8
K26       .WORD        0, 0, 6, 7, 1, 0, 8, 8, 6, 4
K25       .WORD        0, 0, 3, 3, 5, 5, 4, 4, 3, 2
K24       .WORD        0, 0, 1, 6, 7, 7, 7, 2, 1, 6
K23       .WORD        0, 0, 0, 8, 3, 8, 8, 6, 0, 8
K22       .WORD        0, 0, 0, 4, 1, 9, 4, 3, 0, 4
K21       .WORD        0, 0, 0, 2, 0, 9, 7, 1, 5, 2
K20       .WORD        0, 0, 0, 1, 0, 4, 8, 5, 7, 6
K19       .WORD        0, 0, 0, 0, 5, 2, 4, 2, 8, 8
K18       .WORD        0, 0, 0, 0, 2, 6, 2, 1, 4, 4
K17       .WORD        0, 0, 0, 0, 1, 3, 1, 0, 7, 2
K16       .WORD        0, 0, 0, 0, 0, 6, 5, 5, 3, 6
K15       .WORD        0, 0, 0, 0, 0, 3, 2, 7, 6, 8
K14       .WORD        0, 0, 0, 0, 0, 1, 6, 3, 8, 4
K13       .WORD        0, 0, 0, 0, 0, 0, 8, 1, 9, 2
K12       .WORD        0, 0, 0, 0, 0, 0, 4, 0, 9, 6
K11       .WORD        0, 0, 0, 0, 0, 0, 2, 0, 4, 8
K10       .WORD        0, 0, 0, 0, 0, 0, 1, 0, 2, 4
K9        .WORD        0, 0, 0, 0, 0, 0, 0, 5, 1, 2
K8        .WORD        0, 0, 0, 0, 0, 0, 0, 2, 5, 6
K7        .WORD        0, 0, 0, 0, 0, 0, 0, 1, 2, 8
K6        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 6, 4
K5        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 3, 2
K4        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 1, 6
K3        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 8
K2        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 4
K1        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 2
K0        .WORD        0, 0, 0, 0, 0, 0, 0, 0, 0, 1
END_2     .WORD        0


TRIG_TBL  .WORD    00h        ;  sine table, scaled 2^10, 128 + 1 entries
          .WORD    032h
          .WORD    064h
          .WORD    096h
          .WORD    0C8h
          .WORD    0F9h
          .WORD    0129h
          .WORD    0159h
          .WORD    0188h
          .WORD    01B6h
          .WORD    01E3h
          .WORD    020Eh
          .WORD    0239h
          .WORD    0262h
          .WORD    028Ah
          .WORD    02B0h
          .WORD    02D4h
          .WORD    02F7h
          .WORD    0318h
          .WORD    0336h
          .WORD    0353h
          .WORD    036Eh
          .WORD    0387h
          .WORD    039Eh
          .WORD    03B2h
          .WORD    03C4h
          .WORD    03D4h
          .WORD    03E1h
          .WORD    03ECh
          .WORD    03F5h
          .WORD    03FBh
          .WORD    03FFh
          .WORD    0400h
          .WORD    03FFh
          .WORD    03FBh
          .WORD    03F5h
          .WORD    03ECh
          .WORD    03E1h
          .WORD    03D4h
          .WORD    03C4h
          .WORD    03B2h
          .WORD    039Eh
          .WORD    0387h
          .WORD    036Eh
          .WORD    0353h
          .WORD    0336h
          .WORD    0318h
          .WORD    02F7h
          .WORD    02D4h
          .WORD    02B0h
          .WORD    028Ah
          .WORD    0262h
          .WORD    0239h
          .WORD    020Eh
          .WORD    01E3h
          .WORD    01B6h
          .WORD    0188h
          .WORD    0159h
          .WORD    0129h
          .WORD    0F9h
          .WORD    0C8h
          .WORD    096h
          .WORD    064h
          .WORD    032h
          .WORD    00h
          .WORD    0FFCEh
          .WORD    0FF9Ch
          .WORD    0FF6Ah
          .WORD    0FF38h
          .WORD    0FF07h
          .WORD    0FED7h
          .WORD    0FEA7h
          .WORD    0FE78h
          .WORD    0FE4Ah
          .WORD    0FE1Dh
          .WORD    0FDF2h
          .WORD    0FDC7h
          .WORD    0FD9Eh
          .WORD    0FD76h
          .WORD    0FD50h
          .WORD    0FD2Ch
          .WORD    0FD09h
          .WORD    0FCE8h
          .WORD    0FCCAh
          .WORD    0FCADh
          .WORD    0FC92h
          .WORD    0FC79h
          .WORD    0FC62h
          .WORD    0FC4Eh
          .WORD    0FC3Ch
          .WORD    0FC2Ch
          .WORD    0FC1Fh
          .WORD    0FC14h
          .WORD    0FC0Bh
          .WORD    0FC05h
          .WORD    0FC01h
          .WORD    0FC00h
          .WORD    0FC01h
          .WORD    0FC05h
          .WORD    0FC0Bh
          .WORD    0FC14h
          .WORD    0FC1Fh
          .WORD    0FC2Ch
          .WORD    0FC3Ch
          .WORD    0FC4Eh
          .WORD    0FC62h
          .WORD    0FC79h
          .WORD    0FC92h
          .WORD    0FCADh
          .WORD    0FCCAh
          .WORD    0FCE8h
          .WORD    0FD09h
          .WORD    0FD2Ch
          .WORD    0FD50h
          .WORD    0FD76h
          .WORD    0FD9Eh
          .WORD    0FDC7h
          .WORD    0FDF2h
          .WORD    0FE1Dh
          .WORD    0FE4Ah
          .WORD    0FE78h
          .WORD    0FEA7h
          .WORD    0FED7h
          .WORD    0FF07h
          .WORD    0FF38h
          .WORD    0FF6Ah
          .WORD    0FF9Ch
          .WORD    0FFCEh
          .WORD    00h



;  TA, TB, 10000000/(2*TA*TB) (freq in Hz), 10000000/freq (cycles),  
;          sorted by increasing 10000000/(2*TA*TB)
;  Frequencies larger that 65535 are divided by 10
BOTH_TBL

	.WORD	24	,	24	,	8681	,	1152
	.WORD	24	,	23	,	9058	,	1104
	.WORD	23	,	24	,	9058	,	1104
	.WORD	23	,	23	,	9452	,	1058
	.WORD	24	,	22	,	9470	,	1056
	.WORD	22	,	24	,	9470	,	1056
	.WORD	23	,	22	,	9881	,	1012
	.WORD	22	,	23	,	9881	,	1012
	.WORD	24	,	21	,	9921	,	1008
	.WORD	21	,	24	,	9921	,	1008
	.WORD	22	,	22	,	10331	,	968
	.WORD	23	,	21	,	10352	,	966
	.WORD	21	,	23	,	10352	,	966
	.WORD	24	,	20	,	10417	,	960
	.WORD	20	,	24	,	10417	,	960
	.WORD	22	,	21	,	10823	,	924
	.WORD	21	,	22	,	10823	,	924
	.WORD	23	,	20	,	10870	,	920
	.WORD	20	,	23	,	10870	,	920
	.WORD	24	,	19	,	10965	,	912
	.WORD	19	,	24	,	10965	,	912
	.WORD	21	,	21	,	11338	,	882
	.WORD	22	,	20	,	11364	,	880
	.WORD	20	,	22	,	11364	,	880
	.WORD	23	,	19	,	11442	,	874
	.WORD	19	,	23	,	11442	,	874
	.WORD	24	,	18	,	11574	,	864
	.WORD	18	,	24	,	11574	,	864
	.WORD	21	,	20	,	11905	,	840
	.WORD	20	,	21	,	11905	,	840
	.WORD	22	,	19	,	11962	,	836
	.WORD	19	,	22	,	11962	,	836
	.WORD	23	,	18	,	12077	,	828
	.WORD	18	,	23	,	12077	,	828
	.WORD	24	,	17	,	12255	,	816
	.WORD	17	,	24	,	12255	,	816
	.WORD	20	,	20	,	12500	,	800
	.WORD	21	,	19	,	12531	,	798
	.WORD	19	,	21	,	12531	,	798
	.WORD	22	,	18	,	12626	,	792
	.WORD	18	,	22	,	12626	,	792
	.WORD	23	,	17	,	12788	,	782
	.WORD	17	,	23	,	12788	,	782
	.WORD	24	,	16	,	13021	,	768
	.WORD	16	,	24	,	13021	,	768
	.WORD	20	,	19	,	13158	,	760
	.WORD	19	,	20	,	13158	,	760
	.WORD	21	,	18	,	13228	,	756
	.WORD	18	,	21	,	13228	,	756
	.WORD	22	,	17	,	13369	,	748
	.WORD	17	,	22	,	13369	,	748
	.WORD	23	,	16	,	13587	,	736
	.WORD	16	,	23	,	13587	,	736
	.WORD	19	,	19	,	13850	,	722
	.WORD	24	,	15	,	13889	,	720
	.WORD	20	,	18	,	13889	,	720
	.WORD	18	,	20	,	13889	,	720
	.WORD	15	,	24	,	13889	,	720
	.WORD	21	,	17	,	14006	,	714
	.WORD	17	,	21	,	14006	,	714
	.WORD	22	,	16	,	14205	,	704
	.WORD	16	,	22	,	14205	,	704
	.WORD	23	,	15	,	14493	,	690
	.WORD	15	,	23	,	14493	,	690
	.WORD	19	,	18	,	14620	,	684
	.WORD	18	,	19	,	14620	,	684
	.WORD	20	,	17	,	14706	,	680
	.WORD	17	,	20	,	14706	,	680
	.WORD	24	,	14	,	14881	,	672
	.WORD	21	,	16	,	14881	,	672
	.WORD	16	,	21	,	14881	,	672
	.WORD	14	,	24	,	14881	,	672
	.WORD	22	,	15	,	15152	,	660
	.WORD	15	,	22	,	15152	,	660
	.WORD	18	,	18	,	15432	,	648
	.WORD	19	,	17	,	15480	,	646
	.WORD	17	,	19	,	15480	,	646
	.WORD	23	,	14	,	15528	,	644
	.WORD	14	,	23	,	15528	,	644
	.WORD	20	,	16	,	15625	,	640
	.WORD	16	,	20	,	15625	,	640
	.WORD	21	,	15	,	15873	,	630
	.WORD	15	,	21	,	15873	,	630
	.WORD	24	,	13	,	16026	,	624
	.WORD	13	,	24	,	16026	,	624
	.WORD	22	,	14	,	16234	,	616
	.WORD	14	,	22	,	16234	,	616
	.WORD	18	,	17	,	16340	,	612
	.WORD	17	,	18	,	16340	,	612
	.WORD	19	,	16	,	16447	,	608
	.WORD	16	,	19	,	16447	,	608
	.WORD	20	,	15	,	16667	,	600
	.WORD	15	,	20	,	16667	,	600
	.WORD	23	,	13	,	16722	,	598
	.WORD	13	,	23	,	16722	,	598
	.WORD	21	,	14	,	17007	,	588
	.WORD	14	,	21	,	17007	,	588
	.WORD	17	,	17	,	17301	,	578
	.WORD	24	,	12	,	17361	,	576
	.WORD	18	,	16	,	17361	,	576
	.WORD	16	,	18	,	17361	,	576
	.WORD	12	,	24	,	17361	,	576
	.WORD	22	,	13	,	17483	,	572
	.WORD	13	,	22	,	17483	,	572
	.WORD	19	,	15	,	17544	,	570
	.WORD	15	,	19	,	17544	,	570
	.WORD	20	,	14	,	17857	,	560
	.WORD	14	,	20	,	17857	,	560
	.WORD	23	,	12	,	18116	,	552
	.WORD	12	,	23	,	18116	,	552
	.WORD	21	,	13	,	18315	,	546
	.WORD	13	,	21	,	18315	,	546
	.WORD	17	,	16	,	18382	,	544
	.WORD	16	,	17	,	18382	,	544
	.WORD	18	,	15	,	18519	,	540
	.WORD	15	,	18	,	18519	,	540
	.WORD	19	,	14	,	18797	,	532
	.WORD	14	,	19	,	18797	,	532
	.WORD	24	,	11	,	18939	,	528
	.WORD	22	,	12	,	18939	,	528
	.WORD	12	,	22	,	18939	,	528
	.WORD	11	,	24	,	18939	,	528
	.WORD	20	,	13	,	19231	,	520
	.WORD	13	,	20	,	19231	,	520
	.WORD	16	,	16	,	19531	,	512
	.WORD	17	,	15	,	19608	,	510
	.WORD	15	,	17	,	19608	,	510
	.WORD	23	,	11	,	19763	,	506
	.WORD	11	,	23	,	19763	,	506
	.WORD	21	,	12	,	19841	,	504
	.WORD	18	,	14	,	19841	,	504
	.WORD	14	,	18	,	19841	,	504
	.WORD	12	,	21	,	19841	,	504
	.WORD	19	,	13	,	20243	,	494
	.WORD	13	,	19	,	20243	,	494
	.WORD	22	,	11	,	20661	,	484
	.WORD	11	,	22	,	20661	,	484
	.WORD	24	,	10	,	20833	,	480
	.WORD	20	,	12	,	20833	,	480
	.WORD	16	,	15	,	20833	,	480
	.WORD	15	,	16	,	20833	,	480
	.WORD	12	,	20	,	20833	,	480
	.WORD	10	,	24	,	20833	,	480
	.WORD	17	,	14	,	21008	,	476
	.WORD	14	,	17	,	21008	,	476
	.WORD	18	,	13	,	21368	,	468
	.WORD	13	,	18	,	21368	,	468
	.WORD	21	,	11	,	21645	,	462
	.WORD	11	,	21	,	21645	,	462
	.WORD	23	,	10	,	21739	,	460
	.WORD	10	,	23	,	21739	,	460
	.WORD	19	,	12	,	21930	,	456
	.WORD	12	,	19	,	21930	,	456
	.WORD	15	,	15	,	22222	,	450
	.WORD	16	,	14	,	22321	,	448
	.WORD	14	,	16	,	22321	,	448
	.WORD	17	,	13	,	22624	,	442
	.WORD	13	,	17	,	22624	,	442
	.WORD	22	,	10	,	22727	,	440
	.WORD	20	,	11	,	22727	,	440
	.WORD	11	,	20	,	22727	,	440
	.WORD	10	,	22	,	22727	,	440
	.WORD	24	,	9	,	23148	,	432
	.WORD	18	,	12	,	23148	,	432
	.WORD	12	,	18	,	23148	,	432
	.WORD	9	,	24	,	23148	,	432
	.WORD	21	,	10	,	23810	,	420
	.WORD	15	,	14	,	23810	,	420
	.WORD	14	,	15	,	23810	,	420
	.WORD	10	,	21	,	23810	,	420
	.WORD	19	,	11	,	23923	,	418
	.WORD	11	,	19	,	23923	,	418
	.WORD	16	,	13	,	24038	,	416
	.WORD	13	,	16	,	24038	,	416
	.WORD	23	,	9	,	24155	,	414
	.WORD	9	,	23	,	24155	,	414
	.WORD	17	,	12	,	24510	,	408
	.WORD	12	,	17	,	24510	,	408
	.WORD	20	,	10	,	25000	,	400
	.WORD	10	,	20	,	25000	,	400
	.WORD	22	,	9	,	25253	,	396
	.WORD	18	,	11	,	25253	,	396
	.WORD	11	,	18	,	25253	,	396
	.WORD	9	,	22	,	25253	,	396
	.WORD	14	,	14	,	25510	,	392
	.WORD	15	,	13	,	25641	,	390
	.WORD	13	,	15	,	25641	,	390
	.WORD	24	,	8	,	26042	,	384
	.WORD	16	,	12	,	26042	,	384
	.WORD	12	,	16	,	26042	,	384
	.WORD	8	,	24	,	26042	,	384
	.WORD	19	,	10	,	26316	,	380
	.WORD	10	,	19	,	26316	,	380
	.WORD	21	,	9	,	26455	,	378
	.WORD	9	,	21	,	26455	,	378
	.WORD	17	,	11	,	26738	,	374
	.WORD	11	,	17	,	26738	,	374
	.WORD	23	,	8	,	27174	,	368
	.WORD	8	,	23	,	27174	,	368
	.WORD	14	,	13	,	27473	,	364
	.WORD	13	,	14	,	27473	,	364
	.WORD	20	,	9	,	27778	,	360
	.WORD	18	,	10	,	27778	,	360
	.WORD	15	,	12	,	27778	,	360
	.WORD	12	,	15	,	27778	,	360
	.WORD	10	,	18	,	27778	,	360
	.WORD	9	,	20	,	27778	,	360
	.WORD	22	,	8	,	28409	,	352
	.WORD	16	,	11	,	28409	,	352
	.WORD	11	,	16	,	28409	,	352
	.WORD	8	,	22	,	28409	,	352
	.WORD	19	,	9	,	29240	,	342
	.WORD	9	,	19	,	29240	,	342
	.WORD	17	,	10	,	29412	,	340
	.WORD	10	,	17	,	29412	,	340
	.WORD	13	,	13	,	29586	,	338
	.WORD	24	,	7	,	29762	,	336
	.WORD	21	,	8	,	29762	,	336
	.WORD	14	,	12	,	29762	,	336
	.WORD	12	,	14	,	29762	,	336
	.WORD	8	,	21	,	29762	,	336
	.WORD	7	,	24	,	29762	,	336
	.WORD	15	,	11	,	30303	,	330
	.WORD	11	,	15	,	30303	,	330
	.WORD	18	,	9	,	30864	,	324
	.WORD	9	,	18	,	30864	,	324
	.WORD	23	,	7	,	31056	,	322
	.WORD	7	,	23	,	31056	,	322
	.WORD	20	,	8	,	31250	,	320
	.WORD	16	,	10	,	31250	,	320
	.WORD	10	,	16	,	31250	,	320
	.WORD	8	,	20	,	31250	,	320
	.WORD	13	,	12	,	32051	,	312
	.WORD	12	,	13	,	32051	,	312
	.WORD	22	,	7	,	32468	,	308
	.WORD	14	,	11	,	32468	,	308
	.WORD	11	,	14	,	32468	,	308
	.WORD	7	,	22	,	32468	,	308
	.WORD	17	,	9	,	32680	,	306
	.WORD	9	,	17	,	32680	,	306
	.WORD	19	,	8	,	32895	,	304
	.WORD	8	,	19	,	32895	,	304
	.WORD	15	,	10	,	33333	,	300
	.WORD	10	,	15	,	33333	,	300
	.WORD	21	,	7	,	34014	,	294
	.WORD	7	,	21	,	34014	,	294
	.WORD	24	,	6	,	34722	,	288
	.WORD	18	,	8	,	34722	,	288
	.WORD	16	,	9	,	34722	,	288
	.WORD	12	,	12	,	34722	,	288
	.WORD	9	,	16	,	34722	,	288
	.WORD	8	,	18	,	34722	,	288
	.WORD	6	,	24	,	34722	,	288
	.WORD	13	,	11	,	34965	,	286
	.WORD	11	,	13	,	34965	,	286
	.WORD	20	,	7	,	35714	,	280
	.WORD	14	,	10	,	35714	,	280
	.WORD	10	,	14	,	35714	,	280
	.WORD	7	,	20	,	35714	,	280
	.WORD	23	,	6	,	36232	,	276
	.WORD	6	,	23	,	36232	,	276
	.WORD	17	,	8	,	36765	,	272
	.WORD	8	,	17	,	36765	,	272
	.WORD	15	,	9	,	37037	,	270
	.WORD	9	,	15	,	37037	,	270
	.WORD	19	,	7	,	37594	,	266
	.WORD	7	,	19	,	37594	,	266
	.WORD	22	,	6	,	37879	,	264
	.WORD	12	,	11	,	37879	,	264
	.WORD	11	,	12	,	37879	,	264
	.WORD	6	,	22	,	37879	,	264
	.WORD	13	,	10	,	38462	,	260
	.WORD	10	,	13	,	38462	,	260
	.WORD	16	,	8	,	39063	,	256
	.WORD	8	,	16	,	39063	,	256
	.WORD	21	,	6	,	39683	,	252
	.WORD	18	,	7	,	39683	,	252
	.WORD	14	,	9	,	39683	,	252
	.WORD	9	,	14	,	39683	,	252
	.WORD	7	,	18	,	39683	,	252
	.WORD	6	,	21	,	39683	,	252
	.WORD	11	,	11	,	41322	,	242
	.WORD	24	,	5	,	41667	,	240
	.WORD	20	,	6	,	41667	,	240
	.WORD	15	,	8	,	41667	,	240
	.WORD	12	,	10	,	41667	,	240
	.WORD	10	,	12	,	41667	,	240
	.WORD	8	,	15	,	41667	,	240
	.WORD	6	,	20	,	41667	,	240
	.WORD	5	,	24	,	41667	,	240
	.WORD	17	,	7	,	42017	,	238
	.WORD	7	,	17	,	42017	,	238
	.WORD	13	,	9	,	42735	,	234
	.WORD	9	,	13	,	42735	,	234
	.WORD	23	,	5	,	43478	,	230
	.WORD	5	,	23	,	43478	,	230
	.WORD	19	,	6	,	43860	,	228
	.WORD	6	,	19	,	43860	,	228
	.WORD	16	,	7	,	44643	,	224
	.WORD	14	,	8	,	44643	,	224
	.WORD	8	,	14	,	44643	,	224
	.WORD	7	,	16	,	44643	,	224
	.WORD	22	,	5	,	45455	,	220
	.WORD	11	,	10	,	45455	,	220
	.WORD	10	,	11	,	45455	,	220
	.WORD	5	,	22	,	45455	,	220
	.WORD	18	,	6	,	46296	,	216
	.WORD	12	,	9	,	46296	,	216
	.WORD	9	,	12	,	46296	,	216
	.WORD	6	,	18	,	46296	,	216
	.WORD	21	,	5	,	47619	,	210
	.WORD	15	,	7	,	47619	,	210
	.WORD	7	,	15	,	47619	,	210
	.WORD	5	,	21	,	47619	,	210
	.WORD	13	,	8	,	48077	,	208
	.WORD	8	,	13	,	48077	,	208
	.WORD	17	,	6	,	49020	,	204
	.WORD	6	,	17	,	49020	,	204
	.WORD	20	,	5	,	50000	,	200
	.WORD	10	,	10	,	50000	,	200
	.WORD	5	,	20	,	50000	,	200
	.WORD	11	,	9	,	50505	,	198
	.WORD	9	,	11	,	50505	,	198
	.WORD	14	,	7	,	51020	,	196
	.WORD	7	,	14	,	51020	,	196
	.WORD	24	,	4	,	52083	,	192
	.WORD	16	,	6	,	52083	,	192
	.WORD	12	,	8	,	52083	,	192
	.WORD	8	,	12	,	52083	,	192
	.WORD	6	,	16	,	52083	,	192
	.WORD	4	,	24	,	52083	,	192
	.WORD	19	,	5	,	52632	,	190
	.WORD	5	,	19	,	52632	,	190
	.WORD	23	,	4	,	54348	,	184
	.WORD	4	,	23	,	54348	,	184
	.WORD	13	,	7	,	54945	,	182
	.WORD	7	,	13	,	54945	,	182
	.WORD	18	,	5	,	55556	,	180
	.WORD	15	,	6	,	55556	,	180
	.WORD	10	,	9	,	55556	,	180
	.WORD	9	,	10	,	55556	,	180
	.WORD	6	,	15	,	55556	,	180
	.WORD	5	,	18	,	55556	,	180
	.WORD	22	,	4	,	56818	,	176
	.WORD	11	,	8	,	56818	,	176
	.WORD	8	,	11	,	56818	,	176
	.WORD	4	,	22	,	56818	,	176
	.WORD	17	,	5	,	58824	,	170
	.WORD	5	,	17	,	58824	,	170
	.WORD	21	,	4	,	59524	,	168
	.WORD	14	,	6	,	59524	,	168
	.WORD	12	,	7	,	59524	,	168
	.WORD	7	,	12	,	59524	,	168
	.WORD	6	,	14	,	59524	,	168
	.WORD	4	,	21	,	59524	,	168
	.WORD	9	,	9	,	61728	,	162
	.WORD	20	,	4	,	62500	,	160
	.WORD	16	,	5	,	62500	,	160
	.WORD	10	,	8	,	62500	,	160
	.WORD	8	,	10	,	62500	,	160
	.WORD	5	,	16	,	62500	,	160
	.WORD	4	,	20	,	62500	,	160
	.WORD	13	,	6	,	64103	,	156
	.WORD	6	,	13	,	64103	,	156
	.WORD	11	,	7	,	64935	,	154
	.WORD	7	,	11	,	64935	,	154
	.WORD	19	,	4	,	6579	,	152
	.WORD	4	,	19	,	6579	,	152
	.WORD	15	,	5	,	6667	,	150
	.WORD	5	,	15	,	6667	,	150
	.WORD	24	,	3	,	6944	,	144
	.WORD	18	,	4	,	6944	,	144
	.WORD	12	,	6	,	6944	,	144
	.WORD	9	,	8	,	6944	,	144
	.WORD	8	,	9	,	6944	,	144
	.WORD	6	,	12	,	6944	,	144
	.WORD	4	,	18	,	6944	,	144
	.WORD	14	,	5	,	7143	,	140
	.WORD	10	,	7	,	7143	,	140
	.WORD	7	,	10	,	7143	,	140
	.WORD	5	,	14	,	7143	,	140
	.WORD	23	,	3	,	7246	,	138
	.WORD	17	,	4	,	7353	,	136
	.WORD	4	,	17	,	7353	,	136
	.WORD	22	,	3	,	7576	,	132
	.WORD	11	,	6	,	7576	,	132
	.WORD	6	,	11	,	7576	,	132
	.WORD	13	,	5	,	7692	,	130
	.WORD	5	,	13	,	7692	,	130
	.WORD	16	,	4	,	7813	,	128
	.WORD	8	,	8	,	7813	,	128
	.WORD	4	,	16	,	7813	,	128
	.WORD	21	,	3	,	7937	,	126
	.WORD	9	,	7	,	7937	,	126
	.WORD	7	,	9	,	7937	,	126
	.WORD	20	,	3	,	8333	,	120
	.WORD	15	,	4	,	8333	,	120
	.WORD	12	,	5	,	8333	,	120
	.WORD	10	,	6	,	8333	,	120
	.WORD	6	,	10	,	8333	,	120
	.WORD	5	,	12	,	8333	,	120
	.WORD	4	,	15	,	8333	,	120
	.WORD	19	,	3	,	8772	,	114
	.WORD	14	,	4	,	8929	,	112
	.WORD	8	,	7	,	8929	,	112
	.WORD	7	,	8	,	8929	,	112
	.WORD	4	,	14	,	8929	,	112
	.WORD	11	,	5	,	9091	,	110
	.WORD	5	,	11	,	9091	,	110
	.WORD	18	,	3	,	9259	,	108
	.WORD	9	,	6	,	9259	,	108
	.WORD	6	,	9	,	9259	,	108
	.WORD	13	,	4	,	9615	,	104
	.WORD	4	,	13	,	9615	,	104
	.WORD	17	,	3	,	9804	,	102
	.WORD	10	,	5	,	10000	,	100
	.WORD	5	,	10	,	10000	,	100
	.WORD	7	,	7	,	10204	,	98
	.WORD	24	,	2	,	10417	,	96
	.WORD	16	,	3	,	10417	,	96
	.WORD	12	,	4	,	10417	,	96
	.WORD	8	,	6	,	10417	,	96
	.WORD	6	,	8	,	10417	,	96
	.WORD	4	,	12	,	10417	,	96
	.WORD	23	,	2	,	10870	,	92
	.WORD	15	,	3	,	11111	,	90
	.WORD	9	,	5	,	11111	,	90
	.WORD	5	,	9	,	11111	,	90
	.WORD	22	,	2	,	11364	,	88
	.WORD	11	,	4	,	11364	,	88
	.WORD	4	,	11	,	11364	,	88
	.WORD	21	,	2	,	11905	,	84
	.WORD	14	,	3	,	11905	,	84
	.WORD	7	,	6	,	11905	,	84
	.WORD	6	,	7	,	11905	,	84
	.WORD	20	,	2	,	12500	,	80
	.WORD	10	,	4	,	12500	,	80
	.WORD	8	,	5	,	12500	,	80
	.WORD	5	,	8	,	12500	,	80
	.WORD	4	,	10	,	12500	,	80
	.WORD	13	,	3	,	12821	,	78
	.WORD	19	,	2	,	13158	,	76
	.WORD	18	,	2	,	13889	,	72
	.WORD	12	,	3	,	13889	,	72
	.WORD	9	,	4	,	13889	,	72
	.WORD	6	,	6	,	13889	,	72
	.WORD	4	,	9	,	13889	,	72
	.WORD	7	,	5	,	14286	,	70
	.WORD	5	,	7	,	14286	,	70
	.WORD	17	,	2	,	14706	,	68
	.WORD	11	,	3	,	15152	,	66
	.WORD	16	,	2	,	15625	,	64
	.WORD	8	,	4	,	15625	,	64
	.WORD	4	,	8	,	15625	,	64
	.WORD	15	,	2	,	16667	,	60
	.WORD	10	,	3	,	16667	,	60
	.WORD	6	,	5	,	16667	,	60
	.WORD	5	,	6	,	16667	,	60
	.WORD	14	,	2	,	17857	,	56
	.WORD	7	,	4	,	17857	,	56
	.WORD	4	,	7	,	17857	,	56
	.WORD	9	,	3	,	18519	,	54
	.WORD	13	,	2	,	19231	,	52
	.WORD	5	,	5	,	20000	,	50
	.WORD	12	,	2	,	20833	,	48
	.WORD	8	,	3	,	20833	,	48
	.WORD	6	,	4	,	20833	,	48
	.WORD	4	,	6	,	20833	,	48
	.WORD	11	,	2	,	22727	,	44
	.WORD	7	,	3	,	23810	,	42
	.WORD	10	,	2	,	25000	,	40
	.WORD	5	,	4	,	25000	,	40
	.WORD	4	,	5	,	25000	,	40
	.WORD	9	,	2	,	27778	,	36
	.WORD	6	,	3	,	27778	,	36
	.WORD	8	,	2	,	31250	,	32
	.WORD	4	,	4	,	31250	,	32
	.WORD	5	,	3	,	33333	,	30
	.WORD	7	,	2	,	35714	,	28
	.WORD	6	,	2	,	41667	,	24
	.WORD	4	,	3	,	41667	,	24
	.WORD	5	,	2	,	50000	,	20
	.WORD	4	,	2	,	62500	,	16

BOTH_TBL_END

          .END
