;  Title  "TESTRAMP.ASM"                       Copyright TAPR 1994
;  Berived from Bob Stricklin DSP-93 test code
;  Initial version called Stairtst for TAPR Alpha hardware by Lon Cecil
;  Modified for fast led flashing and verification of AIO output
;  Generates one right-to-left rip of LEDs, then enters and loops on
;     stairstep signal for oscilloscope verification of D/A accuracy.
;  Positive pulse added at start, to trigger scope.  Approx. 100ms period.
;
;  Set scope to 1 volt/div vertical, 10 milliseconds/div horizontal
;  Use + edge trigger, and connect probe to J203 pin 10 (near speaker out)
;  Note that each step may have tilt, due to low frequency coupling limits
;
;    |     |     |     |     |     |     |     |     |     |     |
;
;+3  ---   -----
;
;+2              -----
;                      
;+1                    -----
;                            -----
; 0 -   ---                                                      -----  
;                                   -----
;-1                                       -----
;                                               -----
;-2                                                   -----
;
;-3                                                         -----
;
;
; Voltage levels should be approximately +3, +2, +1.1, +0.5, -0.5, -1,
;   -1.8, -2.3, and -3.0.  Voltage levels are in binary steps, and may 
;   not be quite linear across zero DC, with the very low test frequency.
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
; 
; This document, and it executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
; 
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
; 
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
; 
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;
AR0      .EQU    00h
AR1      .EQU    01h
AR2      .EQU    02h
AR3      .EQU    03h
AR4      .EQU    04h
AR5      .EQU    05h
AR6      .EQU    06h
AR7      .EQU    07h

DRR      .EQU    00h        ; data receive register address.
DXR      .EQU    01h        ; data xmit register address.
TIM      .EQU    02h        ; Timer count register address.
PRD      .EQU    03h        ; TImer period register address.
IMR      .EQU    04h        ; Interrupt mask register address.
;       Reserved on chip RAM as flags and storages.
;       (block  B2 108 through 127.)
;
FLAG     .EQU     108H       ; temporary register.
;       FLAG BIT VALUES
;       0       SIO TX
;       1       SIO RX
;       2       SIO INT
;       3       TNC TX
;       4       TNC RX
;       5       TNC TX_CLK
;       6       TNC RX_CLK
;       7       AIO TX READY FOR NEW VALUE = 1
;       8       AIO RX VALUE RECEIVED = 1
;       9       TIMER  
;       10      tba
;       11      tba
;       12      LED1
;       13      LED2
;       14      LED3  
;       15      LED4
PIO      .EQU     07h       ; Latch Buffer
XPIO     .EQU     109H       ; Value for PIO port
RPIO     .EQU     110H       ; Value for PIO port
ACCHST   .EQU     111H       ; stack for ACCH.
ACCLST   .EQU     112H       ; stack for ACCL.
SSTST    .EQU     113H       ; stack for ST0 register.
INTST    .EQU     114H       ; stack for IMR register.
RVECT    .EQU     115H       ; vector for RINT.
XVECT    .EQU     116H       ; vector for XINT.
VRCV     .EQU     117H       ; RINT vector storage.
LAST     .EQU     118H       ; Last value received for data and clock
PIO_BUF  .EQU     60h       ; Next Period Value
DRCV     .EQU     120H       ; receive data storage.
DXMT     .EQU     121H       ; xmit data storage.
D2ND     .EQU     122H       ; secondary data storage.
CHECK    .EQU     123H       ; receive flag.
COUNT    .EQU     124H       ; 
F2ND     .EQU     125H       ; secondary communication flag.
CLOCK    .EQU     126H       ; xmit flag.
FXE      .EQU     127H       ; ignore first XINT flag.
FRE      .EQU     128H       ; ignore first RINT flag.
TMP0     .EQU     0H         ; General Flag
TMP1     .EQU     1H         ; General Flag
TMP2     .EQU     2H         ; General Flag
TMP3     .EQU     3H         ; General Flag
TMP4     .EQU     4H         ; General Flag
TMP5     .EQU     5H         ; General Flag
TEMP     .EQU     6H         ; General Storage
X        .EQU     7H
C        .EQU     8H
SO       .EQU     9H
NUMERA   .EQU     0Ah
DENOM    .EQU     0Bh
TEMSGN   .EQU     0Ch
QUOT     .EQU     0Dh
M        .EQU     0Eh
N        .EQU     0Fh
FL       .EQU     10h
Z        .EQU     11h
AA       .EQU     12h
BB       .EQU     13h
CC       .EQU     14h
ZB       .EQU     15h
ZZ       .EQU     16h
B        .EQU     17h
DA       .EQU     18h
S0       .EQU     19h
SL0      .EQU     1Ah
SL1      .EQU     1Bh
SL2      .EQU     1Ch
SM0      .EQU     1Dh
SM1      .EQU     1Eh
SM2      .EQU     1Fh

CONST    .EQU     16384

G0       .EQU     31
G1       .EQU     62
G2       .EQU     31
H1       .EQU     31313
H2       .EQU     -14991

GG0      .EQU     137
GG1      .EQU     274
GG2      .EQU     137
HH1      .EQU     28941
HH2      .EQU     -13106

FREQ     .EQU     120       ; Step size for sin curve... 1440/(19200/Baud rate)
			    ; 1600 Baud = 120
			    ; 1200 Baud =  90
FRQ_SFT  .EQU     720       ; Step size for sin curve... 1440/(19200/Baud rate)
CONST1   .EQU     11H        ; Step on the sin curve for artificial signal
CONST2   .EQU     22H        ; Step on the sin curve 
CONST3   .EQU     22H        ; Step on the sin curve 

; TNC IO Locations          Input/Output Relative to DSP
SIO_RTS  .EQU     2         ; Input 
SIO      .EQU     3         ; Output
CD       .EQU     5         ; Input Carrier Detect
CONN_ST  .EQU     4         ; Input Connection status
CTS      .EQU     1         ; Output Cllear To Send
X_CLK    .EQU     2         ; Output Xmit clock
R_CLK    .EQU     8         ; Input receive clock
TXD      .EQU     8         ; Output Xmit data
RXD      .EQU     10h       ; Input receive data
ROC_CLK  .EQU     20h       ; Recovered Clock from EPROMs in TNC

;  TASM does not assume the necessary 1000h starting address
	 .ORG     1000h

MODEM   
	B       INIT
	B       INIT
	B       INIT
	B       INIT
	B       MODEM

INIT
	LDPK    8h             ; point to data page
LOOP
; LAMPS LED207-LED201 OFF
	ZAC                    ; zero accumulator to assure 0000h loaded
	SACL    60h            ; direct data memory address
	OUT     60h,06h        ; write to TNC Output latch
	OUT     60h,0Bh        ; write to Radio Port Digital latch
	XORK    0C000h
	SACL    60h
	OUT     60h,07h        ; write to AIO Enable/Gain & Radio Select
		
; LED207 ON
	SACL    61h            ; direct data memory address from 61h
	LT      61h            ; load T register to 
	LACT    60h            ; load value at memory 60h
	XORK    0400h          ; set bit D10 high (LED active)
	SACL    60h
	OUT     60h,06h        ; send value to TNC latch (hi byte)
	CALL    WAIT4
	LACT    60h
	XORK    0400h          ; restore D10 low (LED dark)
	SACL    60h
	OUT     60h,06h
; LED206 ON
	ZAC
	SACL    61h
	LT      61h
	LACT    60h
		XORK    0200h
	SACL    60h
	OUT     60h,06h
		CALL    WAIT4
	LACT    60h
		XORK    0200h
	SACL    60h
	OUT     60h,06h
; LED205 ON
		ZAC
		SACL    61h
		LT      61h
	LACT    60h
		XORK    0100h
	SACL    60h
	OUT     60h,06h
		CALL    WAIT4
	LACT    60h
		XORK    0100h
	SACL    60h
	OUT     60h,06h
; LED204 ON
		ZAC
		SACL    61h
		LT      61h
	LACT    60h
		XORK    0080h
	SACL    60h
	OUT     60h,0Bh
		CALL    WAIT4
	LACT    60h
		XORK    0080h
	SACL    60h
	OUT     60h,0Bh
; LED203 ON
		ZAC
		SACL    61h
		LT      61h
	LACT    60h
		XORK    0040h
	SACL    60h
	OUT     60h,0Bh
		CALL    WAIT4
	LACT    60h
		XORK    0040h
	SACL    60h
	OUT     60h,0Bh
; LED202 ON
		ZAC
		SACL    61h
		LT      61h
	LACT    60h
		XORK    0040h
	SACL    60h
	OUT     60h,07h
		CALL    WAIT4
	LACT    60h
		XORK    0040h
	SACL    60h
	OUT     60h,07h
; LED201 ON
	ZAC
	SACL    61h
	LT      61h
	LACT    60h
	XORK    0080h
	SACL    60h
	OUT     60h,07h
	CALL    WAIT4
	LACT    60h
	XORK    0080h
	SACL    60h
	OUT     60h,07h 
LOOP2
	ZAC                  ; Data page is 08h
	SACL    60h
	LT      60h
	LACT    60h          ; BIT D15 LOW to connect AIO to CPU (DX & DR)
	XORK    0040h        ; BIT D14 LOW to activate and reset AIO
	SACL    60h          ; Save to 60h memory
	OUT     60h,07h      ; Write to port 07h with LED 202 active
	CALL    WAIT4
	CALL    WAIT4
	LACT    60h
	XORK    4040h        ; Rewrite with LED 202 dark 
	SACL    60h
	OUT     60h,07h      ;
	CALL    WAIT4        ; AIO has been activated and reset

; Generate stairstep pattern on AIO D/A output  (J203 pin 10 buffered)
LOOP1
	LDPK    0h        ; point to data page
		ZAC
		LALK    7F00h    ; a full amplitude step for scope trigger
		SACL    1h        
		CALL    WAIT8
		LALK    0000h    ; with a return to baseline 
		SACL    1h
		CALL    WAIT8
		LALK    7FF0h    ; then start a decending staircase 
		SACL    1h       ; from +3 volts to -3 volts
		CALL    WAIT8
		CALL    WAIT8
		LALK    6000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    4000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    2000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    0000h
		SACL    1h
		CALL    WAIT8    ; only one wait period
		LALK    0FFF0h   ; next step is only 1 quantize lower
		SACL    1h
		CALL    WAIT8    ; so we will split the step for display
		LALK    0E000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    0C000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    0A000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    8000h
		SACL    1h
		CALL    WAIT8
		CALL    WAIT8
		LALK    0000h
		SACL    1h
		CALL    WAIT8   
	B       LOOP1

WAIT8   LARK    AR2,02Fh
	LARK    AR3,02Fh
	B       WAIT_B
	CALL    WAIT4
	CALL    WAIT4
	RET
WAIT4   LARK    AR2,0FFh
	CALL    WAIT2
	CALL    WAIT2
	RET
WAIT2   LARK    AR2,0FFh
	CALL    WAIT1
	CALL    WAIT1
	RET
WAIT1   LARK    AR2,0FFh
WAIT_A  LARK    AR3,0FFh
WAIT_B  LARP    AR3
	BANZ    WAIT_B    ; Loop through count
	LARP    AR2
	BANZ    WAIT_A    ; Loop through count
	RET
	.end              ; end of program

