;  Experimental 9600 bps FSK full duplex modem
;
;  To assemble, use the command "asm fsk.asm -dRn -dGn"
;    where Rn is R1 or R2 for radio ports 1 or 2 and
;    where Gn is one of G0, G1, G2, G3, G4, G5, or G6 to set the gain
;
;  LED204 is PTT
;  LED205 is on
;  LED207 is duty cycle
;  LED208 is power
;
;  Version for TAPR/AMSAT DSP-93, File - TPRS.ASM
;  Derived from File - TPRS_93A.ASM
;  Updates for conditional assembly and includes by Ron Parsons W5RKN
;  TLC32044 AIO @ 41,666 samples/sec (24 sec/sample), 40 MHz CPU clock
;  AIO initialization from Lon Cecil's BST5.ASM program
;  Uses TNC disconnect header (set TNC to 9600 bps!!)
;  Updates:
;    Added TX lock out when TNC is off
;    Using RX interrupt only after AIO initialization
;    Using general purpose LPF (no ISI compensation)
;    Modified scramble and unscramble routines for TPRS operation
;    Make TX audio only if PTT on (RGP)
;    Added LED204 as PTT indicator (RGP)
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1991, 1992, 1993 
; and 1994, Frank H. Perkins, Jr., WB5IPM. All rights reserved.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact Greg Jones at TAPR or the author for information on
; commercial licenses of this software.
; 
; Frank H. Perkins, Jr.
;
#include  "REGS.INC"
#include  "PORTS.INC"
#include  "MONITOR.INC"

#ifdef    R1
REC_IN    .EQU      REC_IN_1
#endif
#ifdef    R2
REC_IN    .EQU      REC_IN_2
#endif

#ifdef    G0
GAIN      .EQU      GAIN0
#endif
#ifdef    G1
GAIN      .EQU      GAIN2
#endif
#ifdef    G2
GAIN      .EQU      GAIN2
#endif
#ifdef    G3
GAIN      .EQU      GAIN3
#endif
#ifdef    G4
GAIN      .EQU      GAIN4
#endif
#ifdef    G5
GAIN      .EQU      GAIN5
#endif
#ifdef    G6
GAIN      .EQU      GAIN6
#endif

;  global variables
BUFI      .EQU      060h      ;  AIO input buffer
BUFO      .EQU      061h      ;  AIO output buffer
DO        .EQU      062h      ;  data output buffer

SO        .EQU      063h      ;  data signal buffer
LSO       .EQU      064h      ;  last data signal buffer

CB        .EQU      065h      ;  clock buffer
LCB       .EQU      066h      ;  last clock buffer

RB        .EQU      067h      ;  ramp buffer
SA        .EQU      068h      ;  analog signal buffer

TSO       .EQU      069h      ;  TX data buffer (TNC)
LTSO      .EQU      06Ah      ;  last TX data buffer

TCB       .EQU      06Bh      ;  TX clock buffer
LTCB      .EQU      06Ch      ;  last TX clock buffer

TRB       .EQU      06Dh      ;  TX ramp buffer
DI        .EQU      06Eh      ;  data input buffer
TO        .EQU      06Fh      ;  scrambled TX data
TA        .EQU      070h      ;  TX analog data
RO        .EQU      071h      ;  radio control buffer (PTT)

TSD       .EQU      072h      ;  TX data bit
LTSD      .EQU      073h      ;  last TX data bit
UTX       .EQU      074h      ;  UN NRZI TX data bit
TXE       .EQU      075h      ;  TX enable counter buffer
SAVPTT    .EQU      076h      ;  save PTT status
SAVSCR    .EQU      077h      ;  save scrambled data

CFG       .EQU      02h       ;  AIO CFG buffer (data mem page 8)

CMDA      .EQU      CMDA_VAL(6)     ;  AIO load RA/TA = 6 for SCF @ 833.333 kHz
CMDB      .EQU      CMDB_VAL(20)    ;  AIO load RB/TB = 20 for 41,666 samples/sec
;  AIO cofig:  A/D gain 2, sync, no HPF or sin corr
CMDC      .EQU      AIO_CONFIG|AIO_GAIND71|AIO_GAIND60|AIO_SYNC 

TOP       .EQU      10000     ;  ramp top value
CSW       .EQU      5000      ;  clock 0 -> 1 switch value
INCR      .EQU      2304      ;  incr value for 9.6 kHz
EL        .EQU      230       ;  early-late dither value
TXEC      .EQU      100       ;  TX enable count

;  TASM does not assume the necessary 1000h starting address
          .ORG      1000h


          B         GO        ;  branch to program start (TINT)
          B         RINT      ;  SIO receive interrupt service routine
          B         XINT      ;  SIO transmit interrupt service routine
          B         GO        ;  branch to program start (TRAP)

GO        DINT                ;  disable interrupts
          LDPK      USER_PAGE ;  data mem page 8 (external)

MATH_INI  SSXM                ;  set sign extension mode
          SOVM                ;  saturate on overflow
          SPM       0         ;  set P shift = 0

AIO_RST   LALK      AIO_RESET|GAIN|REC_IN ;  reset D15 to disconnect AIO (DX&DR)
          SACL      CFG       ;  clear D14 to hold AIO in RESET mode
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      LED_202|GAIN|REC_IN ;  LED 202 on
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LALK      AIO_ENABLE|GAIN|REC_IN ;  load AIO run
          SACL      CFG       ;  store in AIO CFG buffer
          OUT       CFG,RADIO_GAIN   ;  OUT to config port

          CALL      WAIT4     ;  delay 400 ms

          LDPK      BASE_PAGE ;  data mem page 0 (internal)

          SFSM                ;  frame sync SIO to AIO
          RTXM                ;  make FSX a CPU input

          LALK      0020h     ;  mask SIO TX interrupt active
          SACL      IMR       ;  store INT mask

          ZAC                 ;  zero A
          SACL      BUFO      ;  store in BUFO

          EINT                ;  enable TX interrupt
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDC      ;  load AIO config command
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDB      ;  load TB/RB AIO command next
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      AIO_CONFIG ;  load AIO command flag
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          LALK      CMDA      ;  load TA/RA AIO command last!!
          SACL      BUFO      ;  store in BUFO
          IDLE                ;  wait for TX interrupt

          DINT                ;  disable interrupts
          LALK      0010h     ;  mask SIO RX interrupt active
          SACL      IMR       ;  store mask in IMR

DSP_INI   ZAC                 ;  zero A

          SACL      DO        ;  zero buffer
          SACL      SO        ;  zero buffer
          SACL      LSO       ;  zero buffer
          SACL      CB        ;  zero buffer
          SACL      LCB       ;  zero buffer
          SACL      RB        ;  zero buffer
          SACL      SA        ;  zero buffer

          SACL      TSO       ;  zero buffer
          SACL      LTSO      ;  zero buffer
          SACL      TCB       ;  zero buffer
          SACL      LTCB      ;  zero buffer
          SACL      TRB       ;  zero buffer
          SACL      DI        ;  zero buffer
          SACL      TO        ;  zero buffer
          SACL      TA        ;  zero buffer
          SACL      RO        ;  zero buffer

          SACL      TSD       ;  zero buffer
          SACL      LTSD      ;  zero buffer
          SACL      UTX       ;  zero buffer

          LALK      TXEC      ;  get TX enable count
          SACL      TXE       ;  store in TX enable buffer

          ZAC                 ;  zero A

FIR_INI   CNFD                ;  set block B0 data mem for coefs load

          LRLK      AR0,0200h ;  0200h is coeffs start
          LARP      AR0       ;  make AR0 pointer @ 0200h
          RPTK      15        ;  repeat (15 + 1) times
          BLKP      LPF,*+    ;  to xfer 16 coeffs

          CNFP                ;  set B0 to prgm mem

DEL_INI   LRLK      AR0,03D4h ;  03D4h is unscrambler X^0 (block B1)
          LARP      AR0       ;  make AR0 pointer @ 03D4h
          ZAC                 ;  zero A
          RPTK      17        ;  repeat (17 + 1) times:
          SACL      *+        ;  store 0, increment pointer

          LRLK      AR1,03E0h ;  point AR1 @ X^-12
          LRLK      AR2,03E5h ;  point AR2 @ X^-17

TDEL_INI  LRLK      AR5,03C1h ;  03C1h is scrambler X^0 (block B1)
          LARP      AR5       ;  make AR5 pointer @ 03C1h
          ZAC                 ;  zero A
          RPTK      17        ;  repeat (17 + 1) times:
          SACL      *+        ;  store 0, incr pointer

          LRLK      AR6,03CDh ;  point AR1 @ X^-12
          LRLK      AR7,03D2h ;  point AR2 @ X^-17

          EINT                ;  enable interrupts             


LOOP      NOP
          B         LOOP      ;  loop here between interrupts


RINT      LAC       DRR       ;  get received AIO data sample from DRR
          SACL      BUFI      ;  store DDR in AIO input buffer
          CALL      DSP       ;  call DSP process

          EINT
          RET


XINT      LAC       BUFO      ;  get data from AIO output buffer
          SACL      DXR       ;  store data in DXR to transmit to AIO

          EINT
          RET


DSP       LAC       DO        ;  load DO
          ORK       CDI|KEYI_207     ;  lock DCD high, LED 207 is duty cycle
          SACL      DO        ;  store DO
          OUT       DO,TNC_OUTPUT    ;  OUT to TNC

FIR       LAC       BUFI      ;  load input sample
          LRLK      AR3,03EFh ;  03EFh is start of FIR data (B1) 
          LARP      AR3       ;  make AR3 pointer @ 03E7h
          SACL      *         ;  store input sample 

          LRLK      AR4,03FEh ;  03FEh is end of FIR data
          LARP      AR4       ;  make AR4 pointer @ 03FEh  

          MPYK      0         ;  zero P!!
          ZAC                 ;  zero A!!

          RPTK      15        ;  repeat (15 + 1) times to do FIR
          MACD      0FF00h,*- ;  MACD, up thru B0 coeffs, down thru B1 data
          APAC                ;  accum last product
          SACH      SA,03h    ;  store A hi, 2^13 * 2^3 * 2^-16 norm

SLICE     DMOV      SO        ;  copy SO into LSO
          LAC       SA        ;  get SA

          BLZ       SOLO      ;  if < 0 goto SOLO

SOHI      LACK      01h       ;  else load 1
          SACL      SO        ;  store SO

          B         ELGT      ;  branch to ELGT

SOLO      ZAC                 ;  load 0
          SACL      SO        ;  store SO     

ELGT      LAC       SO        ;  get SO
          XOR       LSO       ;  SO XOR LSO

          BZ        RAMP      ;  if 0 no data edge, goto RAMP

          LAC       CB        ;  else load CB
          BZ        EARLY     ;  if CB = 0 goto EARLY

LATE      LAC       RB        ;  get RB
          ADLK      EL        ;  add EL to advance RB
          SACL      RB        ;  store RB

          B         RAMP      ;  on to gate

EARLY     LAC       RB        ;  get RB
          SBLK      EL        ;  subtract EL to retard RB
          SACL      RB        ;  store RB

RAMP      LAC       RB        ;  get last ramp value
          ADLK      INCR      ;  add incr value
          SACL      RB        ;  store back in RB
          SBLK      TOP       ;  subtract TOP value
          BLZ       CLK       ;  if < 0 goto CLK

          SACL      RB        ;  else store RB overflow to "wrap" ramp

CLK       LAC       RB        ;  get RB
          SBLK      CSW       ;  subtract CSW

          BLZ       CLO       ;  if < 0 goto CLO

CLH       DMOV      CB        ;  else copy CB into LCB
          LACK      01h       ;  load 1
          SACL      CB        ;  store CB

          B         GATE      ;  goto GATE

CLO       DMOV      CB        ;  copy CB into LCB
          ZAC                 ;  load 0
          SACL      CB        ;  store CB

GATE      LAC       CB        ;  load CB

          BZ        TX        ;  if 0 goto TX

          XOR       LCB       ;  else (CB XOR LCB) to test clock for 0 -> 1

          BZ        TX        ;  if 0 no transition, goto TX

UNSCMB    LRLK      AR0,03E4h ;  load AR0 with location of X^-16
          LARP      AR0       ;  make AR0 pointer 

          RPTK      15        ;  repeat (15 + 1) times:
          DMOV      *-        ;  bump data up, pointer down

          DMOV      *         ;  bump last input up, AR0 @ X^0
          LAC       SO        ;  load data bit
          SACL      *,0,AR1   ;  store SO @ X^0, make AR1 pointer @ X^-12
          XOR       *,AR2     ;  X^0 XOR X^-12, make AR2 pointer @ X^-17
          XOR       *         ;  (X^0 XOR X^-12) XOR X^-17

          BNZ        TX       ;  if <> 0 goto TX

NRZI      LAC       DO        ;  load DO
          XORK      4000h     ;  XOR to NRZI
          SACL      DO        ;  store DO

TX        LAC       TXE       ;  get count in TXE

          BLZ       TX_OFF    ;  if < 0 goto TX_OFF (TX lock out)

          IN        DI,TNC_INPUT    ;  get TNC input port
          LAC       DI        ;  load data
          ANDK      RTSO      ;  isolate PTT (*RTS) bit
          SACL      SAVPTT    ;  save PTT status

          BNZ       TX_OFF    ;  if <> 0 goto TX_OFF

TX_ON     LAC       RO        ;  get RO

#ifdef    R1
          ORK       PTT_R1    ;  turn PTT on for radio 1
#endif
#ifdef    R2
          ORK       PTT_R2    ;  turn PTT on for radio 2
#endif

          ORK       LED_204   ;  turn PTT LED 204 on
          SACL      RO        ;  store RO

          B         TXD

TX_OFF    LAC       RO        ;  get RO
#ifdef    R1
          ANDK      PTT_R1_OFF    ;  turn PTT off for radio 1
#endif
#ifdef    R2
          ANDK      PTT_R2_OFF    ;  turn PTT off for radio 2
#endif
          SACL      RO        ;  store RO

          ANDK      LED_204_OFF   ;  turn PTT LED 204 off
          SACL      RO        ;  store RO

TXD       DMOV      TSO       ;  copy TSO into LTSO
          IN        DI,TNC_INPUT    ;  get input TNC port
          LAC       DI        ;  load data
          ANDK      TDO       ;  isolate TRANSMIT DATA bit

          BZ        T_LO      ;  if 0 goto T_LO

T_HI      LACK      01h       ;  load 1
          SACL      TSO       ;  store in TSO
          LALK      TXEC      ;  get TXEC count
          SACL      TXE       ;  and reset TXE

          B         TELGT     ;  goto TELGT

T_LO      ZAC                 ;  load 0
          SACL      TSO       ;  store in TSO
          LAC       TXE       ;  get count in TXE

          BLZ       TELGT     ;  if < 0 goto TELGT

          SUBK      01h       ;  else decrement count in TXE
          SACL      TXE       ;  and store

TELGT     LAC       TSO       ;  get TSO
          XOR       LTSO      ;  TSO XOR LTSO

          BZ        TRAMP     ;  if 0 no data edge, goto TRAMP

          LAC       TCB       ;  else load TCB
          BZ        TEARLY    ;  if TCB = 0 goto TEARLY

TLATE     LAC       TRB       ;  get TRB
          ADLK      EL        ;  add EL to advance TRB
          SACL      TRB       ;  store TRB

          B         TRAMP     ;  on to TRAMP

TEARLY    LAC       TRB       ;  get TRB
          SBLK      EL        ;  subtract EL to retard TRB
          SACL      TRB       ;  store TRB

TRAMP     LAC       TRB       ;  get last TX ramp value
          ADLK      INCR      ;  add incr value
          SACL      TRB       ;  store back in TRB
          SBLK      TOP       ;  subtract TOP value
          BLZ       TCLK      ;  if < 0 goto TCLK

          SACL      TRB       ;  else store TRB overflow to "wrap" ramp

TCLK      LAC       TRB       ;  get TRB
          SBLK      CSW       ;  subtract CSW

          BLZ       TCLO      ;  if < 0 goto TCLO

TCLH      DMOV      TCB       ;  else copy TCB into LTCB
          LACK      01h       ;  load 1
          SACL      TCB       ;  store TCB

          B         TGATE     ;  goto TGATE

TCLO      DMOV      TCB       ;  copy TCB into LTCB
          ZAC                 ;  load 0
          SACL      TCB       ;  store TCB

TGATE     LAC       TCB       ;  load TCB

          BZ        TXOUT     ;  if 0 goto TXOUT   

          XOR       LTCB      ;  else (TCB XOR LTCB) to test clock for 0 -> 1

          BZ        TXOUT     ;  if 0 no transition, goto TXOUT

UNNRZI    DMOV      TSD       ;  (TSD) -> (LTSD)
          LAC       TSO       ;  get TSO
          SACL      TSD       ;  store in TSD
          XOR       LTSD      ;  TSD XOR LTSD
          XORK      0001h     ;  NOT (TSD XOR LTSD)
          SACL      UTX       ;  UN NRZI in UTX       

SCMB      LRLK      AR5,03D1h ;  load AR5 with position of X^-16
          LARP      AR5       ;  make AR5 pointer 

          RPTK      15        ;  repeat (15 + 1) times:
          DMOV      *-        ;  bump data up, pointer down

          DMOV      *,AR6     ;  bump last input up @ X^0, make AR6 pointer @ X^-12
          LAC       *,0,AR7   ;  load X^-12, make AR7 pointer @ X^-17
          XOR       *,AR5     ;  X^-12 XOR X^-17, make AR5 pointer @ X^0
          XOR       UTX       ;  UTX XOR (X^-12 XOR X^-17)
          SACL      *         ;  store scrambled data @ X^0
          SACL      SAVSCR    ;  save scrambled data
          
          LAC       SAVPTT    ;  get PTT status
          BNZ       ZERO      ;  send no TX audio
          
          LAC       SAVSCR    ;  get scrambled data
          BZ        LO        ;  if data 0 goto LO

HI        LALK      1024      ;  load TX analog HI value  
          SACL      TO        ;  store in TO
          B         TXOUT     ;  goto TXOUT

LO        LALK      -1024     ;  else load TX LO value
          SACL      TO        ;  and store in TO
          B         TXOUT     ;  goto TXOUT

ZERO      ZAC                 ;  else load TX zero value
          SACL      TO        ;  and store in TO

TXOUT     LAC       TO,2      ;  get value, shift 2^2
          ANDK      0FFFCh    ;  mask out AIO control bits
          SACL      DXR       ;  store in DXR for AIO D/A

END_DSP   LAC       DO        ;  load DO
          ANDK      KEYI_207_OFF    ;  reset duty cycle
          SACL      DO        ;  store DO
          OUT       DO,TNC_OUTPUT    ;  output to TNC port

          OUT       RO,RADIO_CTRL    ;  output PTT to radio

          RET                 ;  done!!


WAIT4
          CALL      WAIT2
          CALL      WAIT2

          RET


WAIT2
          CALL      WAIT1
          CALL      WAIT1

          RET


WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
          BANZ      WAIT_B    ; Loop through count

          LARP      AR2
          BANZ      WAIT_A    ; Loop through count

          RET


          .MSFIRST

LPF       .WORD        20    ;  LPF FIR coeffients * 2^13
          .WORD        -8
          .WORD       -98
          .WORD      -205
          .WORD       -69
          .WORD       568
          .WORD      1566
          .WORD      2344
          .WORD      2344
          .WORD      1566
          .WORD       568
          .WORD       -69
          .WORD      -205
          .WORD       -98
          .WORD        -8
          .WORD        20


          .END
